@@ -61,6 +61,9 @@ struct ButtonSheet {			/* A "sheet" of buttons; basically a list of rows. */
 
 static gboolean	button_set_face(GtkWidget *widget, BtnFace face);
 
+static ButtonSheet *	btn_buttonsheet_new_default_commands(MainInfo *min);
+static ButtonSheet *	btn_buttonsheet_new_default_shortcuts(MainInfo *min);
+
 /* ----------------------------------------------------------------------------------------- */
 
 /* 1998-09-15 -	Set the given <bti> to the default (built-in) buttons. */
@@ -69,8 +72,10 @@ void btn_buttoninfo_new_default(MainInfo *min, ButtonInfo *bti)
 	ButtonSheet	*bsh;
 
 	bti->sheets = NULL;
-	if((bsh = btn_buttonsheet_new_default(min)) != NULL)
-		bti->sheets = g_list_append(bti->sheets, bsh);
+	bsh = btn_buttonsheet_new_default_commands(min);
+	bti->sheets = g_list_append(bti->sheets, bsh);
+	bsh = btn_buttonsheet_new_default_shortcuts(min);
+	bti->sheets = g_list_append(bti->sheets, bsh);
 }
 
 /* 1998-09-15 -	Copy <src> into <dst>, which really should be empty (or uninitialized). */
@@ -83,7 +88,7 @@ void btn_buttoninfo_copy(ButtonInfo *dst, ButtonInfo *src)
 
 	for(iter = src->sheets; iter != NULL; iter = g_list_next(iter))
 	{
-		if((bsh = btn_buttonsheet_copy((ButtonSheet *) iter->data)) != NULL)
+		if((bsh = btn_buttonsheet_copy(iter->data)) != NULL)
 			dst->sheets = g_list_append(dst->sheets, bsh);
 	}
 }
@@ -91,19 +96,30 @@ void btn_buttoninfo_copy(ButtonInfo *dst, ButtonInfo *src)
 /* 1998-12-25 -	Add a buttonsheet to given buttoninfo. Really simple stuff. */
 void btn_buttoninfo_add_sheet(ButtonInfo *bti, ButtonSheet *bsh)
 {
+	GList	*iter, *next;
+
 	if(bsh == NULL || bsh == NULL)
 		return;
-
+	for(iter = bti->sheets; iter != NULL; iter = next)
+	{
+		next = g_list_next(iter);
+		if(strcmp(((ButtonSheet *) iter->data)->label, bsh->label) == 0)
+		{
+			bti->sheets = g_list_remove_link(bti->sheets, iter);
+			btn_buttonsheet_destroy(iter->data);
+			g_list_free_1(iter);
+		}
+	}
 	bti->sheets = g_list_append(bti->sheets, bsh);
 }
 
 /* 1998-09-15 -	Destroy all button sheets described by <bti>. */
-void btn_buttoninfo_destroy(ButtonInfo *bti)
+void btn_buttoninfo_clear(ButtonInfo *bti)
 {
 	GList	*iter;
 
 	for(iter = bti->sheets; iter != NULL; iter = g_list_next(iter))
-		btn_buttonsheet_destroy((ButtonSheet *) iter->data);
+		btn_buttonsheet_destroy(iter->data);
 	g_list_free(bti->sheets);
 	bti->sheets = NULL;
 }
@@ -165,8 +181,8 @@ static void save_button(Button *b, guint pos, FILE *out)
 /* 1998-09-15 -	Save a row of buttons. */
 static void save_row(gpointer r, gpointer f)
 {
-	FILE		*out = (FILE *) f;
-	ButtonRow	*brw = (ButtonRow *) r;
+	FILE		*out = f;
+	ButtonRow	*brw = r;
 	guint		i;
 
 	xml_put_node_open(out, "ButtonRow");
@@ -181,8 +197,8 @@ static void save_row(gpointer r, gpointer f)
 /* 1998-09-15 -	Save a sheet of buttons. */
 static void save_sheet(gpointer s, gpointer f)
 {
-	FILE		*out = (FILE *) f;
-	ButtonSheet	*bsh = (ButtonSheet *) s;
+	FILE		*out = f;
+	ButtonSheet	*bsh = s;
 
 	xml_put_node_open(out, "ButtonSheet");
 	xml_put_text(out, "label", bsh->label);
@@ -193,7 +209,7 @@ static void save_sheet(gpointer s, gpointer f)
 }
 
 /* 1999-05-01 -	Save all currently defined buttons. */
-void btn_buttoninfo_save(MainInfo *min, ButtonInfo *bti, FILE *out, const gchar *tag)
+void btn_buttoninfo_save(MainInfo *min, const ButtonInfo *bti, FILE *out, const gchar *tag)
 {
 	xml_put_node_open(out, tag);
 	g_list_foreach(bti->sheets, save_sheet, out);
@@ -231,7 +247,7 @@ static void load_button_faces(const XmlNode *node, gpointer button)
 */
 static void load_button(const XmlNode *node, gpointer r)
 {
-	ButtonRow	*brw = (ButtonRow *) r;
+	ButtonRow	*brw = r;
 	Button		*btn;
 	const gchar	*str;
 	gint		pos;
@@ -276,7 +292,7 @@ static void load_button(const XmlNode *node, gpointer r)
 
 static void load_row(const XmlNode *node, gpointer s)
 {
-	ButtonSheet	*bsh = (ButtonSheet *) s;
+	ButtonSheet	*bsh = s;
 	ButtonRow	*brw;
 	const XmlNode	*data;
 	gint		width;
@@ -288,7 +304,7 @@ static void load_row(const XmlNode *node, gpointer s)
 			if((brw = btn_buttonrow_new(width)) != NULL)
 			{
 				if((data = xml_tree_search(node, "ButtonRowButtons")) != NULL)
-					xml_node_visit_children(data, load_button, (gpointer) brw);
+					xml_node_visit_children(data, load_button, brw);
 				btn_buttonsheet_append_row(bsh, brw);
 			}
 		}
@@ -297,9 +313,8 @@ static void load_row(const XmlNode *node, gpointer s)
 	}
 }
 
-static void load_sheet(const XmlNode *node, gpointer m)
+static void load_sheet(const XmlNode *node, gpointer user)
 {
-	MainInfo	*min = (MainInfo *) m;
 	ButtonSheet	*bsh;
 	const XmlNode	*data;
 
@@ -307,15 +322,14 @@ static void load_sheet(const XmlNode *node, gpointer m)
 	{
 		xml_get_text_copy(node, "label", bsh->label, sizeof bsh->label);
 		if((data = xml_tree_search(node, "ButtonSheetRows")) != NULL)
-			xml_node_visit_children(data, load_row, (gpointer) bsh);
-		btn_buttoninfo_add_sheet(&min->cfg.buttons, bsh);
+			xml_node_visit_children(data, load_row, bsh);
+		btn_buttoninfo_add_sheet(user, bsh);
 	}
 }
 
 void btn_buttoninfo_load(MainInfo *min, ButtonInfo *bti, const XmlNode *node)
 {
-	btn_buttoninfo_destroy(bti);
-	xml_node_visit_children(node, load_sheet, min);
+	xml_node_visit_children(node, load_sheet, bti);
 }
 
 /* ----------------------------------------------------------------------------------------- */
@@ -843,9 +857,7 @@ static gint evt_brow_destroy(GtkWidget *wid, gpointer user)
 	return TRUE;
 }
 
-/* 1998-09-15 -	Build GUI representation of given row. Note: the returned widget has not
-**		been gtk_widget_show():n.
-*/
+/* 1998-09-15 -	Build GUI representation of given row. */
 static GtkWidget * buttonrow_build(MainInfo *min, ButtonRow *brw, gboolean partial, GtkSignalFunc func, gpointer user)
 {
 	GtkWidget	*hbox, *hbox2 = NULL, *btn;
@@ -867,9 +879,8 @@ static GtkWidget * buttonrow_build(MainInfo *min, ButtonRow *brw, gboolean parti
 				{
 					if(!partial)
 						keep_tt |= button_tooltip(min, tt, brw->button + i, btn);
-					gtk_object_set_data(GTK_OBJECT(btn), "row", (gpointer) brw);
+					gtk_object_set_data(GTK_OBJECT(btn), "row", brw);
 					gtk_box_pack_start(GTK_BOX(hbox2), btn, FALSE, FALSE, 0);
-					gtk_widget_show(btn);
 				}
 			}
 			gtk_box_pack_start(GTK_BOX(hbox), hbox2, FALSE, TRUE, 0);
@@ -883,17 +894,15 @@ static GtkWidget * buttonrow_build(MainInfo *min, ButtonRow *brw, gboolean parti
 				{
 					if(!partial)
 						keep_tt |= button_tooltip(min, tt, brw->button + i, btn);
-					gtk_object_set_data(GTK_OBJECT(btn), "row", (gpointer) brw);
+					gtk_object_set_data(GTK_OBJECT(btn), "row", brw);
 					gtk_box_pack_start(GTK_BOX(hbox2), btn, TRUE, TRUE, 0);
-					gtk_widget_show(btn);
 				}
 			}
 			gtk_box_pack_start(GTK_BOX(hbox), hbox2, TRUE, TRUE, 0);
 		}
-		gtk_widget_show(hbox2);
 	}
 	if(keep_tt)
-		gtk_signal_connect(GTK_OBJECT(hbox), "destroy", GTK_SIGNAL_FUNC(evt_brow_destroy), (gpointer) tt);
+		gtk_signal_connect(GTK_OBJECT(hbox), "destroy", GTK_SIGNAL_FUNC(evt_brow_destroy), tt);
 	else if(tt != NULL)
 		gtk_object_destroy(GTK_OBJECT(tt));
 
@@ -1041,21 +1050,43 @@ ButtonSheet * btn_buttonsheet_new(const gchar *label)
 /* 1998-09-15 -	Build the default button sheet, and return it.
 ** 1999-01-05 -	Fixed a *huge* bug, where the same row was added twice to the list!!
 */
-ButtonSheet * btn_buttonsheet_new_default(MainInfo *min)
+static ButtonSheet * btn_buttonsheet_new_default_commands(MainInfo *min)
 {
 	ButtonSheet	*bsh;
 	ButtonRow	*brw;
 
-	if((bsh = g_malloc(sizeof *bsh)) != NULL)
+	bsh = g_malloc(sizeof *bsh);
+	stu_strncpy(bsh->label, BTN_DEFAULT_SHEET, sizeof bsh->label);
+	bsh->rows = NULL;
+	if((brw = btn_buttonrow_new_default(min)) != NULL)
+		btn_buttonsheet_append_row(bsh, brw);
+	if((brw = btn_buttonrow_new(8)) != NULL)
+		btn_buttonsheet_append_row(bsh, brw);
+	if((brw = btn_buttonrow_new(8)) != NULL)
+		btn_buttonsheet_append_row(bsh, brw);
+	return bsh;
+}
+
+/* 2002-05-23 -	Build default shortcut buttonsheet. It's happening, slowly. */
+static ButtonSheet * btn_buttonsheet_new_default_shortcuts(MainInfo *min)
+{
+	ButtonSheet	*bsh;
+	ButtonRow	*brw;
+	const gchar	*lab[] = { "Home", "Local", "/", "CD-ROM" },
+			*pth[] = { "$HOME", "/usr/local/", "/", "/cdrom/" };
+	gchar		buf[1024];
+	guint		i;
+
+	bsh = g_malloc(sizeof *bsh);
+	stu_strncpy(bsh->label, "Shortcuts", sizeof bsh->label);
+	bsh->rows = NULL;
+	for(i = 0; i < sizeof lab / sizeof *lab; i++)
 	{
-		stu_strncpy(bsh->label, BTN_DEFAULT_SHEET, sizeof bsh->label);
-		bsh->rows = NULL;
-		if((brw = btn_buttonrow_new_default(min)) != NULL)
-			btn_buttonsheet_append_row(bsh, brw);
-		if((brw = btn_buttonrow_new(8)) != NULL)
-			btn_buttonsheet_append_row(bsh, brw);
-		if((brw = btn_buttonrow_new(8)) != NULL)
-			btn_buttonsheet_append_row(bsh, brw);
+		brw = btn_buttonrow_new(1);
+		btn_button_set_label(brw->button, BTN_PRIMARY, lab[i]);
+		g_snprintf(buf, sizeof buf, "DirEnter 'dir=%s'", pth[i]);
+		btn_button_set_cmdseq(brw->button, BTN_PRIMARY, buf);
+		btn_buttonsheet_append_row(bsh, brw);
 	}
 	return bsh;
 }
@@ -1074,7 +1105,7 @@ ButtonSheet * btn_buttonsheet_copy(ButtonSheet *src)
 	{
 		for(iter = src->rows; iter != NULL; iter = g_list_next(iter))
 		{
-			if((brw = btn_buttonrow_copy((ButtonRow *) iter->data)) != NULL)
+			if((brw = btn_buttonrow_copy(iter->data)) != NULL)
 				btn_buttonsheet_append_row(bsh, brw);
 		}
 	}
@@ -1118,7 +1149,7 @@ void btn_buttonsheet_insert_row(ButtonSheet *bsh, ButtonRow *anchor, gint width)
 */
 void btn_buttonsheet_delete_row(ButtonSheet *bsh, ButtonRow *brw)
 {
-	bsh->rows = g_list_remove(bsh->rows, (gpointer) brw);
+	bsh->rows = g_list_remove(bsh->rows, brw);
 	btn_buttonrow_destroy(brw);
 	bsh->height = g_list_length(bsh->rows);
 }
@@ -1131,7 +1162,7 @@ void btn_buttonsheet_move_row(ButtonSheet *bsh, ButtonRow *brw, gint delta)
 	GList	*link;
 	gint	pos, np;
 
-	link = g_list_find(bsh->rows, (gpointer) brw);
+	link = g_list_find(bsh->rows, brw);
 	pos = g_list_position(bsh->rows, link);
 	bsh->rows = g_list_remove(bsh->rows, brw);
 	np = pos + delta;
@@ -1160,7 +1191,7 @@ ButtonSheet * btn_buttonsheet_get(ButtonInfo *bti, const gchar *label)
 	for(iter = bti->sheets; iter != NULL; iter = g_list_next(iter))
 	{
 		if(strcmp(((ButtonSheet *) iter->data)->label, label) == 0)
-			return (ButtonSheet *) iter->data;
+			return iter->data;
 	}
 	return NULL;
 }
@@ -1174,14 +1205,27 @@ static GtkWidget * buttonsheet_build(MainInfo *min, ButtonSheet *bsh, gboolean p
 	GtkWidget	*scbox, *vbox, *row;
 	GList		*iter;
 
+	if(bsh->rows == NULL)
+		return NULL;
+
 	scbox = od_scrolled_box_new();
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scbox), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-	od_scrolled_box_set_num(OD_SCROLLED_BOX(scbox), bsh->height);
+
+	/* FIXME: This isn't exactly pure. Make the Shortcuts sheet the same visible height as the buttons. */
+	if(!partial && strcmp(bsh->label, "Shortcuts") == 0)
+	{
+		ButtonSheet	*def;
+
+		if((def = btn_buttonsheet_get(&min->cfg.buttons, BTN_DEFAULT_SHEET)) != NULL)
+			od_scrolled_box_set_num(OD_SCROLLED_BOX(scbox), def->height);
+	}
+	else
+		od_scrolled_box_set_num(OD_SCROLLED_BOX(scbox), bsh->height);
 
 	vbox = gtk_vbox_new(FALSE, 0);
 	for(iter = bsh->rows; iter != NULL; iter = g_list_next(iter))
 	{
-		if((row = buttonrow_build(min, (ButtonRow *) iter->data, partial, func, user)) != NULL)
+		if((row = buttonrow_build(min, iter->data, partial, func, user)) != NULL)
 			gtk_box_pack_start(GTK_BOX(vbox), row, FALSE, FALSE, 0);
 	}
 	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scbox), vbox);
@@ -1191,10 +1235,10 @@ static GtkWidget * buttonsheet_build(MainInfo *min, ButtonSheet *bsh, gboolean p
 }
 
 /* 1998-09-15 -	Build the GTK+ GUI representation of sheet named <label>.
-** 1998-12-23 -	Added the <partial> argument so it's there when GTK+ 1.2.0 is released (if I live to
-**		see it). It's going to rule.
+** 1998-12-23 -	Added the <partial> argument so it's there when GTK+ 1.2.0 is released
+**		(if I live to see it). It's going to rule.
 */
-GtkWidget * btn_buttonsheet_build(MainInfo *min, ButtonInfo *bti, const char *label, gboolean partial, GtkSignalFunc func, gpointer user)
+GtkWidget * btn_buttonsheet_build(MainInfo *min, ButtonInfo *bti, const gchar *label, gboolean partial, GtkSignalFunc func, gpointer user)
 {
 	GtkWidget	*sheet;
 	GList		*iter;
@@ -1209,8 +1253,9 @@ GtkWidget * btn_buttonsheet_build(MainInfo *min, ButtonInfo *bti, const char *la
 	{
 		if(strcmp(((ButtonSheet *) iter->data)->label, label) == 0)
 		{
-			if((sheet = buttonsheet_build(min, (ButtonSheet *) iter->data, partial, func, user)) != NULL)
+			if((sheet = buttonsheet_build(min, iter->data, partial, func, user)) != NULL)
 				return sheet;
+			break;
 		}
 	}
 	return NULL;
@@ -1276,7 +1321,7 @@ void btn_buttonsheet_destroy(ButtonSheet *bsh)
 	GList	*iter;
 
 	for(iter = bsh->rows; iter != NULL; iter = g_list_next(iter))
-		btn_buttonrow_destroy((ButtonRow *) iter->data);
+		btn_buttonrow_destroy(iter->data);
 	g_list_free(bsh->rows);
 	g_free(bsh);
 }
