@@ -22,11 +22,10 @@
 #include "cmdseq_dialog.h"
 #include "color_dialog.h"
 
-#include "cfg_gui.h"
+#include "configure.h"
 #include "cfg_module.h"
 #include "cfg_cmdseq.h"
 
-#include "cfg_menus.h"
 #include "cfg_buttons.h"
 
 /* ----------------------------------------------------------------------------------------- */
@@ -56,8 +55,6 @@ struct P_Buttons {
 	GtkWidget	*dvbox;			/* All definitions. */
 	GtkWidget	*dhbox;			/* Actual function definitions (primary/secondary frames). */
 	BDef		def[2];			/* Primary & secondary face definitions. */
-	GtkWidget	*dmenu;			/* Entry for menu name. */
-	guint		dmenusig;		/* Signal for "changed" event. */
 	GtkWidget	*dtooltip;		/* String widget for tooltip. */
 	GtkWidget	*dnarrow;		/* A check button for the narrow-flag. */
 	guint		dnarsig;		/* Signal identifer for the narrow flag. */
@@ -68,6 +65,7 @@ struct P_Buttons {
 	GtkWidget	*rcmd[5];		/* Row command buttons. */
 
 	MainInfo	*min;
+	const gchar	*cur_sheet;		/* Label of sheet we're editing. */
 	gboolean	modified;
 	ButtonInfo	buttons;		/* Editing copy of the button info. */
 	ButtonSheet	*sheet;			/* Points at the sheet being shown (currently always "Default"). */
@@ -122,7 +120,6 @@ static void reset_widgets(P_Buttons *page)
 		set_blocked_entry(page->def[i].cmdseq, "", page->def[i].cmdseq_sig);
 		gtk_entry_set_text(GTK_ENTRY(page->def[i].key), "");
 	}
-	set_blocked_entry(page->dmenu, "", page->dmenusig);
 	gtk_entry_set_text(GTK_ENTRY(page->dtooltip), "");
 	gtk_signal_handler_block(GTK_OBJECT(page->dnarrow), page->dnarsig);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(page->dnarrow), FALSE);
@@ -152,7 +149,6 @@ static void set_widgets(P_Buttons *page)
 			set_blocked_entry(page->def[i].cmdseq, btn_button_get_cmdseq(but, i), page->def[i].cmdseq_sig);
 			set_blocked_entry(page->def[i].key, btn_button_get_key(but, i), 0);
 		}
-		set_blocked_entry(page->dmenu, btn_button_get_menu(but), page->dmenusig);
 		if((str = btn_button_get_tooltip(but)) != NULL)
 			gtk_entry_set_text(GTK_ENTRY(page->dtooltip), str);
 		else
@@ -178,7 +174,7 @@ static void populate_scwin(P_Buttons *page)
 		gtk_widget_destroy(page->shvbox);
 	page->shvbox = NULL;
 
-	if((page->shvbox = btn_buttonsheet_build(page->min, &page->buttons, NULL, TRUE, GTK_SIGNAL_FUNC(evt_button_clicked), page)) != NULL)
+	if((page->shvbox = btn_buttonsheet_build(page->min, &page->buttons, page->cur_sheet, TRUE, GTK_SIGNAL_FUNC(evt_button_clicked), page)) != NULL)
 	{
 		gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(page->scwin), page->shvbox);
 		gtk_widget_show(page->shvbox);
@@ -259,7 +255,7 @@ static gint evt_button_clicked(GtkWidget *wid, gpointer wuser, guint index, gpoi
 */
 static gint evt_bcmd_clicked(GtkWidget *wid, gpointer user)
 {
-	P_Buttons	*page = (P_Buttons *) user;
+	P_Buttons	*page = user;
 
 	switch(GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(wid))))
 	{
@@ -287,7 +283,7 @@ static gint evt_bcmd_clicked(GtkWidget *wid, gpointer user)
 /* 1998-09-15 -	Pop up a dialog asking (with <text>) for a width. Defaults to <width>, and
 **		calls <func> if the user clicked OK.
 */
-static void width_dialog(char *text, gint width, void (*func)(gpointer userdata, guint width), gpointer userdata)
+static void width_dialog(const gchar *text, gint width, void (*func)(gpointer userdata, guint width), gpointer userdata)
 {
 	Dialog		*dlg;
 	GtkWidget	*vbox, *label, *spin;
@@ -296,7 +292,6 @@ static void width_dialog(char *text, gint width, void (*func)(gpointer userdata,
 	vbox = gtk_vbox_new(FALSE, 0);
 	label = gtk_label_new(text);
 	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
-	gtk_widget_show(label);
 
 	spin = gtk_spin_button_new(NULL, 1, 0);
 	if((adj = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(spin))) != NULL)
@@ -309,8 +304,7 @@ static void width_dialog(char *text, gint width, void (*func)(gpointer userdata,
 	}
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin), width);
 	gtk_box_pack_start(GTK_BOX(vbox), spin, FALSE, FALSE, 0);
-	gtk_widget_show(spin);
-	dlg = dlg_dialog_sync_new(vbox, _("Set Row Width"), _("OK|Cancel"));
+	dlg = dlg_dialog_sync_new(vbox, _("Set Row Width"), NULL);
 	if(dlg_dialog_sync_wait(dlg) == DLG_POSITIVE)
 		func(userdata, gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin)));
 	dlg_dialog_sync_destroy(dlg);
@@ -321,7 +315,7 @@ static void width_dialog(char *text, gint width, void (*func)(gpointer userdata,
 /* 1998-09-15 -	Add a row of <width> buttons. This is a width_dialog() callback function. */
 static void add_row(gpointer user, guint width)
 {
-	P_Buttons	*page = (P_Buttons *) user;
+	P_Buttons	*page = user;
 	ButtonRow	*row = NULL;
 
 	page->modified = TRUE;
@@ -346,7 +340,7 @@ static void add_row(gpointer user, guint width)
 /* 1998-09-15 -	Change the width of a row of buttons. A width_dialog() callback. */
 static void resize_row(gpointer user, guint width)
 {
-	P_Buttons	*page = (P_Buttons *) user;
+	P_Buttons	*page = user;
 	ButtonRow	*row;
 
 	if((row = gtk_object_get_data(GTK_OBJECT(page->button), "row")) != NULL)
@@ -364,7 +358,7 @@ static void resize_row(gpointer user, guint width)
 /* ----------------------------------------------------------------------------------------- */
 
 /* 1998-09-19 -	Move the row of the currently selected button <delta> rows (-1=>up, 1=>down). */
-static void move_row(P_Buttons *page, int delta)
+static void move_row(P_Buttons *page, gint delta)
 {
 	ButtonRow	*row;
 
@@ -387,7 +381,7 @@ static void move_row(P_Buttons *page, int delta)
 /* 1999-01-13 -	This gets called as the user types around in the tooltip string. Inefficient. */
 static gint evt_dtooltip_changed(GtkWidget *wid, gpointer user)
 {
-	P_Buttons	*page = (P_Buttons *) user;
+	P_Buttons	*page = user;
 	Button		*but;
 	const gchar	*text;
 
@@ -408,7 +402,7 @@ static gint evt_dtooltip_changed(GtkWidget *wid, gpointer user)
 /* 1998-09-28 -	The narrow-button was clicked, so grab & store its new value. */
 static gint evt_dnarrow_clicked(GtkWidget *wid, gpointer user)
 {
-	P_Buttons	*page = (P_Buttons *) user;
+	P_Buttons	*page = user;
 	Button		*but;
 
 	if((page != NULL) && ((but = btn_button_get(page->button)) != NULL))
@@ -426,7 +420,7 @@ static gint evt_dnarrow_clicked(GtkWidget *wid, gpointer user)
 /* 1999-01-13 -	User clicked the "Show Tooltip?" button, so grab current state. */
 static gint evt_dshowtip_clicked(GtkWidget *wid, gpointer user)
 {
-	P_Buttons	*page = (P_Buttons *) user;
+	P_Buttons	*page = user;
 	Button		*but;
 
 	if((page != NULL) && (page->button != NULL) && ((but = btn_button_get(page->button)) != NULL))
@@ -434,7 +428,6 @@ static gint evt_dshowtip_clicked(GtkWidget *wid, gpointer user)
 		btn_button_set_flags_boolean(but, BTF_SHOW_TOOLTIP, gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wid)));
 		page->modified = TRUE;
 	}
-
 	return TRUE;
 }
 
@@ -442,19 +435,19 @@ static gint evt_dshowtip_clicked(GtkWidget *wid, gpointer user)
 
 static void delete_row(P_Buttons *page)
 {
-	if(dlg_dialog_sync_new_simple_wait(_("Really delete current\nbutton row?"), _("Sure?"), _("Delete|Cancel")) == DLG_POSITIVE)
+	if(dlg_dialog_sync_new_simple_wait(_("Really delete current button row?"), _("Please Confirm"), _("_Delete|_Cancel")) == DLG_POSITIVE)
 	{
-		page->modified = TRUE;
 		btn_buttonsheet_delete_row(page->sheet, (ButtonRow *) gtk_object_get_data(GTK_OBJECT(page->button), "row"));
 		set_mode_normal(page);
 		populate_scwin(page);
+		page->modified = TRUE;
 	}
 }
 
 /* 1998-09-15 -	Row command button clicked, do something serious. */
 static gint evt_rcmd_clicked(GtkWidget *wid, gpointer user)
 {
-	P_Buttons	*page = (P_Buttons *) user;
+	P_Buttons	*page = user;
 	ButtonRow	*brw = NULL;
 	gint		width = 8;
 
@@ -463,14 +456,13 @@ static gint evt_rcmd_clicked(GtkWidget *wid, gpointer user)
 		brw = gtk_object_get_data(GTK_OBJECT(page->button), "row");
 		width = btn_buttonrow_get_width(brw);
 	}
-
 	switch(GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(wid))))
 	{
 		case 0:			/* Add? */
-			width_dialog(_("Select Width for New Row"), width, add_row, (gpointer) page);
+			width_dialog(_("Select Width for New Row"), width, add_row, page);
 			break;
 		case 1:			/* Change width? */
-			width_dialog(_("Change Width of Row"), width, resize_row, (gpointer) page);
+			width_dialog(_("Change Width of Row"), width, resize_row, page);
 			break;
 		case 2:			/* Move up? */
 			move_row(page, -1);
@@ -669,26 +661,6 @@ static void evt_col_fg_clicked(GtkWidget *wid, gpointer wuser, guint function, g
 	}
 }
 
-static void evt_menu_changed(GtkWidget *wid, gpointer user)
-{
-	P_Buttons	*page = user;
-
-	if((page != NULL) && (page->button != NULL))
-	{
-		btn_button_set_menu_widget(page->button, gtk_entry_get_text(GTK_ENTRY(wid)));
-		page->modified = TRUE;
-	}
-}
-
-static void evt_menu_pick_clicked(GtkWidget *wid, gpointer user)
-{
-	P_Buttons	*page = user;
-	const gchar	*menu;
-
-	if((menu = mnu_menuinfo_select(cmu_get_menuinfo())) != NULL)
-		gtk_entry_set_text(GTK_ENTRY(page->dmenu), menu);
-}
-
 /* 1999-04-24 -	Initialize widgetry for a button definition box. Thanks to the new wonderful
 **		widget (contributed by J. Hanson <johan@tiq.com>) we now have two actions per
 **		button. Very Opus-ish, and very good.
@@ -702,114 +674,93 @@ static void init_bdef(MainInfo *min, P_Buttons *page, BDef *def, const gchar *la
 
 	label = gtk_label_new(_("Label"));
 	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,  0,0,0,0);
-	gtk_widget_show(label);
 	def->label = gtk_entry_new_with_max_length(BTN_LABEL_SIZE - 1);
 	gtk_object_set_data(GTK_OBJECT(def->label), "face", GUINT_TO_POINTER(face));
 	def->label_sig = gtk_signal_connect(GTK_OBJECT(def->label), "changed", GTK_SIGNAL_FUNC(evt_label_changed), page);
 	gtk_table_attach(GTK_TABLE(table), def->label, 1, 3, 0, 1,  GTK_EXPAND|GTK_FILL,0,0,0);
-	gtk_widget_show(def->label);
 
 	label = gtk_label_new(_("Command"));
 	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,  0,0,0,0);
-	gtk_widget_show(label);
 	def->cmdseq = gtk_entry_new();
 	gtk_object_set_data(GTK_OBJECT(def->cmdseq), "face", GUINT_TO_POINTER(face));
 	def->cmdseq_sig = gtk_signal_connect(GTK_OBJECT(def->cmdseq), "changed", GTK_SIGNAL_FUNC(evt_cmdseq_changed), page);
 	gtk_table_attach(GTK_TABLE(table), def->cmdseq, 1, 2, 1, 2,  GTK_EXPAND|GTK_FILL,0,0,0);
-	gtk_widget_show(def->cmdseq);
 	btn = gui_details_button_new(min->gui->window->window);
 	gtk_object_set_data(GTK_OBJECT(btn), "face", GUINT_TO_POINTER(face));
 	gtk_signal_connect(GTK_OBJECT(btn), "clicked", GTK_SIGNAL_FUNC(evt_cmdseq_pick_clicked), page);
 	gtk_table_attach(GTK_TABLE(table), btn, 2, 3, 1, 2,  GTK_FILL,0,0,0);
-	gtk_widget_show(btn);
 
 	label = gtk_label_new(_("Key"));
 	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3,  0,0,0,0);
-	gtk_widget_show(label);
 	def->key = gtk_entry_new();
 	gtk_entry_set_editable(GTK_ENTRY(def->key), FALSE);
 	gtk_object_set_data(GTK_OBJECT(def->key), "face", GUINT_TO_POINTER(face));
 	gtk_signal_connect(GTK_OBJECT(def->key), "key_press_event", GTK_SIGNAL_FUNC(evt_key_press), page);
 	gtk_table_attach(GTK_TABLE(table), def->key, 1, 2, 2, 3,  GTK_EXPAND|GTK_FILL,0,0,0);
-	gtk_widget_show(def->key);
 	btn = gtk_button_new_with_label(_("Clr"));
 	gtk_object_set_data(GTK_OBJECT(btn), "face", GUINT_TO_POINTER(face));
 	gtk_signal_connect(GTK_OBJECT(btn), "clicked", GTK_SIGNAL_FUNC(evt_key_clear_clicked), page);
 	gtk_table_attach(GTK_TABLE(table), btn, 2, 3, 2, 3,  GTK_FILL,0,0,0);
-	gtk_widget_show(btn);
 
 	label = gtk_label_new(_("Colors"));
 	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 3, 4,  0,0,0,0);
-	gtk_widget_show(label);
 	hbox = gtk_hbox_new(FALSE, 0);
 	def->col_bg = od_emil_button_new();
 	od_emil_button_set_label(OD_EMIL_BUTTON(def->col_bg), OD_EMIL_BUTTON_PRIMARY,   _("Background..."), NULL, NULL, GUINT_TO_POINTER(face));
 	od_emil_button_set_label(OD_EMIL_BUTTON(def->col_bg), OD_EMIL_BUTTON_SECONDARY, _("Reset to Default"),  NULL, NULL, GUINT_TO_POINTER(face));
 	gtk_signal_connect(GTK_OBJECT(def->col_bg), "activate", GTK_SIGNAL_FUNC(evt_col_bg_clicked), page);
 	gtk_box_pack_start(GTK_BOX(hbox), def->col_bg, TRUE, TRUE, 0);
-	gtk_widget_show(def->col_bg);
 	def->col_fg = od_emil_button_new();
 	od_emil_button_set_label(OD_EMIL_BUTTON(def->col_fg), OD_EMIL_BUTTON_PRIMARY,   _("Foreground..."), NULL, NULL, GUINT_TO_POINTER(face));
 	od_emil_button_set_label(OD_EMIL_BUTTON(def->col_fg), OD_EMIL_BUTTON_SECONDARY, _("Reset to Default"),  NULL, NULL, GUINT_TO_POINTER(face));
 	gtk_signal_connect(GTK_OBJECT(def->col_fg), "activate", GTK_SIGNAL_FUNC(evt_col_fg_clicked), page);
 	gtk_box_pack_start(GTK_BOX(hbox), def->col_fg, TRUE, TRUE, 0);
-	gtk_widget_show(def->col_fg);
 	gtk_table_attach(GTK_TABLE(table), hbox, 1, 3, 3, 4,  GTK_EXPAND|GTK_FILL,0,0,0);
-	gtk_widget_show(hbox);
 	
 	gtk_container_add(GTK_CONTAINER(def->frame), table);
-	gtk_widget_show(table);
 }
 
-static GtkWidget * init_mdef(MainInfo *min, P_Buttons *page)
+static void evt_sheet_activate(GtkWidget *wid, gpointer user)
 {
-	GtkWidget	*frame;
-	GtkWidget	*table, *label, *btn;
+	P_Buttons	*page = user;
+	guint		index;
 
-	frame = gtk_frame_new(_("Menu"));
-	table = gtk_table_new(1, 3, FALSE);
-
-	label = gtk_label_new(_("Menu"));
-	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,  0,0,0,0);
-	page->dmenu = gtk_entry_new();
-	gtk_entry_set_editable(GTK_ENTRY(page->dmenu), FALSE);
-	page->dmenusig = gtk_signal_connect(GTK_OBJECT(page->dmenu), "changed", GTK_SIGNAL_FUNC(evt_menu_changed), page);
-	gtk_table_attach(GTK_TABLE(table), page->dmenu, 1, 2, 0, 1,  GTK_EXPAND|GTK_FILL,0,0,0);
-
-	btn = gui_details_button_new(min->gui->window->window);
-	gtk_signal_connect(GTK_OBJECT(btn), "clicked", GTK_SIGNAL_FUNC(evt_menu_pick_clicked), page);
-
-	gtk_table_attach(GTK_TABLE(table), btn, 2, 3, 0, 1,  0,0,0,0);
-
-	gtk_container_add(GTK_CONTAINER(frame), table);
-
-	return frame;
+	index = GPOINTER_TO_UINT(gtk_object_get_user_data(GTK_OBJECT(wid)));
+	if(index == 0U)
+		page->cur_sheet = NULL;
+	else
+		page->cur_sheet = "Shortcuts";	/* Hardcoded, yes. */
+	page->sheet = btn_buttonsheet_get(&page->buttons, page->cur_sheet);
+	populate_scwin(page);
 }
 
 static GtkWidget * cbt_init(MainInfo *min, gchar **name)
 {
 	P_Buttons	*page = &the_page;
-	const gchar	*blab[] = { N_("Copy To"), N_("Copy Colors To"), N_("Swap With"), N_("Clear") },
+	const gchar	*mlab[] = { N_("Default"), N_("Shortcuts"), NULL },
+			*blab[] = { N_("Copy To"), N_("Copy Colors To"), N_("Swap With"), N_("Clear") },
 			*rlab[] = { N_("Add Row..."), N_("Row Width..."), N_("Up"), N_("Down"), N_("Delete Row") };
-	GtkWidget	*hbox, *sep, *label, *frame;
+	GtkWidget	*hbox, *menu, *optm, *sep, *label, *frame;
 	guint		i;
 	gint		expand;
 
-	if(name != NULL)
-		*name = _("Buttons");
-
 	page->vbox = gtk_vbox_new(FALSE, 0);
 	page->min  = min;
 	page->button = NULL;
+	page->cur_sheet = NULL;
+
+	hbox = gtk_hbox_new(FALSE, 0);
+	label = gtk_label_new(_("Sheet"));
+	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+	optm = gtk_option_menu_new();
+	menu = gui_build_menu(mlab, GTK_SIGNAL_FUNC(evt_sheet_activate), page);
+	gtk_option_menu_set_menu(GTK_OPTION_MENU(optm), menu);
+	gtk_box_pack_start(GTK_BOX(hbox), optm, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(page->vbox), hbox, FALSE, FALSE, 0);
 
-/*	label = gtk_label_new("Sheet: Default");
-**	gtk_box_pack_start(GTK_BOX(page->vbox), label, FALSE, FALSE, 0);
-**	gtk_widget_show(label);
-*/
 	page->scwin = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(page->scwin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 	gtk_box_pack_start(GTK_BOX(page->vbox), page->scwin, TRUE, TRUE, 0);
-	gtk_widget_show(page->scwin);
 	page->shvbox = NULL;
 
 	page->dvbox = gtk_vbox_new(FALSE, 0);
@@ -817,62 +768,44 @@ static GtkWidget * cbt_init(MainInfo *min, gchar **name)
 	page->dhbox = gtk_hbox_new(FALSE, 0);
 	init_bdef(min, page, &page->def[0], _("Primary"), BTN_PRIMARY);
 	gtk_box_pack_start(GTK_BOX(page->dhbox), page->def[0].frame, TRUE, TRUE, 0);
-	gtk_widget_show(page->def[0].frame);
 	init_bdef(min, page, &page->def[1], _("Secondary"), BTN_SECONDARY);
 	gtk_box_pack_start(GTK_BOX(page->dhbox), page->def[1].frame, TRUE, TRUE, 0);
-	gtk_widget_show(page->def[1].frame);
 	gtk_box_pack_start(GTK_BOX(page->dvbox), page->dhbox, TRUE, TRUE, 0);
-	gtk_widget_show(page->dhbox);
-
-	frame = init_mdef(min, page);
-	gtk_box_pack_start(GTK_BOX(page->dvbox), frame, TRUE, TRUE, 0);
-/*	gtk_widget_show_all(frame);*/
 
 	hbox = gtk_hbox_new(FALSE, 0);
 	label = gtk_label_new(_("Tooltip"));
 	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
-	gtk_widget_show(label);
 	page->dtooltip = gtk_entry_new_with_max_length(BTN_TOOLTIP_SIZE - 1);
 	gtk_signal_connect(GTK_OBJECT(page->dtooltip), "changed", GTK_SIGNAL_FUNC(evt_dtooltip_changed), page);
 	gtk_box_pack_start(GTK_BOX(hbox), page->dtooltip, TRUE, TRUE, 0);
-	gtk_widget_show(page->dtooltip);
 	gtk_box_pack_start(GTK_BOX(page->dvbox), hbox, FALSE, FALSE, 0);
-	gtk_widget_show(hbox);
 
 	frame = gtk_frame_new(_("Flags"));
 	hbox = gtk_hbox_new(FALSE, 0);
 	page->dnarrow = gtk_check_button_new_with_label(_("Narrow?"));
-	page->dnarsig = gtk_signal_connect(GTK_OBJECT(page->dnarrow), "clicked", GTK_SIGNAL_FUNC(evt_dnarrow_clicked), (gpointer) page);
+	page->dnarsig = gtk_signal_connect(GTK_OBJECT(page->dnarrow), "clicked", GTK_SIGNAL_FUNC(evt_dnarrow_clicked), page);
 	gtk_box_pack_start(GTK_BOX(hbox), page->dnarrow, TRUE, TRUE, 0);
-	gtk_widget_show(page->dnarrow);
 
 	page->dshowtip = gtk_check_button_new_with_label(_("Show Tooltip?"));
-	gtk_signal_connect(GTK_OBJECT(page->dshowtip), "clicked", GTK_SIGNAL_FUNC(evt_dshowtip_clicked), (gpointer) page);
+	gtk_signal_connect(GTK_OBJECT(page->dshowtip), "clicked", GTK_SIGNAL_FUNC(evt_dshowtip_clicked), page);
 	gtk_box_pack_start(GTK_BOX(hbox), page->dshowtip, TRUE, TRUE, 0);
-	gtk_widget_show(page->dshowtip);
 	gtk_container_add(GTK_CONTAINER(frame), hbox);
-	gtk_widget_show(hbox);
 
 	gtk_box_pack_start(GTK_BOX(page->dvbox), frame, FALSE, FALSE, 0);
-	gtk_widget_show(frame);
 	gtk_box_pack_start(GTK_BOX(page->vbox), page->dvbox, FALSE, FALSE, 0);
-	gtk_widget_show(page->dvbox);
 
 	hbox = gtk_hbox_new(FALSE, 0);
 	for(i = 0; i < sizeof page->bcmd / sizeof page->bcmd[0]; i++)
 	{
 		page->bcmd[i] = gtk_button_new_with_label(_(blab[i]));
 		gtk_object_set_user_data(GTK_OBJECT(page->bcmd[i]), GINT_TO_POINTER(i));
-		gtk_signal_connect(GTK_OBJECT(page->bcmd[i]), "clicked", GTK_SIGNAL_FUNC(evt_bcmd_clicked), (gpointer) page);
+		gtk_signal_connect(GTK_OBJECT(page->bcmd[i]), "clicked", GTK_SIGNAL_FUNC(evt_bcmd_clicked), page);
 		gtk_box_pack_start(GTK_BOX(hbox), page->bcmd[i], TRUE, TRUE, 0);
-		gtk_widget_show(page->bcmd[i]);
 	}
 	gtk_box_pack_start(GTK_BOX(page->vbox), hbox, FALSE, FALSE, 5);
-	gtk_widget_show(hbox);
 
 	sep = gtk_hseparator_new();
 	gtk_box_pack_start(GTK_BOX(page->vbox), sep, FALSE, FALSE, 0);
-	gtk_widget_show(sep);
 
 	hbox = gtk_hbox_new(FALSE, 0);
 	for(i = 0; i < sizeof page->rcmd / sizeof page->rcmd[0]; i++)
@@ -882,16 +815,15 @@ static GtkWidget * cbt_init(MainInfo *min, gchar **name)
 		else
 			page->rcmd[i] = gtk_button_new_with_label(_(rlab[i]));
 		gtk_object_set_user_data(GTK_OBJECT(page->rcmd[i]), GINT_TO_POINTER(i));
-		gtk_signal_connect(GTK_OBJECT(page->rcmd[i]), "clicked", GTK_SIGNAL_FUNC(evt_rcmd_clicked), (gpointer) page);
+		gtk_signal_connect(GTK_OBJECT(page->rcmd[i]), "clicked", GTK_SIGNAL_FUNC(evt_rcmd_clicked), page);
 		expand = (i == 0) || (i == sizeof page->rcmd / sizeof page->rcmd[0] - 1);
 		gtk_box_pack_start(GTK_BOX(hbox), page->rcmd[i], expand, TRUE, expand ? 5 : 1);
-		gtk_widget_show(page->rcmd[i]);
 	}
 	gtk_box_pack_start(GTK_BOX(page->vbox), hbox, FALSE, FALSE, 5);
-	gtk_widget_show(hbox);
 
-	gtk_widget_show(page->vbox);
-	return page->vbox;
+	cfg_tree_level_begin(_("Buttons"));
+	cfg_tree_level_append(_("Definitions"), page->vbox);
+	return NULL;
 }
 
 /* ----------------------------------------------------------------------------------------- */
@@ -903,7 +835,7 @@ static void cbt_update(MainInfo *min)
 	page->min	= min;
 	page->modified	= FALSE;
 	btn_buttoninfo_copy(&page->buttons, &min->cfg.buttons);
-	page->sheet = btn_buttonsheet_get(&page->buttons, NULL);
+	page->sheet = btn_buttonsheet_get(&page->buttons, page->cur_sheet);
 	populate_scwin(page);
 	set_mode_normal(page);
 }
@@ -916,8 +848,8 @@ static void cbt_accept(MainInfo *min)
 
 	if(page->modified)
 	{
-		btn_buttoninfo_destroy(&min->cfg.buttons);
-		memcpy(&min->cfg.buttons, &page->buttons, sizeof min->cfg.buttons);
+		btn_buttoninfo_clear(&min->cfg.buttons);
+		min->cfg.buttons = page->buttons;
 		page->buttons.sheets = NULL;
 		cfg_set_flags(CFLG_REBUILD_BOTTOM | CFLG_RESET_KEYBOARD);
 		page->modified = FALSE;
@@ -955,9 +887,9 @@ static void cbt_hide(MainInfo *min)
 
 /* ----------------------------------------------------------------------------------------- */
 
-CfgPage * cbt_describe(MainInfo *min)
+const CfgModule * cbt_describe(MainInfo *min)
 {
-	static CfgPage	desc = { NODE, cbt_init, cbt_update, cbt_accept, cbt_save, cbt_load, cbt_hide };
+	static const CfgModule	desc = { NODE, cbt_init, cbt_update, cbt_accept, cbt_save, cbt_load, cbt_hide };
 
 	return &desc;
 }
