@@ -612,11 +612,13 @@ static void pick_external(P_CmdSeq *page)
 					"fp",	N_("First selected, with path"),
 					"fpu",	N_("First selected, with path, unselect"),
 					"fd",	N_("First selected (destination pane)"),
+					"fQ",	N_("First selected, no quotes"),
 					"F",	N_("All selected"),
 					"Fu",	N_("All selected, unselect"),
 					"Fp",	N_("All selected, with paths"),
 					"Fpu",	N_("All selected, with paths, unselect"),
 					"Fd",	N_("All selected (destination pane)"),
+					"FQ",	N_("All selected, no quotes"),
 					"Ps",	N_("Path to source pane's directory"),
 					"Pd",	N_("Path to destination pane's directory"),
 					"Ph",	N_("Path to home directory"),
@@ -626,7 +628,8 @@ static void pick_external(P_CmdSeq *page)
 					"Im:\"label\"=\"text1:choice1\",...",	N_("Input using menu"),
 					"Is:\"label\"=\"default\"",	N_("Input string"),
 					"Ix:\"label\"",			N_("Input check button (gives TRUE or FALSE)"),
-					"It:\"label\"",			N_("Set title text of input window"),
+					"It:\"label\"",			N_("Add label to input window"),
+					"It:\"-\"",			N_("Add a separator bar to input window"),
 					N_("$NAME"),			N_("Value of $NAME (environment)"),
 					"#",				N_("gentoo's PID"),
 					N_("~NAME"),			N_("Home directory for user NAME"),
