@@ -59,6 +59,9 @@ typedef struct {
 	GdkPixmap	*mpix[5];		/* Mouse icon pixmaps and masks. */
 	GdkBitmap	*mmsk[5];
 
+	GtkWidget	*cmccmd;		/* Entry widget for Click-M-Click command. */
+	GtkWidget	*cmcdelay;		/* HScale widget for cmc delay. */
+
 	GtkWidget	*nonumlock;		/* Ignore numlock check button. */
 
 	MainInfo	*min;			/* This is handy sometimes. */
@@ -130,10 +133,15 @@ static void reset_mouse_widgets(P_Controls *page)
 /* 1999-03-16 -	Reset all widgets on page to their most idle state. */
 static void reset_widgets(P_Controls *page)
 {
+	const gchar	*cmd;
+
 	page->curr_key   = NULL;
 	page->curr_mouse = NULL;
 	reset_key_widgets(page);
 	reset_mouse_widgets(page);
+	cmd = ctrl_clickmclick_get_cmdseq(page->ctrlinfo);
+	gtk_entry_set_text(GTK_ENTRY(page->cmccmd), cmd ? cmd : "");
+	gtk_adjustment_set_value(gtk_range_get_adjustment(GTK_RANGE(page->cmcdelay)), ctrl_clickmclick_get_delay(page->ctrlinfo));
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(page->nonumlock), ctrl_numlock_ignore_get(page->ctrlinfo));
 }
 
@@ -242,7 +250,6 @@ static gint evt_kcmd_changed(GtkWidget *wid, gpointer user)
 			gtk_clist_set_text(GTK_CLIST(page->kclist), page->curr_key_row, 1, cmdseq);
 		}
 	}
-
 	return TRUE;
 }
 
@@ -451,6 +458,43 @@ static void evt_mdel_clicked(GtkWidget *wid, gpointer user)
 	}
 }
 
+/* 2003-10-09 -	User set a new Click-M-Click command. */
+static void evt_clickmclick_changed(GtkWidget *wid, gpointer user)
+{
+	P_Controls	*page = user;
+
+	if(page)
+	{
+		const gchar *text = gtk_entry_get_text(GTK_ENTRY(wid));
+
+		ctrl_clickmclick_set_cmdseq(page->ctrlinfo, text);
+		gtk_widget_set_sensitive(page->cmcdelay, *text);
+		page->modified = TRUE;
+	}
+}
+
+/* 2003-10-09 -	User wants help in picking a command for Click-M-Click. */
+static void evt_clickmclick_pick_clicked(GtkWidget *wid, gpointer user)
+{
+	P_Controls	*page = user;
+
+	if(page)
+	{
+		const gchar	*cmd = csq_dialog_sync_new_wait(page->min, ccs_get_current());
+		if(cmd)
+			gtk_entry_set_text(GTK_ENTRY(page->cmccmd), cmd);
+	}
+}
+
+/* 2003-10-09 -	Maximum allowed delay for Click-M-Click recognition changed. */
+static void evt_clickmclick_delay_changed(GtkAdjustment *adj, gpointer user)
+{
+	P_Controls	*page = user;
+
+	if(page)
+		ctrl_clickmclick_set_delay(page->ctrlinfo, adj->value);
+}
+
 /* ----------------------------------------------------------------------------------------- */
 
 /* 2000-02-18 -	User hit the "Ignore Num Lock?" toggle. Update editing state. */
@@ -498,7 +542,8 @@ static void build_mouse_menu(P_Controls *page)
 static GtkWidget * cct_init(MainInfo *min, gchar **name)
 {
 	P_Controls	*page = &the_page;
-	GtkWidget	*vbox, *label, *wid;
+	GtkWidget	*vbox, *frame, *label, *wid, *hbox;
+	GtkObject	*adj;
 
 	if(name == NULL)
 		return NULL;
@@ -589,10 +634,31 @@ static GtkWidget * cct_init(MainInfo *min, gchar **name)
 	gtk_container_add(GTK_CONTAINER(page->mframe), vbox);
 	gtk_box_pack_start(GTK_BOX(page->vbox), page->mframe, TRUE, TRUE, 0);
 
+	frame = gtk_frame_new(_("Click-M-Click Gesture"));
+	hbox = gtk_hbox_new(FALSE, 0);
+	label = gtk_label_new(_("Command"));
+	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+	page->cmccmd = gtk_entry_new();
+	gtk_signal_connect(GTK_OBJECT(page->cmccmd), "changed", GTK_SIGNAL_FUNC(evt_clickmclick_changed), page);
+	gtk_box_pack_start(GTK_BOX(hbox), page->cmccmd, TRUE, TRUE, 0);
+	wid = gui_details_button_new(page->min->gui->window->window);
+	gtk_signal_connect(GTK_OBJECT(wid), "clicked", GTK_SIGNAL_FUNC(evt_clickmclick_pick_clicked), page);
+	gtk_box_pack_start(GTK_BOX(hbox), wid, FALSE, FALSE, 0);
+	label = gtk_label_new(_("Time Limit"));
+	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);
+	adj = gtk_adjustment_new(0.400f, 0.100f, 1.5f, 0.1f, 0.1f, 0.0f);
+	gtk_signal_connect(adj, "value_changed", GTK_SIGNAL_FUNC(evt_clickmclick_delay_changed), page);
+	page->cmcdelay = gtk_hscale_new(GTK_ADJUSTMENT(adj));
+	gtk_scale_set_value_pos(GTK_SCALE(page->cmcdelay), GTK_POS_RIGHT);
+	gtk_scale_set_digits(GTK_SCALE(page->cmcdelay), 3);
+	gtk_box_pack_start(GTK_BOX(hbox), page->cmcdelay, TRUE, TRUE, 0);
+	gtk_container_add(GTK_CONTAINER(frame), hbox);
+	gtk_box_pack_start(GTK_BOX(page->vbox), frame, FALSE, FALSE, 0);
+	
 	page->nonumlock = gtk_check_button_new_with_label(_("Ignore Num Lock For All Bindings?"));
 	gtk_signal_connect(GTK_OBJECT(page->nonumlock), "clicked", GTK_SIGNAL_FUNC(evt_nonumlock_clicked), page);
 	gtk_box_pack_start(GTK_BOX(page->vbox), page->nonumlock, FALSE, FALSE, 0);
-	
+
 	gtk_widget_show_all(page->vbox);
 	return page->vbox;
 }
@@ -642,10 +708,15 @@ static void keys_save(CtrlInfo *ctrl, FILE *out)
 	{
 		for(iter = keys; iter != NULL; iter = g_slist_next(iter))
 		{
-			xml_put_node_open(out, "Key");
-			xml_put_text(out, "keyname", ctrl_key_get_keyname(iter->data));
-			xml_put_text(out, "cmdseq", ctrl_key_get_cmdseq(iter->data));
-			xml_put_node_close(out, "Key");
+			const gchar	*key, *cmd;
+
+			if((key = ctrl_key_get_keyname(iter->data)) && (cmd = ctrl_key_get_cmdseq(iter->data)))
+			{
+				xml_put_node_open(out, "Key");
+				xml_put_text(out, "keyname", key);
+				xml_put_text(out, "cmdseq", cmd);
+				xml_put_node_close(out, "Key");
+			}
 		}
 		g_slist_free(keys);
 	}
@@ -673,12 +744,21 @@ static void mouse_save(CtrlInfo *ctrl, FILE *out)
 	xml_put_node_close(out, "MouseButtons");
 }
 
+static void clickmclick_save(const CtrlInfo *ctrl, FILE *out)
+{
+	xml_put_node_open(out, "ClickMClick");
+	xml_put_text(out, "cmdseq", ctrl_clickmclick_get_cmdseq(ctrl));
+	xml_put_real(out, "delay",  ctrl_clickmclick_get_delay(ctrl));
+	xml_put_node_close(out, "ClickMClick");
+}
+
 /* 1999-03-17 -	Save the current control settings right out in given <file>. */
 static gint cct_save(MainInfo *min, FILE *out)
 {
 	xml_put_node_open(out, "Controls");
 	keys_save(min->cfg.ctrlinfo, out);
 	mouse_save(min->cfg.ctrlinfo, out);
+	clickmclick_save(min->cfg.ctrlinfo, out);
 	xml_put_boolean(out, "ignore_numlock", ctrl_numlock_ignore_get(min->cfg.ctrlinfo));
 	xml_put_node_close(out, "Controls");
 
@@ -738,6 +818,16 @@ static void cct_load(MainInfo *min, const XmlNode *node)
 			ctrl_mouse_remove_all(min->cfg.ctrlinfo);
 			mousebuttons_load(min, data);
 		}
+		if((data = xml_tree_search(node, "ClickMClick")) != NULL)
+		{
+			const gchar	*cmd;
+			gfloat	delay;
+			if(xml_get_text(data, "cmdseq", &cmd) && xml_get_real(data, "delay", &delay))
+			{
+				ctrl_clickmclick_set_cmdseq(min->cfg.ctrlinfo, cmd);
+				ctrl_clickmclick_set_delay(min->cfg.ctrlinfo, delay);
+			}
+		}
 		if(xml_get_boolean(node, "ignore_numlock", &tmp))
 			ctrl_numlock_ignore_set(min->cfg.ctrlinfo, tmp);
 	}
