@@ -830,6 +830,14 @@ static void evt_default_path_grab(GtkWidget *wid, gpointer user)
 	gtk_entry_set_text(GTK_ENTRY(def->dentry), page->min->gui->pane[def->index].dir.path);
 }
 
+/* 2004-02-25 -	Set default path to "take from history". Not totally beatiful, but still. */
+static void evt_default_path_history(GtkWidget *wid, gpointer user)
+{
+	PDef	*def = user;
+
+	gtk_entry_set_text(GTK_ENTRY(def->dentry), "@history[0]");
+}
+
 /* ----------------------------------------------------------------------------------------- */
 
 /* 1999-05-15 -	Set state of "path above" flag. */
@@ -1046,8 +1054,11 @@ static void init_pane_page(P_DirPane *page, guint index, const gchar *other)
 	button = gtk_button_new_with_label(_("Grab Current"));
 	gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(evt_default_path_grab), def);
 	gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
-	gtk_container_add(GTK_CONTAINER(frame), hbox);
 	gtk_box_pack_start(GTK_BOX(def->vbox), frame, FALSE, FALSE, 0);
+	button = gtk_button_new_with_label(_("From History"));
+	gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(evt_default_path_history), def);
+	gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
+	gtk_container_add(GTK_CONTAINER(frame), hbox);
 
 	/* General section below default directory (flags, scrollbar, and 'ctrl' modification. */
 	hbox = gtk_hbox_new(FALSE, 0);
