@@ -11,7 +11,7 @@
 #include "strutil.h"
 #include "xmlutil.h"
 
-#include "cfg_gui.h"
+#include "configure.h"
 #include "cfg_module.h"
 
 #include "cfg_paths.h"
@@ -51,9 +51,9 @@ static P_Paths	the_page;
 /* ----------------------------------------------------------------------------------------- */
 
 /* 1998-08-24 -	User modified a path by editing it. Remember that. */
-static gint evt_path_changed(GtkWidget *wid, gpointer data)
+static gint evt_path_changed(GtkWidget *wid, gpointer user)
 {
-	P_Paths	*page = (P_Paths *) data;
+	P_Paths	*page = user;
 
 	if(page->open)
 		page->modified = TRUE;
@@ -62,9 +62,9 @@ static gint evt_path_changed(GtkWidget *wid, gpointer data)
 }
 
 /* 1998-10-30 -	User just changed the hiding mode. */
-static gint evt_hmode_clicked(GtkWidget *wid, gpointer data)
+static gint evt_hmode_clicked(GtkWidget *wid, gpointer user)
 {
-	P_Paths	*page = (P_Paths *) data;
+	P_Paths	*page = user;
 
 	if(GTK_TOGGLE_BUTTON(wid)->active)
 	{
@@ -79,19 +79,19 @@ static gint evt_hmode_clicked(GtkWidget *wid, gpointer data)
 }
 
 /* 1998-10-30 -	User edited the hide RE. Register change, and grab the string. */
-static gint evt_hre_changed(GtkWidget *wid, gpointer data)
+static gint evt_hre_changed(GtkWidget *wid, gpointer user)
 {
-	P_Paths	*page = (P_Paths *) data;
+	P_Paths	*page = user;
 
 	stu_strncpy(page->hideinfo.hide_re_src, gtk_entry_get_text(GTK_ENTRY(wid)), sizeof page->hideinfo.hide_re_src);
 	page->modified = TRUE;
 	return TRUE;
 }
 
-/* 1998-10-26 -	The "No Case?" check button for hide RE has been hit. */
-static gint evt_hicase_clicked(GtkWidget *wid, gpointer data)
+/* 1998-10-26 -	The "Ignore Case?" check button for hide RE has been hit. */
+static gint evt_hicase_clicked(GtkWidget *wid, gpointer user)
 {
-	P_Paths	*page = (P_Paths *) data;
+	P_Paths	*page = user;
 
 	page->hideinfo.no_case = GTK_TOGGLE_BUTTON(wid)->active;
 	page->modified = TRUE;
@@ -106,7 +106,7 @@ static gint evt_hicase_clicked(GtkWidget *wid, gpointer data)
 */
 static void populate_paths(CfgInfo *cfg, P_Paths *page)
 {
-	int	i;
+	gint	i;
 
 	for(i = 0; i < PTID_NUM_PATHS; i++)
 		gtk_entry_set_text(GTK_ENTRY(page->path[i].path), cfg->path.path[i]->str);
@@ -124,7 +124,7 @@ static void populate_paths(CfgInfo *cfg, P_Paths *page)
 static GtkWidget * cpt_init(MainInfo *min, gchar **name)
 {
 	P_Paths		*page = &the_page;
-	GtkWidget	*label, *vbox;
+	GtkWidget	*algn, *label, *vbox;
 	const gchar	*labtxt[] = { N_("Icons"), N_("GTK+ RC"), N_("fstab"), N_("mtab") },
 			*hlab[] = { N_("None"), N_("Beginning With Dot (.)"), N_("Matching RE") };
 	guint		i;
@@ -146,10 +146,12 @@ static GtkWidget * cpt_init(MainInfo *min, gchar **name)
 	{
 		for(i = 0; i < PTID_NUM_PATHS; i++)
 		{
+			algn  = gtk_alignment_new(0.9f, 0.0f, 0.0f, 0.f);
 			label = gtk_label_new(_(labtxt[i]));
-			gtk_table_attach(GTK_TABLE(page->ptable), label, 0, 1, i, i+1, 0,0,0,0);
+			gtk_container_add(GTK_CONTAINER(algn), label);
+			gtk_table_attach(GTK_TABLE(page->ptable), algn, 0, 1, i, i+1, GTK_FILL,0,0,0);
 			page->path[i].path = gtk_entry_new();
-			gtk_signal_connect(GTK_OBJECT(page->path[i].path), "changed", GTK_SIGNAL_FUNC(evt_path_changed), (gpointer) page);
+			gtk_signal_connect(GTK_OBJECT(page->path[i].path), "changed", GTK_SIGNAL_FUNC(evt_path_changed), page);
 			gtk_table_attach_defaults(GTK_TABLE(page->ptable), page->path[i].path, 1, 2, i, i+1);
 #if defined __OpenBSD__ || defined __FreeBSD__ || defined __NetBSD__
 			/* Mountlist and mounted fs files are non-configurable on BSD systems. */
@@ -170,7 +172,7 @@ static GtkWidget * cpt_init(MainInfo *min, gchar **name)
 	for(i = 0; i < sizeof page->hide.hmode / sizeof page->hide.hmode[0]; i++)
 	{
 		gtk_object_set_data(GTK_OBJECT(page->hide.hmode[i]), "mode", GINT_TO_POINTER(i));
-		gtk_signal_connect(GTK_OBJECT(page->hide.hmode[i]), "clicked", GTK_SIGNAL_FUNC(evt_hmode_clicked), (gpointer) page);
+		gtk_signal_connect(GTK_OBJECT(page->hide.hmode[i]), "clicked", GTK_SIGNAL_FUNC(evt_hmode_clicked), page);
 		if(i < 2)
 			gtk_box_pack_start(GTK_BOX(vbox), page->hide.hmode[i], FALSE, FALSE, 0);
 		else
@@ -180,10 +182,10 @@ static GtkWidget * cpt_init(MainInfo *min, gchar **name)
 			hbox = gtk_hbox_new(FALSE, 0);
 			gtk_box_pack_start(GTK_BOX(hbox), page->hide.hmode[i], FALSE, FALSE, 0);
 			page->hide.hre = gtk_entry_new_with_max_length(sizeof min->cfg.path.hideinfo.hide_re_src - 1);
-			gtk_signal_connect(GTK_OBJECT(page->hide.hre), "changed", GTK_SIGNAL_FUNC(evt_hre_changed), (gpointer) page);
+			gtk_signal_connect(GTK_OBJECT(page->hide.hre), "changed", GTK_SIGNAL_FUNC(evt_hre_changed), page);
 			gtk_box_pack_start(GTK_BOX(hbox), page->hide.hre, TRUE, TRUE, 0);
-			page->hide.hicase = gtk_check_button_new_with_label(_("No Case?"));
-			gtk_signal_connect(GTK_OBJECT(page->hide.hicase), "clicked", GTK_SIGNAL_FUNC(evt_hicase_clicked), (gpointer) page);
+			page->hide.hicase = gtk_check_button_new_with_label(_("Ignore Case?"));
+			gtk_signal_connect(GTK_OBJECT(page->hide.hicase), "clicked", GTK_SIGNAL_FUNC(evt_hicase_clicked), page);
 			gtk_box_pack_start(GTK_BOX(hbox), page->hide.hicase, FALSE, FALSE, 0);
 			gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 		}
@@ -280,7 +282,7 @@ static gint cpt_save(MainInfo *min, FILE *out)
 
 static void load_path(const XmlNode *data, gpointer user)
 {
-	CfgInfo	*cfg = (CfgInfo *) user;
+	CfgInfo	*cfg = user;
 	gint	index;
 
 	if(xml_get_integer(data, "index", &index))
@@ -313,7 +315,7 @@ static void cpt_load(MainInfo *min, const XmlNode *node)
 	const XmlNode		*data;
 
 	if((data = xml_tree_search(node, "PathList")) != NULL)
-		xml_node_visit_children(data, load_path, (gpointer) &min->cfg);
+		xml_node_visit_children(data, load_path, &min->cfg);
 	if((data = xml_tree_search(node, "HideInfo")) != NULL)
 		load_hideinfo(min, data);
 }
@@ -329,9 +331,9 @@ static void cpt_hide(MainInfo *min)
 /* ----------------------------------------------------------------------------------------- */
 
 /* 1998-08-24 -	Describe the paths config page. */
-CfgPage * cpt_describe(MainInfo *min)
+const CfgModule * cpt_describe(MainInfo *min)
 {
-	static CfgPage	desc = { NODE, cpt_init, cpt_update, cpt_accept, cpt_save, cpt_load, cpt_hide };
+	static const CfgModule	desc = { NODE, cpt_init, cpt_update, cpt_accept, cpt_save, cpt_load, cpt_hide };
 
 	return &desc;
 }
@@ -339,7 +341,7 @@ CfgPage * cpt_describe(MainInfo *min)
 /* ----------------------------------------------------------------------------------------- */
 
 /* 1998-08-25 -	Return pointer to current setting for path <id>. */
-char * cpt_get_path(PathID id)
+const gchar * cpt_get_path(PathID id)
 {
 	GtkEntry	*ent = NULL;
 
@@ -349,9 +351,8 @@ char * cpt_get_path(PathID id)
 			ent = GTK_ENTRY(the_page.path[id].path);
 			break;
 		default:
-			fprintf(stderr, "**CFGPATHS: Someone requested an illegal path!\n");
-	}
-	if(ent != NULL)
+			g_warning("cpt_get_path(): Unknown path %d requested", id);
+	}	if(ent != NULL)
 		return gtk_entry_get_text(ent);
 
 	return NULL;
