@@ -48,12 +48,11 @@ typedef struct {
 
 /* ----------------------------------------------------------------------------------------- */
 
-static void cho_body(MainInfo *min, DirPane *src, DirRow *row, GtkWindow *win, gpointer user)
+static void cho_body(MainInfo *min, DirPane *src, DirRow *row, gpointer user)
 {
 	gchar	label[MAXNAMLEN + 64];
-	ChoInfo	*cho = (ChoInfo *) user;
+	ChoInfo	*cho = user;
 
-	gtk_window_set_title(win, _("Change ownership"));
 	g_snprintf(label, sizeof label, _("Set Ownership for '%s':"), DP_ROW_NAME(row));
 	gtk_label_set_text(GTK_LABEL(cho->label), label);
 	gtk_widget_grab_focus(GTK_COMBO(cho->row[0].combo)->entry);
@@ -63,9 +62,9 @@ static void cho_body(MainInfo *min, DirPane *src, DirRow *row, GtkWindow *win, g
 
 static gboolean parse_or_lookup(const gchar *str, long lookup(const gchar *name), long *tmp)
 {
-	while(isspace((int) *str))
+	while(isspace((guchar) *str))
 		str++;
-	if(isdigit((int) *str))
+	if(isdigit((guchar) *str))
 	{
 		gchar	*eptr;
 
@@ -99,59 +98,6 @@ static gboolean get_own(ChoInfo *cho, uid_t *owner, gid_t *group)
 	return FALSE;
 }
 
-#if 0
-static int cho_action(MainInfo *min, DirPane *src, DirPane *dst, DirRow *row, gpointer user)
-{
-	ChoInfo	*cho = (ChoInfo *) user;
-	gchar	*text, namebuf[PATH_MAX];
-
-	err_clear(min);
-
-	g_snprintf(namebuf, sizeof namebuf, "%s/%s", src->dir.path, DP_ROW_NAME(row));
-
-	if((text = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(cho->row[0].combo)->entry))) != NULL)
-	{
-		long	uid;
-
-		errno = EINVAL;
-
-		while(isspace((int) *text))
-			text++;
-		if(isdigit((int) *text))
-			uid = strtol(text, NULL, 0);
-		else
-			uid = usr_lookup_uid(text);
-
-		if(uid >= 0)
-		{
-			if((text = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(cho->row[1].combo)->entry))) != NULL)
-			{
-				long	gid;
-
-				while(isspace((int) *text))
-					text++;
-				if(isdigit((int) *text))
-					gid = strtol(text, NULL, 0);
-				else
-					gid = usr_lookup_gid(text);
-
-				if(gid >= 0)
-				{
-					err_clear(min);
-					chown(namebuf, uid, gid);
-				}
-			}
-		}
-	}
-	if(errno)
-		err_set(min, errno, CMD_ID, DP_ROW_NAME(row));
-	else
-		dp_unselect(src, DP_ROW_INDEX(src, row));
-
-	return errno == 0;
-}
-#endif
-
 static gboolean chown_file(MainInfo *min, const gchar *name, uid_t owner, gid_t group)
 {
 	if(chown(name, owner, group) != 0)
@@ -199,7 +145,7 @@ static gboolean chown_dir(MainInfo *min, const gchar *name, uid_t owner, gid_t g
 	return errno == 0;
 }
 
-static int cho_action(MainInfo *min, DirPane *src, DirPane *dst, DirRow *row, gpointer user)
+static gint cho_action(MainInfo *min, DirPane *src, DirPane *dst, DirRow *row, gpointer user)
 {
 	ChoInfo		*cho = user;
 	uid_t		owner;
@@ -222,12 +168,12 @@ static int cho_action(MainInfo *min, DirPane *src, DirPane *dst, DirRow *row, gp
 
 /* ----------------------------------------------------------------------------------------- */
 
-int cmd_chown(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
+gint cmd_chown(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 {
 	UsrCategory	cat[] = { UIC_USER, UIC_GROUP };
 	gchar		*labtext[] = { N_("User"), N_("Group") };
 	gint		i, index;
-	GtkWidget	*label;
+	GtkWidget	*algn, *label;
 	GList		*list;
 	static ChoInfo	cho;
 
@@ -238,8 +184,10 @@ int cmd_chown(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
 	gtk_box_pack_start(GTK_BOX(cho.vbox), cho.rowtable, FALSE, FALSE, 5);
 	for(i = 0; i < 2; i++)
 	{
+		algn  = gtk_alignment_new(0.9f, 0.0f, 0.0f, 0.0f);
 		label = gtk_label_new(_(labtext[i]));
-		gtk_table_attach_defaults(GTK_TABLE(cho.rowtable), label, 0, 1, i, i+1);
+		gtk_container_add(GTK_CONTAINER(algn), label);
+		gtk_table_attach(GTK_TABLE(cho.rowtable), algn, 0, 1, i, i+1,  GTK_FILL,0,0,0);
 		cho.row[i].combo = gtk_combo_new();
 		list = usr_string_list_create(cat[i], cat[i] == UIC_USER ? geteuid() : getgid(), &index);
 		gtk_combo_set_popdown_strings(GTK_COMBO(cho.row[i].combo), list);
@@ -251,5 +199,6 @@ int cmd_chown(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(cho.recurse), cho.last_recurse);
 	gtk_box_pack_start(GTK_BOX(cho.vbox), cho.recurse, FALSE, FALSE, 0);
 	gtk_widget_show_all(cho.vbox);
-	return cmd_generic(min, CGF_NODST, cho_body, cho_action, NULL, &cho);
+
+	return cmd_generic(min, _("Change Ownership"), CGF_NODST, cho_body, cho_action, NULL, &cho);
 }
