@@ -27,6 +27,7 @@
 
 #include "cmd_delete.h"
 #include "cmdseq_config.h"
+#include "dialog.h"
 #include "dirpane.h"
 #include "errors.h"
 #include "fileutil.h"
@@ -43,6 +44,7 @@
 typedef struct {			/* Options used by the "Copy" command (and relatives). */
 	gboolean	modified;
 	gboolean	copy_dates;		/* Copy access and modification dates? */
+	gboolean	ignore_attrib_err;	/* Allow attribute-copying to fail silently. */
 	gboolean	leave_fullsize;		/* If destination has same size as source, leave it even if error occured. */
 	gsize		buf_size;		/* Buffer size. */
 } OptCopy;
@@ -87,72 +89,82 @@ static void copy_mode(const gchar *from, const struct stat *from_stat, const gch
 	chmod(to, from_stat->st_mode);
 }
 
+static void copy_owner(const char *from, const struct stat *from_stat, const char *to)
+{
+	struct stat	sbuf;
+
+	if(from_stat == NULL)
+	{
+		if(lstat(from, &sbuf) != 0)
+			return;
+		from_stat = &sbuf;
+	}
+	chown(to, from_stat->st_uid, from_stat->st_gid);
+}
+
+/* 2003-10-21 -	Copy attributes: dates, mode/protection flags and owner. Optionally hides error. */
+static gboolean copy_attributes(const char *from, const struct stat *from_stat, const char *to)
+{
+	gint		old_errno = errno;
+	gboolean	ret;
+
+	if(from_stat == NULL)
+	{
+		static struct stat	sbuf;
+
+		if(lstat(from, &sbuf) == 0)
+			from_stat = &sbuf;
+		else
+			return FALSE;
+	}
+
+	errno = 0;
+	copy_dates(from, from_stat, to);
+	copy_mode(from,  from_stat, to);
+	copy_owner(from, from_stat, to);
+	ret = errno == 0;
+	if(copy_options.ignore_attrib_err)
+		errno = old_errno;
+	return ret;
+}
+
 /* ----------------------------------------------------------------------------------------- */
 
-/* 1998-05-21 -	Copy a single regular file from <from> to <to>. The destination name should not
-**		include the source name (i.e., it should end with a directory or a slash). The
-**		<sstat> structure should contain the result of a stat() call on the source file,
-**		or NULL.
-** 1998-05-30 -	Now uses the passed in stat to change protection flags on the destination file
-**		(otherwise it has no bits at all, which is boring).
-** 1998-06-07 -	Now removes the destination file if copying fails in mid-copy.
-** 1998-09-01 -	Now uses the new fut_copy() function, saving lots of code. Also opens the
-**		destination file with the source's mode directly, rather than setting the
-**		flags afterward.
-** 1998-09-12 -	Modified. The <full_to> parameter now gives the complete absolute destination
-**		name directly; there is no need to append anything. Also removed a HUGE silly
-**		bug: the files weren't being closed!!! Shoot me.
-** 1999-03-02 -	Added a new (as of yet non-configurable) option (leave_fullsize): if set, when
-**		an error occurs during copy, we will check if the destination file has the same
-**		size as the source, and if so ignore the error.
-** 1999-03-15 -	Cleaned up the implementation of the leave_fullsize feature a great deal.
-*/
-gboolean copy_file(MainInfo *min, const gchar *from, const gchar *full_to, const struct stat *sstat)
+/* 2003-11-24 -	Placeholder. Implement! */
+gboolean copy_socket(MainInfo *min, const gchar *from, const gchar *full_to, const struct stat *sstat)
 {
-	gint		fd_in, fd_out;
-	off_t		put = 0;
-	struct stat	stat;
+	return FALSE;
+}
 
-	if(sstat == NULL)
+/* 2003-11-25 -	Copy a symbolic link. Does not alter the contents of the link, which might mean
+**		that links break.
+*/
+gboolean copy_link(MainInfo *min, const gchar *from, const gchar *full_to, const struct stat *sstat)
+{
+	gchar	buf[PATH_MAX];
+	gint	len;
+
+	if((len = readlink(from, buf, sizeof buf - 1)) > 0)
 	{
-		if(lstat(from, &stat) == 0)
-			sstat = &stat;
-		else
-		{
-			err_set(min, errno, CMD_ID, from);
-			return FALSE;
-		}
+		buf[len] = '\0';
+		symlink(buf, full_to);
+		/* Don't try copying any attributes for symlink, it won't fly! */
 	}
-	pgs_progress_item_begin(min, full_to, sstat->st_size);
-	if((fd_in = open(from, O_RDONLY)) >= 0)
-	{
-		if((fd_out = open(full_to, O_CREAT | O_WRONLY | O_TRUNC, sstat->st_mode | S_IWUSR)) >= 0)
-		{
-			put = fut_copy(fd_in, fd_out, copy_options.buf_size);
-			close(fd_out);
-		}
-		close(fd_in);
-		if((errno == 0) && (put == sstat->st_size))
-		{
-			copy_dates(from, sstat, full_to);
-			copy_mode(from, sstat, full_to);
-		}
-	}
-	pgs_progress_item_end(min);
 	if(errno)
-	{
-		if((put == sstat->st_size) && (put > 0) && (copy_options.leave_fullsize))	/* Leave fullsized copy? */
-		{
-			errno = 0;
-			return TRUE;
-		}
-		err_set(min, errno, CMD_ID, full_to);
-		if(fut_exists(full_to))				/* Preserves errno. */
-			unlink(full_to);			/* Attempt to remove failed destination file. */
-	}
+		err_set(min, errno, CMD_ID, from);
 	return (errno == 0) ? TRUE : FALSE;
 }
 
+/* 2003-11-24 -	Copy a FIFO. */
+gboolean copy_fifo(MainInfo *min, const gchar *from, const gchar *full_to, const struct stat *sstat)
+{
+	pgs_progress_item_begin(min, full_to, 0);
+	if(mkfifo(full_to, sstat->st_mode) == 0)
+		copy_attributes(from, sstat, full_to);
+	pgs_progress_item_end(min);
+	return errno == 0;
+}
+
 /* 1998-05-31 -	Copy a device special file. Pretty simple, since there are no contents
 **		to worry about. Note that non-root users rarely have enough power for this.
 ** 1998-09-12 -	Modified; the <full_to> parameter now gives the complete destination name.
@@ -173,10 +185,7 @@ gboolean copy_device(MainInfo *min, const gchar *from, const gchar *full_to, con
 	}
 	pgs_progress_item_begin(min, full_to, stat.st_size);
 	if(mknod(full_to, sstat->st_mode, sstat->st_rdev) == 0)
-	{
-		copy_dates(from, sstat, full_to);
-		copy_mode(from, sstat, full_to);
-	}
+		copy_attributes(from, sstat, full_to);
 	pgs_progress_item_update(min, sstat->st_size);
 	pgs_progress_item_end(min);
 	if(errno)
@@ -184,51 +193,102 @@ gboolean copy_device(MainInfo *min, const gchar *from, const gchar *full_to, con
 	return (errno == 0) ? TRUE : FALSE;
 }
 
-/* 1998-06-07 -	Copy a symbolic (soft) link. Copies the actual link, meaning that
-**		there will be a new link created at the destination. Inspects the
-**		contents of the link; if it begins with its "own" path, that part
-**		is replaced with the destination path. I feel that this might
-**		be the right thing to do.
-**		Note: Copying links is considered a rare event, and as such need
-**		      not be manically optimized.
-** 1998-09-12 -	Modified; the <full_to> param now gives the complete destination name.
-** 1998-09-23 -	Completely rewritten. Now is a lot more complex, but sometimes works
-**		better. Still, I must say I would sleep better if there were no symbolic
-**		links. They're great in theory (and in /usr/local/lib), but a pain in
-**		the butt to handle in programs like this... :(
-*/
-gboolean copy_link(MainInfo *min, const gchar *from, const gchar *full_to)
+/* 2003-11-24 -	Copy regular file. */
+gboolean copy_regular(MainInfo *min, const gchar *from, const gchar *full_to, const struct stat *sstat)
 {
-	gchar	buf[PATH_MAX], newbuf[PATH_MAX], here[PATH_MAX];
-	gchar	dbuf[PATH_MAX], *dslash;
-	gint	len;
+	gint	fd_in, fd_out;
+	off_t	put = 0;
 
-	if((getcwd(here, sizeof here)) != NULL)
+	pgs_progress_item_begin(min, full_to, sstat->st_size);
+	if((fd_in = open(from, O_RDONLY)) >= 0)
 	{
-		if((len = readlink(from, buf, sizeof buf - 1)) > 0)
+		if((fd_out = open(full_to, O_CREAT | O_WRONLY | O_TRUNC, sstat->st_mode | S_IWUSR)) >= 0)
 		{
-			buf[len] = '\0';
-			if((buf[0] == G_DIR_SEPARATOR) || (!strncmp(buf, "..", 2)))	/* Absolute or "parent-relative"? */
-				strcpy(newbuf, buf);
-			else						/* Nope, so relative. */
-			{
-				strcpy(dbuf, full_to);
-				if((dslash = strrchr(dbuf, G_DIR_SEPARATOR)) != NULL)
-					*dslash = '\0';
-				strcat(dbuf, G_DIR_SEPARATOR_S);
-				strcat(dbuf, buf);
-				if(fut_exists(dbuf))			/* Does link target exist? */
-					strcpy(newbuf, dbuf);		/* Then create relative link to it. */
-				else					/* Otherwise create absolute link to original. */
-					g_snprintf(newbuf, sizeof newbuf, "%s/%s", here, buf);
+			put = fut_copy(fd_in, fd_out, copy_options.buf_size);
+			close(fd_out);
+		}
+		close(fd_in);
+		if((errno == 0) && (put == sstat->st_size))
+			copy_attributes(from, sstat, full_to);
+	}
+	pgs_progress_item_end(min);
+	if(errno)
+	{
+		if((put == sstat->st_size) && (put > 0) && (copy_options.leave_fullsize))	/* Leave fullsized copy? */
+		{
+			errno = 0;
+			return TRUE;
+		}
+		err_set(min, errno, CMD_ID, full_to);
+		if(fut_exists(full_to))				/* Preserves errno. */
+			unlink(full_to);			/* Attempt to remove failed destination file. */
+	}
+	return (sstat->st_size == 0 || put == sstat->st_size) && (errno == 0);
+}
 
-			}
-			symlink(newbuf, full_to);
+/* 2003-11-24 -	Copy a "file". Since we speak Unix here, a file can actually be quite a lot,
+**		the core defining criteria is mainly that a file is not a directory. It can be
+**		a regular file, a device special file, a symlink, a fifo, or maybe even a socket.
+*/
+gboolean copy_file(MainInfo *min, const gchar *from, const gchar *full_to, const struct stat *sstat)
+{
+	struct stat	stat;
+	OvwRes		owres;
+
+	/* We really need to stat() the source in order to copy it. */
+	if(sstat == NULL)
+	{
+		if(lstat(from, &stat) == 0)
+			sstat = &stat;
+		else
+		{
+			err_set(min, errno, CMD_ID, from);
+			return FALSE;
 		}
 	}
-	if(errno)
-		err_set(min, errno, CMD_ID, from);
-	return (errno == 0) ? TRUE : FALSE;
+	/* Check for overwrite, and clear the way if necessary. */
+	owres = ovw_overwrite_file(min, full_to, from);
+	if(owres == OVW_SKIP)
+		return TRUE;
+	else if(owres == OVW_CANCEL)
+		return FALSE;
+	else if(owres == OVW_PROCEED_FILE)
+		del_delete_file(min, full_to);
+	else if(owres == OVW_PROCEED_DIR)
+		del_delete_dir(min, full_to, FALSE);
+	/* Look at the mode stat field, and call the correct copying function. */
+	err_clear(min);
+	if(S_ISREG(sstat->st_mode))
+		return copy_regular(min, from, full_to, sstat);
+	else if(S_ISBLK(sstat->st_mode) || S_ISCHR(sstat->st_mode))
+		return copy_device(min, from, full_to, sstat);
+	else if(S_ISLNK(sstat->st_mode))
+		return copy_link(min, from, full_to, sstat);
+	else if(S_ISFIFO(sstat->st_mode))
+		return copy_fifo(min, from, full_to, sstat);
+	else if(S_ISSOCK(sstat->st_mode))
+		return copy_socket(min, from, full_to, sstat);
+	return FALSE;
+}
+
+/* ----------------------------------------------------------------------------------------- */
+
+/* 2003-10-10 -	Check if the directory <to> is a child directory of <from>. If it
+**		is, copying <from> into <to> is a genuine Bad Idea.
+*/
+static gboolean is_parent_of(const char *from, const char *to)
+{
+	gchar	buf1[PATH_MAX], buf2[PATH_MAX];
+
+	if(fut_path_canonicalize(from, buf1, sizeof buf1) &&
+	   fut_path_canonicalize(to,   buf2, sizeof buf2))
+	{
+		gsize	len = strlen(buf1);
+		
+		if(strncmp(buf1, buf2, len) == 0)
+			return buf2[len] == G_DIR_SEPARATOR;	/* If last char is /, buf1 is really a prefix. */
+	}
+	return FALSE;
 }
 
 /* 1998-05-21 -	Copy a directory recursively, by first creating the appropriate
@@ -241,42 +301,61 @@ gboolean copy_link(MainInfo *min, const gchar *from, const gchar *full_to)
 **		created directly, without appending anything. Also changed the order of
 **		the fut_cd() and mkdir() calls, for better handling of unreable dirs.
 ** 2002-07-13 -	Will now completely delete an existing directory before copying.
+** 2003-10-06 -	No longer deletes existing directory, only non-dir on name collision.
+** 2003-10-12 -	Added simplistic check against source-contains-dest copying.
 */
 gboolean copy_dir(MainInfo *min, const gchar *from, const gchar *dest_name)
 {
 	gchar		old_dir[PATH_MAX];
 	DIR		*dir;
 	struct dirent	*de;
-	struct stat	stat, dstat;
+	struct stat	sstat;
+	OvwRes		owres;
 
-	if(lstat(from, &stat) != 0)
+	if(is_parent_of(from, dest_name))
+	{
+		gchar	ebuf[2 * PATH_MAX + 128];
+
+		g_snprintf(ebuf, sizeof ebuf, _("Can't copy directory \"%s\"\nto \"%s\":\nthe source contains the destination."), from, dest_name);
+		dlg_dialog_async_new_error(ebuf);
+		return FALSE;
+	}
+
+	if(lstat(from, &sstat) != 0)
 	{
 		err_set(min, errno, CMD_ID, from);
 		return FALSE;
 	}
-	if(lstat(dest_name, &dstat) == 0)
+	owres = ovw_overwrite_file(min, dest_name, from);
+	switch(owres)
 	{
-		if(S_ISDIR(dstat.st_mode) && !del_delete_directory(min, dest_name, FALSE))
-		{
-			err_set(min, errno, CMD_ID, dest_name);
-			return FALSE;
-		}
-		else if(!del_delete_file(min, dest_name))
+	case OVW_PROCEED_FILE:
+		if(!del_delete_file(min, dest_name))
 		{
 			err_set(min, errno, CMD_ID, dest_name);
 			return FALSE;
 		}
+		break;
+	case OVW_PROCEED:
+	case OVW_PROCEED_DIR:
+		break;
+	case OVW_SKIP:
+	case OVW_CANCEL:
+		return FALSE;
 	}
+
 	err_clear(min);
-	pgs_progress_item_begin(min, dest_name, stat.st_size);
+	pgs_progress_item_begin(min, dest_name, sstat.st_size);
 	if(fut_cd(from, old_dir, sizeof old_dir))
 	{
-		if(mkdir(dest_name, stat.st_mode | S_IWUSR) == 0)
+		if(owres == OVW_PROCEED_DIR || mkdir(dest_name, sstat.st_mode | S_IWUSR) == 0)
 		{
 			if((dir = opendir(".")) != NULL)
 			{
 				while(!errno && (de = readdir(dir)) != NULL)
 				{
+					struct stat	stat;
+
 					if(!min->cfg.dir_filter(de->d_name))
 						continue;
 					if(lstat(de->d_name, &stat) == 0)
@@ -285,21 +364,22 @@ gboolean copy_dir(MainInfo *min, const gchar *from, const gchar *dest_name)
 
 						g_snprintf(dest2, sizeof dest2, "%s%c%s", dest_name, G_DIR_SEPARATOR, de->d_name);
 						if(S_ISDIR(stat.st_mode))
-							copy_dir(min, de->d_name, dest2);
-						else if(S_ISREG(stat.st_mode))
-							copy_file(min, de->d_name, dest2, &stat);
-						else if(S_ISLNK(stat.st_mode))
-							copy_link(min, de->d_name, dest2);
-						else if(S_ISBLK(stat.st_mode) || S_ISCHR(stat.st_mode))
-							copy_device(min, de->d_name, dest2, &stat);
+						{
+							if(!copy_dir(min, de->d_name, dest2))
+								break;
+						}
+						else
+						{
+							if(!copy_file(min, de->d_name, dest2, &stat))
+								break;
+						}
 					}
 				}
 				closedir(dir);
 			}
 		}
 		fut_cd(old_dir, NULL, 0);
-		copy_dates(from, NULL, dest_name);
-		copy_mode(from, NULL, dest_name);
+		copy_attributes(from, &sstat, dest_name);
 	}
 	pgs_progress_item_end(min);
 	if(errno)
@@ -309,13 +389,11 @@ gboolean copy_dir(MainInfo *min, const gchar *from, const gchar *dest_name)
 
 /* ----------------------------------------------------------------------------------------- */
 
-/* 2002-07-23 -	Killed historical comments here, they were boring. Also FAM-adapted rescan. */
+/* 2003-11-24 -	Copy selected files and/or directories to destination pane's directory. */
 gint cmd_copy(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 {
 	gchar	old_path[PATH_MAX], dest[PATH_MAX];
 	guint	num = 0;
-	mode_t	mode;
-	OvwRes	res;
 	GSList	*slist, *iter;
 
 	err_clear(min);
@@ -332,20 +410,16 @@ gint cmd_copy(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 	for(iter = slist; !errno && (iter != NULL); iter = g_slist_next(iter))
 	{
 		g_snprintf(dest, sizeof dest, "%s/%s", dst->dir.path, DP_SEL_NAME(iter));
-		mode = DP_SEL_LSTAT(iter).st_mode;
-		res = ovw_overwrite_file(min, dest, dp_full_name(src, DP_SEL_INDEX(src, iter)));
-		if(res == OVW_SKIP)
-			continue;
-		else if(res == OVW_CANCEL)
-			break;
-		if(S_ISLNK(mode))
-			copy_link(min, DP_SEL_NAME(iter), dest);
-		else if(S_ISDIR(mode))
-			copy_dir(min, DP_SEL_NAME(iter), dest);
-		else if(S_ISREG(mode))
-			copy_file(min, DP_SEL_NAME(iter), dest, NULL);
-		else if(S_ISCHR(mode) || S_ISBLK(mode))
-			copy_device(min, DP_SEL_NAME(iter), dest, NULL);
+		if(S_ISDIR(DP_SEL_LSTAT(iter).st_mode))
+		{
+			if(!copy_dir(min, DP_SEL_NAME(iter), dest))
+				break;
+		}
+		else
+		{
+			if(!copy_file(min, DP_SEL_NAME(iter), dest, &DP_SEL_LSTAT(iter)))
+				break;
+		}
 		if(errno == 0)
 			dp_unselect(src, DP_SEL_INDEX(src, iter));
 		num++;
@@ -380,6 +454,7 @@ void cfg_copy(MainInfo *min)
 		copy_cmc = cmc_config_new("Copy", &copy_options);
 		cmc_field_add_boolean(copy_cmc, "modified", NULL, offsetof(OptCopy, modified));
 		cmc_field_add_boolean(copy_cmc, "copy_dates", _("Preserve Dates During Copy?"), offsetof(OptCopy, copy_dates));
+		cmc_field_add_boolean(copy_cmc, "ignore_attrib_err", _("Ignore Failure to Copy Attributes (Date, Owner, Mode)?"), offsetof(OptCopy, ignore_attrib_err));
 		cmc_field_add_boolean(copy_cmc, "leave_fullsize", _("Leave Failed Destination if Full Size?"), offsetof(OptCopy, leave_fullsize));
 		cmc_field_add_size(copy_cmc, "buf_size", _("Buffer Size"), offsetof(OptCopy, buf_size), 1024, (1<<20), 1024);
 		cmc_config_register(copy_cmc);
