@@ -16,6 +16,7 @@
 #include "progress.h"
 #include "strutil.h"
 #include "cmd_copy.h"
+#include "cmd_delete.h"
 #include "cmd_generic.h"
 
 #include "cmd_copyas.h"
@@ -106,7 +107,6 @@ static const gchar * op_name(CpaAction action)
 static gint cpa_action(MainInfo *min, DirPane *src, DirPane *dst, DirRow *row, gpointer user)
 {
 	gchar		dest[PATH_MAX], *text;
-	const gchar	*name;
 	CpaInfo		*cpa = user;
 	OvwRes		ores;
 
@@ -142,21 +142,23 @@ static gint cpa_action(MainInfo *min, DirPane *src, DirPane *dst, DirRow *row, g
 				err_set(min, errno, op_name(cpa->action), dp_full_name(src, DP_ROW_INDEX(src, row)));
 			return 0;
 		}
+		else if(ores == OVW_PROCEED_FILE)
+		{
+			if(!del_delete_file(min, dest))
+				return 0;
+		}
+		else if(ores == OVW_PROCEED_DIR)
+		{
+			if(!del_delete_dir(min, dest, FALSE))
+				return 0;
+		}
 
 		if((cpa->action == CPA_COPYAS) || (cpa->action == CPA_CLONE))	/* Copy? */
 		{
-			mode_t	mode;
-
-			name = DP_ROW_NAME(row);
-			mode = DP_ROW_LSTAT(row).st_mode;
-			if(S_ISLNK(mode))
-				copy_link(min, name, dest);
-			else if(S_ISDIR(mode))
-				copy_dir(min, name, dest);
-			else if(S_ISREG(mode))
-				copy_file(min, name, dest, NULL);
-			else if(S_ISCHR(mode) || S_ISBLK(mode))
-				copy_device(min, name, dest, NULL);
+			if(S_ISDIR(DP_ROW_LSTAT(row).st_mode))
+				copy_dir(min, DP_ROW_NAME(row), dest);
+			else
+				copy_file(min, DP_ROW_NAME(row), dest, &DP_ROW_LSTAT(row));
 		}
 		else								/* Link. */
 			symlink(dp_full_name(src, DP_ROW_INDEX(src, row)), dest);
@@ -179,7 +181,7 @@ static void cpa_free(gpointer user)
 
 /* ----------------------------------------------------------------------------------------- */
 
-/* 1998-09-12 -	Fire up either a COPYAS or a CLONE command, depending on the <action> arg. */
+/* 1998-09-12 -	Fire up either a COPYAS, CLONE, or SYMLINK command, depending on the <action> arg. */
 static gint copyas_or_clone(MainInfo *min, DirPane *src, DirPane *dst, CpaAction action)
 {
 	static CpaInfo	cpa;
