@@ -63,7 +63,15 @@ gint cmd_direnter(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 
 	if(dir != NULL)
 	{
-		gchar	buf[PATH_MAX] = "", component[PATH_MAX];
+		gchar	buf[PATH_MAX] = "";
+		guint	index;
+
+		if(sscanf(dir, "@history[%u]", &index) == 1)	/* Access history buffer? */
+		{
+			dir = dph_dirhistory_get_entry(src->hist, index);
+			if(dir == NULL)
+				dir = "";
+		}
 
 		if((dir[0] == G_DIR_SEPARATOR) || (dir[0] == '~'))	/* Absolute? */
 			stu_strncpy(buf, dir, sizeof buf);
@@ -77,24 +85,24 @@ gint cmd_direnter(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 				g_snprintf(buf, sizeof buf, "%s%c%s", src->dir.path, G_DIR_SEPARATOR, dir);
 		}
 		err_clear(min);
-		if(fut_path_component(buf, component))
+		fut_interpolate(buf);
+		dph_state_save(src);
+		dp_freeze(src);
+		if(dp_enter_dir(src, buf))
 		{
-			dph_state_save(src);
-			dp_freeze(src);
-			dp_enter_dir(src, component);
 			dp_redisplay(src);
 			/* Annoying hacky code to get GTK+ to resize the pane before we restore
 			** the history state; if not resized, vpos setting will likely fail.
 			*/
 			gtk_widget_queue_resize(GTK_WIDGET(src->list));
-/*			gtk_widget_queue_draw(src->scwin);*/
 			while(gtk_events_pending())
 				gtk_main_iteration();
 			dph_state_restore(src);
-			dp_thaw(src);
-			dp_show_stats(src);
-			fam_monitor(src);
+			dp_focus_enter_dir(src);
 		}
+		dp_thaw(src);
+		dp_show_stats(src);
+		fam_monitor(src);
 		if(errno != 0)
 		{
 			err_set(min, errno, "DirEnter", buf);
