@@ -56,6 +56,7 @@ static gint mva_action(MainInfo *min, DirPane *src, DirPane *dst, DirRow *row, g
 	MvaInfo		*mva = user;
 	mode_t		mode;
 	OvwRes		ores;
+	gboolean	ret = FALSE;
 
 	if(!fut_cd(src->dir.path, NULL, 0))
 		return 0;
@@ -67,34 +68,17 @@ static gint mva_action(MainInfo *min, DirPane *src, DirPane *dst, DirRow *row, g
 			g_snprintf(dest, sizeof dest, "%s/%s", dst->dir.path, text);
 		else
 			stu_strncpy(dest, text, sizeof dest);
-		ores = ovw_overwrite_file(mva->min, dest, dp_full_name(src, DP_ROW_INDEX(src, row)));
-		if(ores == OVW_SKIP)
-			return 1;
-		else if(ores == OVW_CANCEL)
-			return 0;
 		pgs_progress_begin(min, _("Moving As..."), PFLG_COUNT_RECURSIVE | PFLG_ITEM_VISIBLE | PFLG_BYTE_VISIBLE);
-		name = DP_ROW_NAME(row);
-		if(rename(name, dest) == -1)	/* Rename failed? */
-		{
-			if(errno == EXDEV)		/* Rename across devices attempted? */
-			{
-				err_clear(min);
-				mode = DP_ROW_LSTAT(row).st_mode;
-				if(S_ISLNK(mode))
-					move_link(min, name, dest);
-				else if(S_ISDIR(mode))
-					move_dir(min, name, dest);
-				else if(S_ISREG(mode))
-					move_file(min, name, dest, &DP_ROW_LSTAT(row));
-				else if(S_ISCHR(mode) || S_ISBLK(mode))
-					move_device(min, name, dest, &DP_ROW_LSTAT(row));
-			}
-		}
+		if(S_ISDIR(DP_ROW_LSTAT(row).st_mode))
+			ret = move_dir(min, DP_ROW_NAME(row), dest);
+		else
+			ret = move_file(min, DP_ROW_NAME(row), dest, &DP_ROW_STAT(row));
 		pgs_progress_end(min);
 	}
-	dp_unselect(src, DP_ROW_INDEX(src, row));
+	if(ret)
+		dp_unselect(src, DP_ROW_INDEX(src, row));
 
-	return errno == 0;
+	return ret && errno == 0;
 }
 
 static void mva_free(gpointer user)
