@@ -9,6 +9,7 @@
 #include <stdlib.h>
 
 #include "cmdseq.h"
+#include "strutil.h"
 
 #include "cmd_parent.h"
 
@@ -43,7 +44,7 @@ gint cmd_parent(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 			g_string_truncate(comp, ptr - src->dir.path);
 			if(*comp->str == '\0')
 				g_string_assign(comp, "/");
-			csq_execute_format(min, "DirEnter 'dir=%s'", comp->str);
+			csq_execute_format(min, "DirEnter 'dir=%s'", stu_escape(comp->str));
 			break;
 		}
 	}
