@@ -387,34 +387,36 @@ gint cmd_selectre(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 	SelParam	sp;
 	const gchar	*str;
 	guint		res = 0;
+	const gchar	*content;
 
 	/* Always start from the previous set of parameters. */
 	sp = old_sp;
 
-	if((str = car_bareword_get(ca, 0)) != NULL)	/* Was the expression given on the command line? */
+	/* Always allow non-bareword arguments to alter the settings. */
+	sp.set    = car_keyword_get_enum(ca, "set", sp.set, "all rows", "selected", "unselected", NULL);
+	sp.type   = car_keyword_get_enum(ca, "type", sp.type, "all types", "directories only", "non-directories only", NULL);
+	sp.action = car_keyword_get_enum(ca, "action", sp.action, "select", "unselect", "toggle", NULL);
+	if((content = car_keyword_get_value(ca, "content", "name")) != NULL)
 	{
-		const gchar	*content;
+		DPContent	ac;
 
+		if((ac = dpf_get_content_from_mnemonic(content)) < DPC_NUM_TYPES)
+			sp.content = ac;
+	}
+	sp.glob   = car_keyword_get_boolean(ca, "glob", sp.glob);
+	sp.invert = car_keyword_get_boolean(ca, "invert", sp.invert);
+	sp.full   = car_keyword_get_boolean(ca, "full", sp.full);
+	sp.nocase = car_keyword_get_boolean(ca, "nocase", sp.nocase);
+
+	/* If actual bareword expression found, it's all we need to do a search. */
+	if((str = car_bareword_get(ca, 0)) != NULL)
+	{
 		if(sp.re == NULL)
 			sp.re = g_string_new(str);
 		else
 			g_string_assign(sp.re, str);
-		sp.set    = car_keyword_get_enum(ca, "set", sp.set, "all rows", "selected", "unselected", NULL);
-		sp.type   = car_keyword_get_enum(ca, "type", sp.type, "all types", "directories only", "non-directories only", NULL);
-		sp.action = car_keyword_get_enum(ca, "action", sp.action, "select", "unselect", "toggle", NULL);
-		if((content = car_keyword_get_value(ca, "content", "name")) != NULL)
-		{
-			DPContent	ac;
-
-			if((ac = dpf_get_content_from_mnemonic(content)) < DPC_NUM_TYPES)
-				sp.content = ac;
-		}
-		sp.glob   = car_keyword_get_boolean(ca, "glob", sp.glob);
-		sp.invert = car_keyword_get_boolean(ca, "invert", sp.invert);
-		sp.full   = car_keyword_get_boolean(ca, "full", sp.full);
-		sp.nocase = car_keyword_get_boolean(ca, "nocase", sp.nocase);
 	}	
-	else			/* Use interactive dialog to get options. */
+	else			/* No bareword, so pop up dialog with settings, asking for expression. */
 	{
 		GtkWidget	*vbox, *opt_menu[4], *glob, *invert, *full, *nocase, *re;
 		guint		*mptr[sizeof opt_menu / sizeof opt_menu[0]];
