@@ -66,6 +66,7 @@ typedef struct {
 } CmdInp;
 
 typedef struct {
+	MainInfo	*min;			/* So unbelievably handy. */
 	Dialog		*dlg;			/* The dialog window. */
 	CmdInp		input[DLG_MAX_INPUT];
 	GtkWidget	*body;			/* This holds the entire GUI. */
@@ -76,7 +77,11 @@ typedef struct {
 	gboolean	str_focused;		/* Gets set as the string is first focused. */
 } CmdDlg;
 
-static const gchar	*the_true = "TRUE", *the_false = "FALSE";
+static const gchar	the_true[] = "TRUE", the_false[] = "FALSE";
+
+/* ----------------------------------------------------------------------------------------- */
+
+static void	parse_code(MainInfo *min, GString *cstr, gchar *code, CmdDlg *dlg, gboolean input_ok);
 
 /* ----------------------------------------------------------------------------------------- */
 
@@ -90,7 +95,8 @@ static void parse_code_file(MainInfo *min, GString *cstr, const gchar *code)
 	const gchar	*name;
 	gboolean	unsel = (strchr(code + 1, 'u') != NULL),
 			path  = (strchr(code + 1, 'p') != NULL),
-			dest  = (strchr(code + 1, 'd') != NULL);
+			dest  = (strchr(code + 1, 'd') != NULL),
+			quote = (strchr(code + 1, 'Q') == NULL);	/* Upper case for default-on. */
 	guint		cnt = 0U;
 	GSList		*slist, *iter;
 
@@ -101,7 +107,7 @@ static void parse_code_file(MainInfo *min, GString *cstr, const gchar *code)
 
 	for(iter = slist; iter != NULL; iter = g_slist_next(iter))
 	{
-		name = dp_name_quoted(dp, DP_SEL_INDEX(dp, iter), path);
+		name = quote ? dp_name_quoted(dp, DP_SEL_INDEX(dp, iter), path) : DP_SEL_NAME(iter);
 		g_string_append(cstr, name);
 		g_string_append_c(cstr, ' ');
 		cnt++;
@@ -229,6 +235,7 @@ static gint evt_input_string_realize(GtkWidget *wid, gpointer user)
 
 /* 1998-09-25 -	Build a string widget for an input dialog. Cut-down version.
 ** 1999-02-27 -	Modified for the new packing scheme.
+** 2003-11-16 -	Now supports brace codes in the default specifier.
 */
 static gint build_input_string(CmdDlg *dlg, const gchar *def, CmdInp *inp)
 {
@@ -246,9 +253,14 @@ static gint build_input_string(CmdDlg *dlg, const gchar *def, CmdInp *inp)
 	{
 		if((end = strchr(def + 2, '"')) != NULL)
 		{
+			GString	*dstr;
+
 			*end = '\0';
-			gtk_entry_set_text(GTK_ENTRY(ent), def + 2);
+			dstr = g_string_new("");
+			parse_code(dlg->min, dstr, def + 2, dlg, FALSE);
+			gtk_entry_set_text(GTK_ENTRY(ent), dstr->str);
 			gtk_entry_select_region(GTK_ENTRY(ent), 0, -1);
+			g_string_free(dstr, TRUE);
 		}
 	}
 	pack_input_widget(dlg, inp->inp.string.entry = ent, 1, 0);
@@ -403,6 +415,7 @@ static void parse_code_input(MainInfo *min, GString *cstr, gchar *code, CmdDlg *
 	gint		succ = 0;
 	GtkWidget	*label;
 
+	dlg->min = min;
 	if(dlg->body == NULL)
 		init_input_dialog(dlg);
 
@@ -565,24 +578,57 @@ static void destroy_input(CmdDlg *dlg)
 
 /* ----------------------------------------------------------------------------------------- */
 
-/* 1998-09-25 -	Parse a word that contains 1 or more special "brace-codes". The codes need
-**		to be replaced by a value, or an input-dialog place holder.
-*/
-static void parse_code(MainInfo *min, GString *cstr, gchar *code, CmdDlg *dlg)
+/* 2003-11-16 -	Get ending brace for a code definition. Knows about embedded braces, unlike strchr(). */
+static gchar * get_code_end(gchar *def)
 {
-	if(code == NULL || cstr == NULL || dlg == NULL)
+	const gchar	*base = def;
+	gint		level;
+
+	for(level = 1; *def; def++)
+	{
+		if(def > base && def[-1] != '\\')
+		{
+			if(*def == '{')
+				level++;
+			else if(*def == '}')
+				level--;
+			if(level == 0)
+				break;
+		}
+	}
+	if(*def && level == 0)
+		return def;
+	return NULL;
+}
+
+/* 1998-09-25 -	Parse a word that contains 1 or more special "brace-codes". The codes
+**		need to be replaced by a value, or an input-dialog place holder.
+*/
+static void parse_code(MainInfo *min, GString *cstr, gchar *code, CmdDlg *dlg, gboolean input_ok)
+{
+	if(code == NULL || cstr == NULL)
 		return;
 
 	while(code && *code)
 	{
-		if(*code == '{')		/* Code here? */
+		if(*code == '\\')		/* Backslash escape? */
+		{
+			if(code[1])
+			{
+				g_string_append_c(cstr, code[1]);
+				code += 2;
+			}
+			else
+				break;
+		}
+		else if(*code == '{')		/* Code here? */
 		{
 			gchar	*end;
 
 			code++;
-			if((end = strchr(code, '}')) != NULL)
+			if((end = get_code_end(code)) != NULL)
 			{
-				*end = '\0';		/* This is in GString, so OK. */
+				*end = '\0';		/* This is in a GString, so OK. */
 				if(code[0] == 'f' || code[0] == 'F')
 					parse_code_file(min, cstr, code);
 				else if(code[0] == 'P')
@@ -593,7 +639,7 @@ static void parse_code(MainInfo *min, GString *cstr, gchar *code, CmdDlg *dlg)
 					parse_code_pid(min, cstr, code);
 				else if(code[0] == '~')
 					parse_code_home(min, cstr, code);
-				else if(code[0] == 'I')
+				else if(code[0] == 'I' && input_ok)
 					parse_code_input(min, cstr, code, dlg);
 				code = end + 1;
 			}
@@ -616,9 +662,9 @@ static const gchar * code_end(const gchar *ptr)
 
 		for(count = 1, ptr++; *ptr && count > 0; ptr++)
 		{
-			if(*ptr == '{' && *(ptr - 1) != '\\')
+			if(*ptr == '{' && ptr[-1] != '\\')
 				count++;
-			else if(*ptr == '}' && *(ptr - 1) != '\\')
+			else if(*ptr == '}' && ptr[-1] != '\\')
 				count--;
 		}
 		return ptr;
@@ -655,7 +701,7 @@ static GString * interpolate_brace_codes(MainInfo *min, const gchar *str, CmdDlg
 						g_string_append_c(tmp, *base++);
 					code = g_string_new(ptr);
 					g_string_truncate(code, end - ptr);
-					parse_code(min, tmp, code->str, dlg);
+					parse_code(min, tmp, code->str, dlg, TRUE);
 					g_string_free(code, TRUE);
 					base = end;
 				}
