@@ -708,8 +708,9 @@ void csq_init_commands(MainInfo *min)
 	csq_add_builtin(min, "DpRecenter",	cmd_dprecenter,		NULL);
 	csq_add_builtin(min, "DpReorient",	cmd_dpreorient,		NULL);
 	csq_add_builtin(min, "DpFocus",		cmd_dpfocus,		cfg_dpfocus);
-	csq_add_builtin(min, "DpGotoRow",	cmd_dpgotorow,		NULL);
 	csq_add_builtin(min, "DpFocusPath",	cmd_dpfocuspath,	NULL);
+	csq_add_builtin(min, "DpFocusISrch",	cmd_dpfocusisrch,		cfg_dpfocusisrch);
+	csq_add_builtin(min, "DpGotoRow",	cmd_dpgotorow,		NULL);
 
 	csq_add_builtin(min, "ActivateOther",	cmd_activateother,	NULL);
 	csq_add_builtin(min, "ActivateLeft",	cmd_activateleft,	NULL);
@@ -722,7 +723,7 @@ void csq_init_commands(MainInfo *min)
 	csq_add_builtin(min, "Clone",		cmd_clone,		NULL);
 	csq_add_builtin(min, "Move",		cmd_move,		NULL);
 	csq_add_builtin(min, "MoveAs",		cmd_moveas,		NULL);
-	csq_add_builtin(min, "Delete",		cmd_delete,		NULL);
+	csq_add_builtin(min, "Delete",		cmd_delete,		cfg_delete);
 	csq_add_builtin(min, "Rename",		cmd_rename,		NULL);
 	csq_add_builtin(min, "RenameRE",	cmd_renamere,		NULL);
 	csq_add_builtin(min, "RenameSeq",	cmd_renameseq,		NULL);
@@ -759,7 +760,7 @@ void csq_init_commands(MainInfo *min)
 
 	csq_add_builtin(min, "About",		cmd_about,		NULL);
 	csq_add_builtin(min, "Run",		cmd_run,		NULL);
-	csq_add_builtin(min, "Configure",	cmd_configure,		NULL);
+	csq_add_builtin(min, "Configure",	cmd_configure,		cfg_configurecmd);
 	csq_add_builtin(min, "ConfigureSave",	cmd_configuresave,	NULL);
 	csq_add_builtin(min, "Quit", 		cmd_quit,		NULL);
 }
