@@ -14,6 +14,7 @@
 
 #include "color_dialog.h"
 #include "dialog.h"
+#include "guiutil.h"
 #include "sizeutil.h"
 #include "strutil.h"
 #include "xmlutil.h"
@@ -30,12 +31,18 @@ struct CmdCfg {
 	GList		*fields;
 };
 
-typedef enum { CFT_INTEGER = 0, CFT_BOOLEAN, CFT_SIZE, CFT_STRING, CFT_COLOR } CFType;
+typedef enum { CFT_INTEGER = 0, CFT_BOOLEAN, CFT_ENUM, CFT_SIZE, CFT_STRING, CFT_COLOR } CFType;
 
 typedef struct {
 	gint	min, max;
 } CFInt;
 
+typedef struct {
+	gsize	num;
+	gchar	*label[4];		/* Static limits... They rule! */
+	gchar	*def;
+} CFEnum;
+
 typedef struct {
 	gsize	size;
 } CFStr;
@@ -53,6 +60,7 @@ typedef struct {
 
 	union {
 	CFInt	integer;
+	CFEnum	fenum;
 	CFSize	size;
 	CFStr	string;
 	}	field;
@@ -121,6 +129,9 @@ void cmc_config_base_save(CmdCfg *cmc, FILE *out)
 				case CFT_BOOLEAN:
 					xml_put_boolean(out, fld->name, *(gboolean *) ((gchar *) cmc->base + fld->offset));
 					break;
+				case CFT_ENUM:
+					xml_put_uinteger(out, fld->name, *(gint *) ((gchar *) cmc->base + fld->offset));
+					break;
 				case CFT_SIZE:
 					xml_put_uinteger(out, fld->name, *(guint *) ((gchar *) cmc->base + fld->offset));
 					break;
@@ -158,6 +169,9 @@ void cmc_config_base_load(CmdCfg *cmc, const XmlNode *node)
 				case CFT_BOOLEAN:
 					xml_get_boolean(node, fld->name, (gboolean *) ((gchar *) cmc->base + fld->offset));
 					break;
+				case CFT_ENUM:
+					xml_get_uinteger(node, fld->name, (gint *) ((gchar *) cmc->base + fld->offset));
+					break;
 				case CFT_SIZE:
 					xml_get_uinteger(node, fld->name, (gsize *) ((gchar *) cmc->base + fld->offset));
 					break;
@@ -302,6 +316,31 @@ void cmc_field_add_boolean(CmdCfg *cmc, const gchar *name, const gchar *desc, gs
 		field_add(cmc, field_new(CFT_BOOLEAN, name, desc, offset));
 }
 
+/* 2003-10-23 -	Split a string "looking|like|this" into enum labels. */
+void cmc_field_add_enum(CmdCfg *cmc, const gchar *name, const gchar *desc, gsize offset, const gchar *def)
+{
+	if(cmc != NULL)
+	{
+		gchar	*str;
+		CField	*fld = field_new(CFT_ENUM, name, desc, offset);
+
+		fld->field.fenum.num = 0;
+		fld->field.fenum.def = g_strdup(def);
+		for(str = fld->field.fenum.def; *str;)
+		{
+			fld->field.fenum.label[fld->field.fenum.num++] = str;
+			while(*str && *str != '|')
+				str++;
+			if(*str == '|')
+			{
+				*str = '\0';
+				str++;
+			}
+		}
+		field_add(cmc, fld);
+	}
+}
+
 void cmc_field_add_size(CmdCfg *cmc, const gchar *name, const gchar *desc, gsize offset, gsize min, gsize max, gsize step)
 {
 	if(cmc != NULL)
@@ -416,6 +455,24 @@ static gint evt_boolean_clicked(GtkWidget *wid, gpointer user)
 	return TRUE;
 }
 
+static gint evt_enum_changed(GtkWidget *wid, gpointer user)
+{
+	if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wid)))
+	{
+		CmdCfg		*cmc = user;
+		CField		*fld = NULL;
+		gpointer	instance = NULL;
+
+		get_object_data(GTK_OBJECT(wid), &fld, &instance);
+		if(cmc && fld && instance)
+		{
+			*(gint *) ((gchar *) instance + fld->offset) = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(wid), "value"));
+			set_modified(cmc, instance, TRUE);
+		}
+	}
+	return TRUE;
+}
+
 /* 1999-04-05 -	A string has been modified, so we need to update the instance. */
 static gint evt_string_changed(GtkWidget *wid, gpointer user)
 {
@@ -541,6 +598,28 @@ static GtkWidget * field_build(CmdCfg *cmc, CField *fld, gpointer instance, GtkW
 				gtk_signal_connect(GTK_OBJECT(wid), "clicked", GTK_SIGNAL_FUNC(evt_boolean_clicked), cmc);
 				gtk_table_attach(GTK_TABLE(table), wid, 0, 2, row, row + 1,  GTK_EXPAND|GTK_FILL,0,5,0);
 				break;
+			case CFT_ENUM:
+				wid = gtk_frame_new(fld->desc);
+				{
+					GtkWidget	*vbox, *radio[sizeof fld->field.fenum.label / sizeof *fld->field.fenum.label];
+					GSList	*group;
+					gint	i, current = *(gint *) ((gchar *) instance + fld->offset);
+
+					vbox = gtk_vbox_new(FALSE, 0);
+					group = gui_radio_group_new(fld->field.fenum.num, fld->field.fenum.label, radio);
+					for(i = 0; i < fld->field.fenum.num; i++)
+					{
+						set_object_data(GTK_OBJECT(radio[i]), fld, instance);
+						gtk_object_set_data(GTK_OBJECT(radio[i]), "value", GINT_TO_POINTER(i));
+						gtk_signal_connect(GTK_OBJECT(radio[i]), "toggled", GTK_SIGNAL_FUNC(evt_enum_changed), cmc);
+						gtk_box_pack_start(GTK_BOX(vbox), radio[i], FALSE, FALSE, 0);
+						if(i == current)
+							gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio[i]), TRUE);
+					}
+					gtk_container_add(GTK_CONTAINER(wid), vbox);
+				}
+				gtk_table_attach(GTK_TABLE(table), wid, 0, 2, row, row + 1,  GTK_EXPAND|GTK_FILL,0,5,0);
+				break;
 			case CFT_SIZE:
 				label_build(fld->desc, table, 0, row);
 				hbox = gtk_hbox_new(FALSE, 0);
@@ -639,6 +718,9 @@ gpointer cmc_instance_new(CmdCfg *cmc)
 				case CFT_BOOLEAN:
 					fsize += sizeof (gboolean);
 					break;
+				case CFT_ENUM:
+					fsize += sizeof (gint);
+					break;
 				case CFT_SIZE:
 					fsize += sizeof (gsize);
 					break;
@@ -699,6 +781,9 @@ void cmc_instance_copy(CmdCfg *cmc, gpointer dst, gpointer src)
 				case CFT_BOOLEAN:
 					*(gboolean *) ((gchar *) dst + fld->offset) = *(gboolean *) ((gchar *) src + fld->offset);
 					break;
+				case CFT_ENUM:
+					*(gint *) ((gchar *) dst + fld->offset) = *(gboolean *) ((gchar *) src + fld->offset);
+					break;
 				case CFT_SIZE:
 					*(gsize *) ((gchar *) dst + fld->offset) = *(gsize *) ((gchar *) src + fld->offset);
 					break;
