@@ -16,14 +16,19 @@
 /* ----------------------------------------------------------------------------------------- */
 
 struct CtrlInfo {
-	MainInfo	*min;		/* So incredibly handy to have around. */
+	MainInfo	*min;			/* So incredibly handy to have around. */
 
-	gboolean	nonumlock;	/* Ignore num lock for both keyboard and mouse bindings? */
+	gboolean	nonumlock;		/* Ignore num lock for both keyboard and mouse bindings? */
 
-	GHashTable	*keys;		/* Contains all key-to-cmdseq bindings. */
-	GSList		*keys_inst;	/* List of KbdContexts into which we're currently installed. */
+	GHashTable	*keys;			/* Contains all key-to-cmdseq bindings. */
+	GSList		*keys_inst;		/* List of KbdContexts into which we're currently installed. */
 
-	GList		*mouse;		/* List of CtrlMouse mouse button mappings. Why hurry? */
+	GList		*mouse;			/* List of CtrlMouse mouse button mappings. Why hurry? */
+
+	struct {
+	GString		*cmdseq;		/* Command assigned to Click-M-Click action gesture. */
+	gfloat		delay;			/* Max delay in seconds to trigger cmc. */
+	}			clickmclick;
 };
 
 struct CtrlKey {
@@ -57,6 +62,9 @@ CtrlInfo * ctrl_new(MainInfo *min)
 
 	ctrl->mouse = NULL;
 
+	ctrl->clickmclick.cmdseq = NULL;
+	ctrl->clickmclick.delay  = 0.400f;
+
 	return ctrl;
 }
 
@@ -132,6 +140,8 @@ CtrlInfo * ctrl_copy(CtrlInfo *ctrl)
 		copy = ctrl_new(ctrl->min);
 		g_hash_table_foreach(ctrl->keys, copy_key, copy);
 		g_list_foreach(ctrl->mouse, copy_mouse, copy);
+		ctrl_clickmclick_set_cmdseq(copy, ctrl_clickmclick_get_cmdseq(ctrl));
+		ctrl_clickmclick_set_delay(copy,  ctrl_clickmclick_get_delay(ctrl));
 		ctrl_numlock_ignore_set(copy, ctrl_numlock_ignore_get(ctrl));
 	}
 	return copy;
@@ -146,6 +156,8 @@ void ctrl_destroy(CtrlInfo *ctrl)
 		g_hash_table_foreach_remove(ctrl->keys, key_remove, NULL);
 		g_hash_table_destroy(ctrl->keys);
 		g_list_foreach(ctrl->mouse, mouse_remove, NULL);
+		if(ctrl->clickmclick.cmdseq)
+			g_string_free(ctrl->clickmclick.cmdseq, TRUE);
 		g_free(ctrl);
 	}
 }
@@ -605,7 +617,7 @@ GSList * ctrl_mouse_get_list(CtrlInfo *ctrl)
 **		Ultimately, it shouldn't be possible to create such a situation, but in the current
 **		GUI it is. So this function can be used to at least warn the user.
 */
-gboolean ctrl_mouse_ambiguity_exists(CtrlInfo *ctrl)
+gboolean ctrl_mouse_ambiguity_exists(const CtrlInfo *ctrl)
 {
 	if(ctrl != NULL)
 	{
@@ -624,3 +636,35 @@ gboolean ctrl_mouse_ambiguity_exists(CtrlInfo *ctrl)
 	}
 	return FALSE;
 }
+
+void ctrl_clickmclick_set_cmdseq(CtrlInfo *ctrl, const gchar *cmdseq)
+{
+	if(cmdseq)
+	{
+		if(ctrl->clickmclick.cmdseq)
+			g_string_assign(ctrl->clickmclick.cmdseq, cmdseq);
+		else
+			ctrl->clickmclick.cmdseq = g_string_new(cmdseq);
+	}
+	else if(ctrl->clickmclick.cmdseq)
+	{
+		g_string_free(ctrl->clickmclick.cmdseq, TRUE);
+		ctrl->clickmclick.cmdseq = NULL;
+	}
+}
+
+const gchar * ctrl_clickmclick_get_cmdseq(const CtrlInfo *ctrl)
+{
+	return ctrl ? ctrl->clickmclick.cmdseq ? ctrl->clickmclick.cmdseq->str : 0 : 0;
+}
+
+void ctrl_clickmclick_set_delay(CtrlInfo *ctrl, gfloat delay)
+{
+	ctrl->clickmclick.delay = delay;
+}
+
+gfloat ctrl_clickmclick_get_delay(const CtrlInfo *ctrl)
+{
+	return ctrl ? ctrl->clickmclick.delay : 0.0f;
+}
+
