@@ -54,6 +54,12 @@ void		ctrl_mouse_remove(CtrlInfo *ctrl, CtrlMouse *mouse);
 void		ctrl_mouse_remove_all(CtrlInfo *ctrl);
 const gchar *	ctrl_mouse_map(CtrlInfo *ctrl, GdkEventButton *evt);
 GSList *	ctrl_mouse_get_list(CtrlInfo *ctrl);
-gboolean	ctrl_mouse_ambiguity_exists(CtrlInfo *ctrl);
+gboolean	ctrl_mouse_ambiguity_exists(const CtrlInfo *ctrl);
+
+/* Functions for assigning a command to Click-M-Click, which is kind of a simple gesture. */
+void		ctrl_clickmclick_set_cmdseq(CtrlInfo *ctrl, const gchar *cmdseq);
+const gchar * ctrl_clickmclick_get_cmdseq(const CtrlInfo *ctrl);
+void		ctrl_clickmclick_set_delay(CtrlInfo *ctrl, gfloat delay);
+gfloat		ctrl_clickmclick_get_delay(const CtrlInfo *ctrl);
 
 #endif		/* CONTROLS_H */
