@@ -27,6 +27,7 @@
 #include "cmdseq.h"
 #include "colorutil.h"
 #include "configure.h"
+#include "controls.h"
 #include "dirhistory.h"
 #include "dpformat.h"
 #include "errors.h"
@@ -65,6 +66,8 @@ static struct {
 	DirPane	*old_cur;
 } the_activate_queue_info = { 0, NULL };
 
+static GSList	*pathwidgetry_builders = NULL;
+
 /* ----------------------------------------------------------------------------------------- */
 
 #include "graphics/icon_dirparent.xpm"		/* Graphics for the parent button. */
@@ -179,7 +182,7 @@ const gchar * dp_name_quoted(const DirPane *dp, gint row, gboolean path)
 	*put++ = '"';
 	for(ptr = fn; *ptr; ptr++)
 	{
-		if(*ptr == '"')
+		if(*ptr == '"' || *ptr == '\\')
 			*put++ = '\\';
 		*put++ = *ptr;
 	}
@@ -218,7 +221,7 @@ static void evt_path_new(GtkWidget *wid, gpointer user)
 
 	dp_activate(dp);
 	gtk_widget_grab_focus(GTK_WIDGET(dp->main->gui->top));
-	csq_execute_format(dp->main, "DirEnter 'dir=%s'", gtk_entry_get_text(GTK_ENTRY(wid)));
+	csq_execute_format(dp->main, "DirEnter 'dir=%s'", stu_escape(gtk_entry_get_text(GTK_ENTRY(wid))));
 	if(strcmp(dp->dir.path, gtk_entry_get_text(GTK_ENTRY(wid))))	/* No set? */
 		gtk_entry_set_text(GTK_ENTRY(wid), dp->dir.path);	/* Then restore entry contents. */
 }
@@ -412,11 +415,9 @@ void dp_thaw(DirPane *dp)
 */
 static void doubleclick(DirPane *dp)
 {
-/*	err_clear(dp->main);*/
 	csq_execute(dp->main, "FileAction");
 	if(chd_get_running(NULL) == NULL)	/* If command sequence finished, reset. Else keep around. */
 		dp->dbclk_row = -1;
-/*	err_show(dp->main);*/
 }
 
 /* 1999-05-13 -	Simulate a double click on <row> in <dp>. Handy for use by the focusing
@@ -424,7 +425,7 @@ static void doubleclick(DirPane *dp)
 */
 void dp_dbclk_row(DirPane *dp, gint row)
 {
-	if(row != -1)
+	if(row >= 0)
 	{
 		dp->dbclk_row = row;
 		doubleclick(dp);
@@ -459,13 +460,10 @@ void dp_select_all(DirPane *dp)
 	dp_show_stats(dp);
 }
 
-/* 1999-03-04 -	Rewritten in a lot simpler way. */
+/* 2004-02-29 -	No longer reset the dbclk_row value, so this is now a very simple wrapper function. */
 void dp_unselect(DirPane *dp, gint row)
 {
-	if(row == dp->dbclk_row)
-		dp->dbclk_row = -1;
-	else
-		gtk_clist_unselect_row(GTK_CLIST(dp->list), row, 0);
+	gtk_clist_unselect_row(GTK_CLIST(dp->list), row, 0);
 }
 
 /* 1999-03-04 -	Unselect all rows. Real simple. */
@@ -503,7 +501,7 @@ static void evt_dirpane_select_row(GtkWidget *wid, gint row, gint col, GdkEventB
 {
 	DirPane	*dp = user;
 	DirRow	*dr;
-
+	
 	if((dr = gtk_clist_get_row_data(GTK_CLIST(wid), row)) == NULL)
 		return;
 
@@ -518,8 +516,22 @@ static void evt_dirpane_select_row(GtkWidget *wid, gint row, gint col, GdkEventB
 
 	focus_select(dp, row);
 
-	if(dp->dbclk_row == row)
-		doubleclick(dp);
+	/* If mouse-driven selection, check for double click and do stuff. */
+	if(ev != NULL && ev->type == GDK_2BUTTON_PRESS)
+	{
+		static guint32	last_time = 0U;		/* Needed to filter out mysterius duplicates. */
+
+		if(ev->time != last_time)
+		{
+			dp->dbclk_row = row;
+			dp_unselect(dp, row);
+			while(gtk_events_pending())
+				gtk_main_iteration();
+			doubleclick(dp);
+			dp->dbclk_row = -1;
+			last_time = ev->time;
+		}
+	}
 }
 
 /* 1999-03-04 -	A row has been unselected, so we need to update our statistics.
@@ -533,9 +545,6 @@ static void evt_dirpane_unselect_row(GtkWidget *wid, gint row, gint col, GdkEven
 
 	update_selection_stats(dp);
 	dp_show_stats(dp);
-
-	if(dp->dbclk_row >= 0)		/* Did a double-click occur resently? */
-		doubleclick(dp);
 }
 
 /* 1999-03-05 -	Update the state of the GDK_CONTROL_MASK bitmask in <state>, depending on how
@@ -562,30 +571,20 @@ static guint update_ctrl_state(DirPane *dp, guint state)
 	return state;
 }
 
-/* 1998-06-18 -	This handles the situation when a user clicks on a row in a directory pane. Might envoke
-**		action on files/dirs.
-** 1998-06-04 -	Added support for a little menu popping up when the right button is pressed. First it changes
-**		to the given pane.
-** 1998-06-17 -	Added shift-sensing to extend the current selection. Not so crucial in my mind, since dragging
-**		is so supported. But, nice never the less (and somewhat standard, so people might expect it).
-** 1998-06-17 -	Added control-sensing to "extend" the current UNselection. Cool?
-** 1999-03-05 -	Simplified since we now use GTK+'s built-in CList selection.
-*/
-static gboolean evt_dirpane_button_press(GtkWidget *wid, GdkEventButton *event, gpointer user)
+/* 2004-02-28 -	This is gets called when user clicks in a dirpane. It gets called early, before GTK+'s own stuff. */
+static gboolean evt_dirpane_button_press_early(GtkWidget *wid, GdkEventButton *event, gpointer user)
 {
-	const gchar	*mcmd;
-	gint		ret, row, col;
-	DirPane		*dp = user;
 	static DirPane	*last_dp = NULL;
-
-	last_dp = dp;
-
-	gtk_widget_grab_focus(GTK_WIDGET(dp->main->gui->top));
-	dp_activate_queued(dp);
+	static GTimeVal	last_time;
+	const gchar	*mcmd;
+	DirPane		*dp = user;
+	GTimeVal	now;
 
 	/* Handle commands mapped to mouse buttons. This includes the RMB menu, typically. */
 	if((event->type == GDK_BUTTON_PRESS) && (mcmd = ctrl_mouse_map(dp->main->cfg.ctrlinfo, event)) != NULL)
 	{
+		dp_activate_queued(dp);
+		last_dp = dp;
 		evt_event_set((GdkEvent *) event);
 		csq_execute(dp->main, mcmd);
 		evt_event_clear(event->type);
@@ -593,34 +592,27 @@ static gboolean evt_dirpane_button_press(GtkWidget *wid, GdkEventButton *event,
 		return TRUE;
 	}
 
-	ret = gtk_clist_get_selection_info(GTK_CLIST(wid), event->x, event->y, &row, &col);
-	if(ret && (row < dp->dir.num_rows))
+	/* Look for click-m-click, i.e. a rapid click in opposite pane after a selection. */
+	g_get_current_time(&now);
+	if(last_dp && dp != last_dp)
 	{
-		if((event->type == GDK_2BUTTON_PRESS) && (row == dp->last_row2))
+		const gchar *cmd = ctrl_clickmclick_get_cmdseq(dp->main->cfg.ctrlinfo);
+		if(cmd && *cmd)
 		{
-			MainInfo	*min = dp->main;
-
-			dp->dbclk_row = row;
-
-			/* FIXME: This is horribly ugly. If control mode is set to "system default" (i.e., ignore),
-			** we need to handle the double-click right now, since we can't punt it to the selection
-			** handler as we usually do. This is very ugly, but I think it works.
-			*/
-			if(min->cfg.dp_format[dp->index].ctrl_mode == CTM_IGNORE)
+			gfloat	elapsed = 1E-6f * (now.tv_usec - last_time.tv_usec) + (now.tv_sec - last_time.tv_sec);
+			if(elapsed < ctrl_clickmclick_get_delay(dp->main->cfg.ctrlinfo))
 			{
-				doubleclick(dp);
-				dp_unselect(dp, row);
+				csq_execute(dp->main, cmd);
+				gtk_signal_emit_stop_by_name(GTK_OBJECT(dp->list), "button_press_event");
+				return TRUE;
 			}
-			gtk_signal_emit_stop_by_name(GTK_OBJECT(dp->list), "button_press_event");
 		}
-		else
-			event->state = update_ctrl_state(dp, event->state);
-
-		dp->last_row2 = dp->last_row;
-		dp->last_row  = row;
 	}
-	else
-		dp->last_row = dp->last_row2 = -1;
+	dp_activate_queued(dp);
+	/* Poke the Control key state, if set to do so. */
+	event->state = update_ctrl_state(dp, event->state);
+	last_time = now;
+	last_dp = dp;
 
 	return TRUE;
 }
@@ -687,7 +679,6 @@ static gint qsort_result(gint s, const DirRow *da, const DirRow *db)
 		isdb = S_ISDIR(DP_ROW_STAT(db).st_mode);
 	else
 		isdb = S_ISDIR(DP_ROW_LSTAT(db).st_mode);
-
 	switch(the_sort.mode)
 	{
 		case DPS_DIRS_FIRST:
@@ -1067,6 +1058,16 @@ void dp_focus(DirPane *dp, gint row)
 	dp->focus_row = row;
 }
 
+/* 2003-11-30 -	A new dir has been entered, figure out if there should be focusing. */
+void dp_focus_enter_dir(DirPane *dp)
+{
+	if(dp->focus_row != -1)
+		return;
+	if(dp->old_focus_row == -1)
+		return;
+	dp_focus(dp, 0);
+}
+
 /* ----------------------------------------------------------------------------------------- */
 
 /* 1998-05-25 -	Moved this old workhorse into the new dirpane module, and discovered that
@@ -1185,7 +1186,7 @@ static void dp_activate_queued(DirPane *dp)
 /* ----------------------------------------------------------------------------------------- */
 
 /* 1999-03-05 -	Answer whether given pane has one or more rows selected or not. */
-gint dp_has_selection(DirPane *dp)
+gboolean dp_has_selection(DirPane *dp)
 {
 	return (dp->dbclk_row != -1) || (dp->focus_row != -1) || (GTK_CLIST(dp->list)->selection != NULL && !no_repeat);
 }
@@ -1194,7 +1195,7 @@ gint dp_has_selection(DirPane *dp)
 **		than getting the entire list through dp_get_selection(), but only if
 **		you're not going to iterate the selection anyway.
 */
-gint dp_is_selected(DirPane *dp, DirRow *row)
+gboolean dp_is_selected(DirPane *dp, DirRow *row)
 {
 	if((dp != NULL) && (row != NULL))
 	{
@@ -1202,7 +1203,7 @@ gint dp_is_selected(DirPane *dp, DirRow *row)
 			return TRUE;
 		return g_list_find(GTK_CLIST(dp->list)->selection, GINT_TO_POINTER(DP_ROW_INDEX(dp, row))) != NULL;
 	}
-	return 0;
+	return FALSE;
 }
 
 /* 1999-03-15 -	Compare two rows based on their indexes in the directory. Assumes their addresses
@@ -1236,7 +1237,7 @@ GSList * dp_get_selection(DirPane *dp)
 	else
 		for(iter = GTK_CLIST(dp->list)->selection; iter != NULL; iter = g_list_next(iter))
 			sel = g_slist_insert_sorted(sel, gtk_clist_get_row_data(GTK_CLIST(dp->list), GPOINTER_TO_INT(iter->data)),
-							compare_rows);
+						    compare_rows);
 	if(sel)
 		fam_rescan_block();
 
@@ -1318,8 +1319,6 @@ static void clear_total_stats(DirPane *dp)
 /* 1999-01-03 -	Clear the statistics for <dp>. Useful when the pane's path is about to change. */
 static void clear_stats(DirPane *dp)
 {
-	dp->dir.num_rows = 0;
-
 	clear_total_stats(dp);
 	clear_selection_stats(&dp->dir.sel);
 	dp->last_row = dp->last_row2 = -1;
@@ -1384,38 +1383,84 @@ static void update_fs_info(DirPane *dp)
 	errno = old_errno;
 }
 
-/* 1998-05-17 -	Heia Norge?! Not. Read in a directory, and insert file names into the given
-**		directory pane.
-** 1998-05-18 -	Added the <root> argument, which must be absolute. The <path> argument will, if
-**		it is not also absolute, be added to the <root>, and the result will be read.
-** 1998-05-25 -	Moved into dirpane module, and tidied up somewhat.
-** 1998-06-03 -	Fixed a bug where files where sometimes selected from the start.
-** 1998-06-07 -	Now pokes away any selection info there might be.
-** 1998-06-15 -	Did a two-liner (sort of) hack to support environment variables in source path.
-**		Very simplistic; does not handle embedded vars.
-** 1998-07-13 -	Redesigned the way lines are stored; the name is no longer stored in its full
-**		MAXNAMLEN glory, since that wastes *heaps* of memory. Instead, the fut_dir_length()
-**		function now is kind enough to also compute the number of name-bytes, and
-**		return that. This makes it possible to just allocate one more chunk of memory,
-**		and store all names in there, with pointers from the individual lines. This
-**		shrank the /dev representation on my system from 421524 bytes (!) down to 93672
-**		(1301 entries). Still much, but a lot better.
-** 1998-08-25 -	Now uses the fut_path_component() function to parse the path. This buys better
-**		handling of environment variable look-ups.
-** 1998-09-02 -	Optimized and improved. Now doesn't get all confused if we can't enter a given
-**		directory (which easily happens).
-** 1999-01-03 -	After some serious bug hunting (cheered on by Alain Valleton), I discover that
-**		the realloc() I'm linking against is broken: realloc(some_ptr, 0) doesn't work.
-** 1999-05-11 -	Contemplated the prototype to this function, and modified it. Now takes a single
-**		argument <path>, which is the absolute path we wish to go to. If the path is not
-**		absolute, it is taken relative to the current directory of the pane.
-*/
-void dp_enter_dir(DirPane *dp, const gchar *path)
+/* 2003-11-11 -	Grow allocation from <n>. Grows expontentially up to a limit, then linearly. Heuristic. */
+static gsize row_grow(gsize n)
 {
-	gchar		new_path[PATH_MAX];
-	gint		num;
-	guint		num_link = 0;
-	gsize		auxsize = 0;
+	const gsize	LINEAR = 128;
+
+	if(n == 0)
+		return 1;
+	else if(n < LINEAR)
+		return n << 1;
+	return n + LINEAR;
+}
+
+/* 2003-11-11 -	Store a row into given pane. Will grow buffers if required. Gruesome. */
+static void store_row(DirPane *dp, const char *name, const struct stat *stbuf)
+{
+	DirContents	*dir = &dp->dir;
+	DirRow		*row;
+	const gsize	nlen = strlen(name);
+
+	if(dir->num_rows == dir->row_alloc)
+	{
+		dir->row_alloc = row_grow(dir->row_alloc);
+		dir->row = g_realloc(dir->row, dir->row_alloc * sizeof *dir->row);
+	}
+
+	if((dir->name_alloc - dir->name_use) < nlen + PATH_MAX + 2)
+	{
+		dir->name_alloc += PATH_MAX + nlen + 256;	/* Enough for this name, even if symlink, and a few more. */
+		dir->name = g_realloc(dir->name, dir->name_alloc);
+	}
+	row = dir->row + dir->num_rows;
+	memcpy(dir->name + dir->name_use, name, nlen + 1);
+	DP_ROW_NAME(row) = GUINT_TO_POINTER(dir->name_use);
+	dir->name_use += nlen + 1;
+	DP_ROW_LSTAT(row) = *stbuf;
+	DP_ROW_SET_STAT(row, NULL);
+	DP_ROW_FLAGS(row) = 0;
+	if(!S_ISDIR(DP_ROW_LSTAT(row).st_mode))
+		DP_ROW_FLAGS_SET(row, DPRF_HAS_SIZE);
+
+	DP_ROW_SET_LINKNAME(row, NULL);
+
+	if(S_ISLNK(DP_ROW_LSTAT(row).st_mode))	/* Is it a symlink? */
+	{
+		gint	len, oen = errno;
+
+		len = readlink(name, dir->name + dir->name_use, PATH_MAX);
+		if(len < 0)
+			len = 0;
+		dir->name[dir->name_use + len] = '\0';
+		DP_ROW_SET_LINKNAME(row, GUINT_TO_POINTER(dir->name_use));
+		/* Do we have enough stat buffer space? */
+		if((dir->stat_alloc - dir->stat_use) < 1)
+		{
+			dir->stat_alloc = row_grow(dir->stat_alloc);
+			dir->stat = g_realloc(dir->stat, dir->stat_alloc * sizeof *dir->stat);
+		}
+		if(stat(dir->name + dir->name_use, dir->stat + dir->stat_use) == 0)
+		{
+			DP_ROW_SET_STAT(row, GINT_TO_POINTER(dir->stat_use));
+			DP_ROW_FLAGS_SET(row, DPRF_LINK_EXISTS);
+			dir->stat_use++;
+		}
+		errno = oen;
+		dir->name_use += len + 1;
+	}
+	dir->num_rows++;
+}
+
+/* 2003-11-14 -	Brand new version of this very old workhorse. Now reads in the directory in a single
+**		pass, using realloc() to grow various buffers on the fly. Slightly more wasteful of
+**		memory than the old two-pass version, but less code, more robust, and possibly a
+**		little bit faster. Returns TRUE on success.
+*/
+gboolean dp_enter_dir(DirPane *dp, const gchar *path)
+{
+	gchar	new_path[PATH_MAX];
+	DIR	*dir;
 
 	if((path[0] != '\0') && (path[0] != G_DIR_SEPARATOR))
 		g_snprintf(new_path, sizeof new_path, "%s%s%s", dp->dir.path, G_DIR_SEPARATOR_S, path);
@@ -1424,97 +1469,66 @@ void dp_enter_dir(DirPane *dp, const gchar *path)
 
 	mnt_entering(dp->main, new_path, MNT_ALWAYS);
 
+	/* To read a directory easily, we need to change to it. */
 	if(!fut_cd(new_path, NULL, 0))
 	{
 		if(dp->dir.path[0] == '\0' && fut_cd(".", NULL, 0))	/* If no previous, try current. */
-			dp_enter_dir(dp, ".");
-		else
-		{
-			err_set(dp->main, errno, "CD", new_path);
-			err_show(dp->main);
-		}
-		return;
+			return dp_enter_dir(dp, ".");
+		return FALSE;
 	}
 
-	if((num = fut_dir_length(dp, ".", &auxsize, &num_link)) < 0)
-		return;
-
 	stu_strncpy(dp->dir.path, new_path, sizeof dp->dir.path);
 	clear_stats(dp);
 	update_fs_info(dp);
-	dp->focus_row = -1;
-	if(num == 0)			/* Treat empty directories specially. */
+	dp_unfocus(dp);
+
+	/* Reset buffer usage, reusing existing space if any. */
+	dp->dir.num_rows = 0;
+	dp->dir.name_use = 0;
+	dp->dir.stat_use = 0;
+
+	/* Open directory, and read in all entries. */
+	if((dir = opendir(".")) != NULL)
 	{
-		if(dp->dir.row != NULL)
-		{
-			g_free(dp->dir.row);
-			dp->dir.row = NULL;
-		}
-		if(dp->dir.auxbuf != NULL)
-		{
-			g_free(dp->dir.auxbuf);
-			dp->dir.auxbuf = NULL;
-		}
-	}
-	else
-	{
-		DIR		*dir;
 		struct dirent	*de;
+		guint		i;
 
-		dp->dir.row    = g_realloc(dp->dir.row, num * sizeof *dp->dir.row);
-		dp->dir.auxbuf = g_realloc(dp->dir.auxbuf, auxsize);
-		if((dir = opendir(".")) != NULL)
+		/* Read in all items we can find, and let buffers grow as needed. */
+		while((de = readdir(dir)) != NULL)
 		{
-			struct stat	*auxstat = (struct stat *) dp->dir.auxbuf;
-			gchar		*auxstr = dp->dir.auxbuf + (num_link * sizeof *auxstat);
-			DirRow		*row;
+			struct stat	stbuf;
 
-			typ_identify_begin(dp->main);
-			for(row = dp->dir.row; ((de = readdir(dir)) != NULL);)
-			{
-				if(!(dp->main->cfg.dir_filter(de->d_name) && fut_check_hide(dp, de->d_name)))
-					continue;
-
-				if(lstat(de->d_name, &DP_ROW_LSTAT(row)) == 0)
-				{
-					DP_ROW_NAME(row) = auxstr;
-					strcpy(auxstr, de->d_name);
-					auxstr += strlen(de->d_name) + 1;
-					DP_ROW_SET_STAT(row, &DP_ROW_LSTAT(row));
-					DP_ROW_SET_LINKNAME(row, NULL);
-
-					if(S_ISLNK(DP_ROW_LSTAT(row).st_mode))	/* Is it a symlink? */
-					{
-						gint	len, oen;
-
-						len = readlink(de->d_name, auxstr, PATH_MAX);
-						if(len < 0)
-							len = 0;
-						auxstr[len] = '\0';
-						DP_ROW_SET_LINKNAME(row, auxstr);
-						auxstr += (len + 1);
-						oen = errno;
-						if(stat(de->d_name, auxstat) == 0)
-						{
-							DP_ROW_SET_STAT(row, (struct stat *) auxstat);
-							auxstat++;
-						}
-						errno = oen;
-					}
-					statistics_add_entry(dp, &DP_ROW_LSTAT(row));
-					DP_ROW_FLAGS(row) = 0UL;
-					if(!S_ISDIR(DP_ROW_LSTAT(row).st_mode))
-						DP_ROW_FLAGS_SET(row, DPRF_HAS_SIZE);
-					typ_identify(dp->main, row);
-					row++;
-				}
-			}
-			typ_identify_end(dp->main, dp->dir.path);
-			dp->dir.num_rows = row - dp->dir.row;
-			closedir(dir);
+			if(!(dp->main->cfg.dir_filter(de->d_name) && fut_check_hide(dp, de->d_name)))
+				continue;
+			if(lstat(de->d_name, &stbuf) == 0)
+				store_row(dp, de->d_name, &stbuf);
 		}
+		closedir(dir);
+
+		/* Now when scan is done, add buffer base to offsets stored, leaving final pointers. */
+		typ_identify_begin(dp->main);
+		for(i = 0; i < dp->dir.num_rows; i++)
+		{
+			DirRow	*row = dp->dir.row + i;
+
+			DP_ROW_NAME(row) = dp->dir.name + GPOINTER_TO_UINT(DP_ROW_NAME(row));
+			if(DP_ROW_LINKNAME(row))
+			{
+				DP_ROW_SET_LINKNAME(row, dp->dir.name + GPOINTER_TO_UINT(DP_ROW_LINKNAME(row)));
+				if(DP_ROW_FLAGS_CHK(row, DPRF_LINK_EXISTS))
+					DP_ROW_SET_STAT(row, dp->dir.stat + GPOINTER_TO_UINT(DP_ROW_GET_STAT(row)));
+				else
+					DP_ROW_SET_STAT(row, &DP_ROW_LSTAT(row));
+			}
+			else
+				DP_ROW_SET_STAT(row, &DP_ROW_LSTAT(row));
+			typ_identify(dp->main, row);
+			statistics_add_entry(dp, &DP_ROW_LSTAT(row));
+		}
+		typ_identify_end(dp->main, dp->dir.path);
 	}
 	dp_resort(dp);
+	return TRUE;
 }
 
 /* ----------------------------------------------------------------------------------------- */
@@ -1523,19 +1537,16 @@ void dp_enter_dir(DirPane *dp, const gchar *path)
 ** NOTE NOTE	This routine does NOT change the error status, i.e. it does not call anything
 **		in the error module. This is important, since it's used in the exiting code
 **		of generic commands, for instance.
+** 2003-09-26 -	Don't activate <dp> for refresh, just do it anyway. Faster, hopefully safe.
 */
 void dp_rescan(DirPane *dp)
 {
-	DirPane	*prev = dp->main->gui->cur_pane;
-
-	dp_activate(dp);
 	dph_state_save(dp);
 	dp_freeze(dp);
 	dp_enter_dir(dp, dp->dir.path);
 	dp_redisplay(dp);
 	dph_state_restore(dp);
 	dp_thaw(dp);
-	dp_activate(prev);
 }
 
 /* 2002-07-23 -	Special "weaker" version of dp_rescan(), with the added semantic difference that
@@ -1557,6 +1568,34 @@ void dp_rescan_post_cmd(DirPane *dp)
 
 /* ----------------------------------------------------------------------------------------- */
 
+/* 2003-11-08 -	Add a pathwidgtry user. Indirect, just adds a function that is later called
+**		(in dp_build()) to construct the widgetry. Returns key to use with show().
+*/
+guint dp_pathwidgetry_add(PageBuilder func)
+{
+	if(func && !g_slist_find(pathwidgetry_builders, func))
+	{
+		pathwidgetry_builders = g_slist_append(pathwidgetry_builders, func);
+		return g_slist_length(pathwidgetry_builders);
+	}
+	return 0;
+}
+
+/* 2003-11-08 -	Change to a new page of dirpane path widgetry. Returns page. */
+GtkWidget ** dp_pathwidgetry_show(DirPane *dp, guint key)
+{
+	GtkWidget	*page;
+
+	if((page = gtk_notebook_get_nth_page(GTK_NOTEBOOK(dp->notebook), key)) != NULL)
+	{
+		gtk_notebook_set_page(GTK_NOTEBOOK(dp->notebook), key);
+		return gtk_object_get_data(GTK_OBJECT(page), "dp-pathwidgetry");
+	}
+	return NULL;
+}
+
+/* ----------------------------------------------------------------------------------------- */
+
 /* 1998-06-26 -	A lot of code broken out from the giant function below and put here. This
 **		makes rebuilding just the list (and not the path entry, for example) a
 **		whole lot easier. Useful when config changes...
@@ -1582,11 +1621,11 @@ static void dp_build_list(DPFormat *fmt, DirPane *dp)
 		gtk_clist_set_column_justification(GTK_CLIST(list), i, fmt->format[i].just);
 		GTK_WIDGET_UNSET_FLAGS(GTK_CLIST(list)->column[i].button, GTK_CAN_FOCUS);
 	}
-	gtk_signal_connect(GTK_OBJECT(list), "button_press_event", GTK_SIGNAL_FUNC(evt_dirpane_button_press), dp);
-	gtk_signal_connect(GTK_OBJECT(list), "click_column", GTK_SIGNAL_FUNC(evt_dirpane_col_click), dp);
-	gtk_signal_connect(GTK_OBJECT(list), "select_row", GTK_SIGNAL_FUNC(evt_dirpane_select_row), dp);
-	gtk_signal_connect(GTK_OBJECT(list), "unselect_row", GTK_SIGNAL_FUNC(evt_dirpane_unselect_row), dp);
-	gtk_signal_connect(GTK_OBJECT(list), "resize_column", GTK_SIGNAL_FUNC(evt_dirpane_resize_column), dp);
+	gtk_signal_connect(GTK_OBJECT(list), "button_press_event",	GTK_SIGNAL_FUNC(evt_dirpane_button_press_early), dp);
+	gtk_signal_connect(GTK_OBJECT(list), "click_column",		GTK_SIGNAL_FUNC(evt_dirpane_col_click), dp);
+	gtk_signal_connect(GTK_OBJECT(list), "select_row",		GTK_SIGNAL_FUNC(evt_dirpane_select_row), dp);
+	gtk_signal_connect(GTK_OBJECT(list), "unselect_row",		GTK_SIGNAL_FUNC(evt_dirpane_unselect_row), dp);
+	gtk_signal_connect(GTK_OBJECT(list), "resize_column",		GTK_SIGNAL_FUNC(evt_dirpane_resize_column), dp);
 
 	gtk_widget_set_events(list, GDK_POINTER_MOTION_MASK);
 	if(fmt->scrollbar_always)
@@ -1609,6 +1648,7 @@ GtkWidget * dp_build(MainInfo *min, DPFormat *fmt, DirPane *dp)
 	GdkPixmap	*pix;
 	GdkBitmap	*msk;
 	GtkWidget	*hbox, *pr, *ihbox;
+	GSList	*iter;
 
 	if(pane_selected != NULL)
 	{
@@ -1636,6 +1676,10 @@ GtkWidget * dp_build(MainInfo *min, DPFormat *fmt, DirPane *dp)
 		gtk_scrolled_window_set_placement(GTK_SCROLLED_WINDOW(dp->scwin), GTK_CORNER_TOP_LEFT);
 	dp_build_list(fmt, dp);
 	gtk_container_add(GTK_CONTAINER(dp->scwin), GTK_WIDGET(dp->list));
+
+	dp->notebook = gtk_notebook_new();
+	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(dp->notebook), FALSE);
+	gtk_notebook_set_show_border(GTK_NOTEBOOK(dp->notebook), FALSE);
 	hbox = gtk_hbox_new(FALSE, 0);
 	pr   = gtk_button_new();
 	if((pix = gdk_pixmap_colormap_create_from_xpm_d(NULL, gtk_widget_get_default_colormap(), &msk, NULL, icon_dirparent_xpm)) != NULL)
@@ -1670,6 +1714,20 @@ GtkWidget * dp_build(MainInfo *min, DPFormat *fmt, DirPane *dp)
 	gtk_signal_connect(GTK_OBJECT(dp->hide), "clicked", GTK_SIGNAL_FUNC(evt_hide_clicked), dp);
 	gtk_box_pack_start(GTK_BOX(hbox), dp->hide, FALSE, FALSE, 0);
 	gtk_tooltips_set_tip(min->gui->main_tt, dp->hide, _("Click to enable/disable Hide rule (When pressed in, the hide rule is active, and matching entries are hidden)"), NULL);
+	gtk_notebook_append_page(GTK_NOTEBOOK(dp->notebook), hbox, NULL);
+
+	/* If we have any registered pathwidgetry builders, go through and add their output to notebook. */
+	for(iter = pathwidgetry_builders; iter != NULL; iter = g_slist_next(iter))
+	{
+		PageBuilder	func = iter->data;
+		GtkWidget	**page;
+
+		if(func && (page = func(min)) != NULL)
+		{
+			gtk_object_set_data(GTK_OBJECT(*page), "dp-pathwidgetry", page);
+			gtk_notebook_append_page(GTK_NOTEBOOK(dp->notebook), *page, NULL);
+		}
+	}
 	if(fmt->huge_parent)
 	{
 		GtkWidget	*pbtn;
@@ -1695,13 +1753,13 @@ GtkWidget * dp_build(MainInfo *min, DPFormat *fmt, DirPane *dp)
 
 	if(fmt->path_above)
 	{
-		gtk_box_pack_start(GTK_BOX(dp->vbox), hbox, FALSE, FALSE, 0);
+		gtk_box_pack_start(GTK_BOX(dp->vbox), dp->notebook, FALSE, FALSE, 0);
 		gtk_box_pack_start(GTK_BOX(dp->vbox), ihbox, TRUE, TRUE, 0);
 	}
 	else
 	{
 		gtk_box_pack_start(GTK_BOX(dp->vbox), ihbox, TRUE, TRUE, 0);
-		gtk_box_pack_start(GTK_BOX(dp->vbox), hbox, FALSE, FALSE, 0);
+		gtk_box_pack_start(GTK_BOX(dp->vbox), dp->notebook, FALSE, FALSE, 0);
 	}		
 	gtk_widget_show_all(dp->vbox);
 	dp->dbclk_row = -1;
