@@ -347,8 +347,8 @@ static void put_size(gchar **out, const DC_Size *dcs, const DirRow *row)
 				unit = _("GB");
 			}
 		}
-		if(dcs->digits && (size - (gint) size) != 0.0)
-			uptr = *out + 32 + sprintf(*out + 32, dcs->dformat, size - (gint) size);
+		if(dcs->digits && (size - (guint64) size) != 0.0)
+			uptr = *out + 32 + sprintf(*out + 32, dcs->dformat, size - (guint64) size);
 		else
 			*(uptr = *out + 33) = '\0';
 		if(dcs->ticks)
@@ -392,7 +392,7 @@ void dpf_format_column(MainInfo *min, const DirPane *dp, const DirRow *row, guin
 				format_typed_name(*out, DP_ROW_NAME(row), DP_ROW_LSTAT(row).st_mode);
 			else
 				sprintf(*out, "%s", DP_ROW_NAME(row));
-			if(S_ISLNK(DP_ROW_LSTAT(row).st_mode) && col_fmt->extra.name.show_linkname)
+			if(S_ISLNK(DP_ROW_LSTAT(row).st_mode) && col_fmt->extra.name.show_linkname && DP_ROW_LINKNAME(row)[0])
 				sprintf(strchr(*out, '\0'), " -> %s", DP_ROW_LINKNAME(row));
 			break;
 		case DPC_SIZE:
