@@ -1,7 +1,6 @@
 /*
 ** 1998-05-25 -	This module contains various file and directory related operations, that
 **		are needed here and there in the application.
-** 1998-09-02 -	Debugged and improved the fut_dir_length() function somewhat.
 ** 1999-11-13 -	Rewrote the core fut_copy() function. Shorter, simpler, better. Added
 **		a complementary fut_copy_partial() function that copies just parts of files.
 */
@@ -46,6 +45,151 @@ gboolean fut_cd(const gchar *new_dir, gchar *old, gsize old_max)
 	return chdir(new_dir) == 0 ? TRUE : FALSE;
 }
 
+#if 0
+/* 2003-10-10 -	Compute a directory stat chain from <path>. It's made
+**		up of a list of stat(2) structures up to the root.
+*/
+GList * fut_stat_chain_new(const gchar *path)
+{
+	gchar	canon[PATH_MAX];
+	GList	*chain = NULL;
+	GString	*dir;
+
+	if(!fut_path_canonicalize(path, canon, sizeof canon))
+		return NULL;
+
+	dir = g_string_new(canon);
+	printf("chaining '%s'\n", canon);
+
+	while(dir->len && strcmp(dir->str, G_DIR_SEPARATOR_S) != 0)
+	{
+		struct stat	stbuf;
+
+		if(stat(dir->str, &stbuf) == 0)
+		{
+			struct stat	*st = g_malloc(sizeof *st);
+			const char *sep;
+
+			*st = stbuf;
+			chain = g_list_append(chain, st);
+			printf("chain: got <%llu,%llu> for '%s'\n", st->st_dev, st->st_ino, dir->str);
+			if((sep = strrchr(dir->str, G_DIR_SEPARATOR)) != NULL)
+				g_string_truncate(dir, sep - dir->str);
+			else
+				break;
+		}
+		else	/* A stat() call failed, then path is bad and no chain exists. Fail. */
+		{
+			fut_stat_chain_free(chain);
+			return NULL;
+		}
+	}
+	return chain;
+}
+
+/* 2003-10-10 -	Returns true if the second chain is a prefix of the first, which can also
+**		be more clearly put as "chain1 represents a subdir of chain2". Um, at least
+**		that's a different wording.
+*/
+gboolean fut_stat_chain_prefix_equal(const GList *chain1, const GList *chain2)
+{
+	if(!chain1 || !chain2)
+		return FALSE;
+
+	printf("prefix: chain1=%p chain2=%p\n", chain1, chain2);
+	for(; chain1 && chain2; chain1 = g_list_next(chain1), chain2 = g_list_next(chain2))
+	{
+		const struct stat	*s1 = chain1->data, *s2 = chain2->data;
+
+		printf("comparing stats at %p,%p\n", s1, s2);
+		if(s1->st_dev != s2->st_dev || s1->st_ino != s2->st_ino)
+			return FALSE;
+	}
+	return TRUE;
+}
+
+void fut_stat_chain_free(GList *chain)
+{
+	GList	*iter;
+
+	for(iter = chain; iter != NULL; iter = g_list_next(iter))
+		g_free(iter->data);
+	g_list_free(chain);
+}
+#endif
+
+/* 2003-10-10 -	Get next path part, up to (but not including) the next separator. */
+static GString * get_part(GString *in, char *out)
+{
+	const char	*sep = strchr(in->str, G_DIR_SEPARATOR);
+
+	if(sep)
+	{
+		gsize	len = sep - in->str;
+		strncpy(out, in->str, len);
+		out[len] = '\0';
+		g_string_erase(in, 0, len + 1);
+		return in;
+	}
+	return NULL;
+}
+
+/* 2003-10-10 -	Make a path canonical, i.e. make it absolute and remove any "..", "."
+**		and repeated slashes it might contain. Pure string operation.
+*/
+gboolean fut_path_canonicalize(const gchar *path, gchar *outpath, gsize maxout)
+{
+	GString	*dir, *out;
+	char	part[PATH_MAX];
+
+	if(!path || *path == '\0' || !outpath || maxout < 2)
+		return FALSE;
+
+	if(*path == G_DIR_SEPARATOR)
+		dir = g_string_new(path);
+	else
+	{
+		char	buf[PATH_MAX];
+
+		if(getcwd(buf, sizeof buf) == NULL)
+			return FALSE;
+		if(buf[0] != G_DIR_SEPARATOR)
+			dir = g_string_new(G_DIR_SEPARATOR_S);
+		else
+			dir = g_string_new(buf);
+		g_string_append_c(dir, G_DIR_SEPARATOR);
+		g_string_append(dir, path);
+	}
+	g_string_append_c(dir, G_DIR_SEPARATOR);		/* Must end in separator for get_part(). */
+
+	out = g_string_new("");
+	while(get_part(dir, part))
+	{
+		if(*part)
+		{
+			if(strcmp(part, ".") == 0)
+				continue;
+			else if(strcmp(part, "..") == 0)
+			{
+				const char	*sep = strrchr(out->str, G_DIR_SEPARATOR);
+				g_string_truncate(out, sep - out->str);
+				continue;
+			}
+			g_string_append_c(out, G_DIR_SEPARATOR);
+			g_string_append(out, part);
+		}
+	}
+	g_string_free(dir, TRUE);
+	if(out->len < maxout)
+	{
+		strcpy(outpath, out->str);
+		g_string_free(out, TRUE);
+		return TRUE;
+	}
+	g_string_free(out, TRUE);
+	return FALSE;
+}
+
 /* 1998-09-23 -	Return 1 if the named file exists, 0 otherwise. Since this is Unix
 **		(I think), this matter is somewhat complex. But I choose to ignore
 **		that for now, and pretend it's nice and simple. :)
@@ -91,66 +235,6 @@ gboolean fut_fsize(gint fd, gsize *size)
 
 /* ----------------------------------------------------------------------------------------- */
 
-/* 1998-05-17 -	Return number of entries in given directory.
-** 1998-07-13 -	If <nsize> is non-NULL, it is filled-in with the number of bytes required to
-**		hold all file names (including '\0'-terminations). This saves **heaps** of memory.
-** 1998-09-02 -	If the directory couldn't be read, we now return -1 (which is what the dirpane
-**		module expected all along).
-** 1998-12-14 -	Changed old <min> arg into <dp>, for compatibility with the fut_check_hide()
-**		function's changed prototype.
-** 1999-05-29 -	The old <nsize> (for name size) argument has been extended. It's now called
-**		<auxsize>, and includes size requirements for symbolic links (link contents and
-**		extra stat buffer).
-** 1999-06-01 -	If <num_links> is non-NULL, it will be filled-in with the number of symlinks found.
-*/
-gint fut_dir_length(DirPane *dp, const gchar *path, gsize *auxsize, guint *num_links)
-{
-	DIR		*dir;
-	MainInfo	*min = dp->main;
-	struct dirent	*de;
-	struct stat	stbuf;
-	gint		num = -1;
-	guint		nlink = 0;
-	gsize		nl = 0;
-
-	if((dir = opendir(path)) != NULL)
-	{
-		for(num = 0; (de = readdir(dir)) != NULL; num++)
-		{
-			if(min->cfg.dir_filter(de->d_name) && fut_check_hide(dp, de->d_name))
-			{
-				nl += strlen(de->d_name) + 1;
-				if((auxsize != NULL) && (lstat(de->d_name, &stbuf) == 0))
-				{
-					if(S_ISLNK(stbuf.st_mode))
-					{
-						gchar	lbuf[PATH_MAX];
-						gint	len;
-
-						nlink++;
-						nl += sizeof stbuf + 1;
-						if((len = readlink(de->d_name, lbuf, sizeof lbuf)) > 0)
-						{
-							lbuf[len] = '\0';
-							nl += len;
-						}
-					}
-				}
-			}
-			else
-				num--;		/* Compensate for ++ in loop head. Ugly. */
-		}
-		closedir(dir);
-	}
-	if(auxsize != NULL)
-		*auxsize = nl;
-	if(num_links != NULL)
-		*num_links = nlink;
-	return num;
-}
-
-/* ----------------------------------------------------------------------------------------- */
-
 /* 1998-12-20 -	This is under the hood of the fut_dir_size() function. */
 static gint recurse_dir_size(MainInfo *min, const gchar *path, guint64 *bytes, FUCount *fc, gboolean with_progress)
 {
@@ -265,6 +349,228 @@ gint fut_dir_size_progress(MainInfo *min, const gchar *path, guint64 *bytes, FUC
 
 /* ----------------------------------------------------------------------------------------- */
 
+/* Transversal state for a single directory when doing asynchronous dir sizing. */
+typedef struct {
+	DIR 	*dir;
+	gchar	*path;
+} DirSizeState;
+
+/* An "operation" is the result of a single call to fut_dir_size_start(). */
+typedef struct {
+	SizeFunc	func;
+	gpointer	user;
+	FUCount	count;
+	DirSizeState	*current;
+	GSList	*stack;
+} DirSizeOp;
+
+/* Global state for asynchronous directory size computation. */
+static struct {
+	guint	idle;
+	GList	*ops;
+} dir_size_info = { 0U };
+
+/* 2003-10-16 -	Open given path and return new dir size state struct describing it. */
+static DirSizeState * dir_size_open(const gchar *path)
+{
+	DirSizeState	*st;
+
+	st = g_malloc(sizeof *st);
+	if((st->dir = opendir(path)) != NULL)
+	{
+		st->path = g_strdup(path);
+		return st;
+	}
+	g_free(st);
+	return NULL;
+}
+
+/* 2003-10-16 -	Free a directory state. */
+static void dir_size_free(DirSizeState *state)
+{
+	if(state->dir)
+		closedir(state->dir);
+	g_free(state->path);
+	g_free(state);
+}
+
+/* 2003-10-16 -	Update asynchronous directory sizing. */
+static gboolean dir_size_update(DirSizeOp *op)
+{
+	int	i;
+	struct dirent	*de;
+	struct stat	stbuf;
+
+	for(i = 0; op->current && i < 32; i++)
+	{
+		if((de = readdir(op->current->dir)) != NULL)
+		{
+			gchar	full[PATH_MAX];
+
+			if(strcmp(de->d_name, ".") == 0 || strcmp(de->d_name, "..") == 0)
+				continue;
+			/* Don't CD for asynchronous reasons. Build full paths, it's fairly cheap. */
+			g_snprintf(full, sizeof full, "%s%s%s", op->current->path, G_DIR_SEPARATOR_S, de->d_name);
+			if(lstat(full, &stbuf) == 0)
+			{
+				op->count.num_bytes  += (guint64) stbuf.st_size;
+				op->count.num_blocks += (guint64) stbuf.st_blocks;
+				if(S_ISDIR(stbuf.st_mode))
+				{
+					DirSizeState	*nd;
+
+					if((nd = dir_size_open(full)) != NULL)
+					{
+						op->stack   = g_slist_prepend(op->stack, op->current);
+						op->current = nd;
+						op->count.num_dirs++;
+					}
+				}
+				else
+				{
+					if(S_ISREG(stbuf.st_mode))
+						op->count.num_files++;
+					else if(S_ISLNK(stbuf.st_mode))
+						op->count.num_links++;
+					else if(S_ISCHR(stbuf.st_mode))
+						op->count.num_cdevs++;
+					else if(S_ISBLK(stbuf.st_mode))
+						op->count.num_bdevs++;
+					else if(S_ISFIFO(stbuf.st_mode))
+						op->count.num_fifos++;
+					else if(S_ISSOCK(stbuf.st_mode))
+						op->count.num_socks++;
+				}
+			}
+		}
+
+		if(de == NULL)
+		{
+			dir_size_free(op->current);
+			op->current = NULL;
+			if(op->stack != NULL)
+			{
+				GSList	*top = op->stack;
+
+				op->stack = g_slist_remove_link(op->stack, top);
+				op->current = top->data;
+				g_slist_free_1(top);
+			}
+		}
+	}
+	/* After looping around for a while, tell op owner new numbers. */
+	op->func(&op->count, op->user);
+	return op->current != NULL;
+}
+
+/* 2003-10-16 -	GTK+ idle callback for asynchronous directory sizing. Update all on-going operations. */
+static gint idle_dir_size(gpointer user)
+{
+	GList	*iter, *next;
+
+	for(iter = dir_size_info.ops; iter != NULL; iter = next)
+	{
+		next = g_list_next(iter);
+		if(!dir_size_update(iter->data))
+			fut_dir_size_stop(iter->data);
+	}
+	return TRUE;
+}
+
+/* 2003-10-16 -	Start off a new operation, by allocating, initializing it, and adding it to list. */
+gpointer fut_dir_size_start(const gchar *path, SizeFunc func, gpointer user)
+{
+	DirSizeOp	*op;
+
+	op = g_malloc(sizeof *op);
+	op->func = func;
+	op->user = user;
+	memset(&op->count, 0, sizeof op->count);
+	op->current = dir_size_open(path);
+	op->stack = NULL;
+
+	dir_size_info.ops = g_list_append(dir_size_info.ops, op);
+
+	/* If the idle function isn't active, start it up. */
+	if(dir_size_info.idle == 0U)
+		dir_size_info.idle = gtk_idle_add_priority(G_PRIORITY_LOW, idle_dir_size, NULL);
+	return op;
+}
+
+/* 2003-10-16 -	Stop a directory sizing operation. */
+void fut_dir_size_stop(gpointer handle)
+{
+	DirSizeOp	*op = handle;
+	GList	*link;
+
+	if((link = g_list_find(dir_size_info.ops, op)) != NULL)
+	{
+		GSList	*iter;
+
+		dir_size_info.ops = g_list_remove(dir_size_info.ops, op);
+
+		op->func(NULL, op->user);
+
+		if(op->current)
+			dir_size_free(op->current);
+		for(iter = op->stack; iter != NULL; iter = g_slist_next(iter))
+			dir_size_free(iter->data);
+		g_slist_free(op->stack);
+		g_free(op);
+
+		/* If no more operations active, stop the idle function too. */
+		if(dir_size_info.ops == NULL)
+		{
+			gtk_idle_remove(dir_size_info.idle);
+			dir_size_info.idle = 0U;
+		}
+	}
+}
+
+/* ----------------------------------------------------------------------------------------- */
+
+/* 2003-11-29 -	Here's a strtok()-workalike. Almost. The arg <paths> should be either a string
+**		of colon-separated path components (e.g. "/usr/local:/home/emil:/root"), or NULL.
+**		The function fills in <component> with either the first component, or, when <paths>
+**		is NULL, with the _next_ component (relative to the last one). If this sounds confusing,
+**		read the man page for strtok(3) and associate freely from there. The main difference
+**		between this and strtok(3) is that the latter modifies the string being tokenized,
+**		while this function does not. The <component> buffer is assumed to have room for
+**		PATH_MAX bytes. Returns 0 if no more components were found, 1 otherwise.
+*/
+int fut_path_component(const gchar *paths, gchar *component)
+{
+	static const gchar	*src = NULL;
+	gchar			here, *dst = component;
+
+	if(paths != NULL)
+		src = paths;
+	else if(src == NULL)
+		return 0;
+
+	/* First, find the component, if processing a full multi-directory path. */
+	while(*src == ':')
+		src++;
+	for(; (here = *src) != '\0'; src++)
+	{
+		if(here == ':' && src[1] == G_DIR_SEPARATOR)	/* Colon followed by separator ends a path. */
+		{
+			src += 1;	/* Leave src at final separator for next time. */
+			break;
+		}
+		*dst++ = here;
+	}
+	*dst = '\0';
+	if(here == '\0')		/* End of path list reached? */
+		src = NULL;		/* Makes us stop next time. */
+	if(dst > component)
+		fut_interpolate(component);
+
+	return dst > component;
+}
+
+/* ----------------------------------------------------------------------------------------- */
+
 /* 1998-09-22 -	Evaluate (look up) any environment variable(s) used in <component>. Replaces
 **		the entire string with a new version, where all environment variable names
 **		have been replaced by the corresponding variable's value. Undefined variables
@@ -274,45 +580,55 @@ gint fut_dir_size_progress(MainInfo *min, const gchar *path, guint64 *bytes, FUC
 */
 static void eval_env(gchar *component)
 { 
-	GString	*temp;
+	GString	*temp = NULL;
 	gchar	*ptr, vname[256], *vptr, *vval;
 
-	if((temp = g_string_new(NULL)) != NULL)
+	for(ptr = component; *ptr != '\0'; ptr++)
 	{
-		for(ptr = component; *ptr != '\0'; ptr++)
+		if(*ptr == '$')		/* Environment variable? */
 		{
-			if(*ptr == '$')		/* Environment variable? */
+			const gchar	*start = ptr;
+
+			ptr++;
+			if(*ptr == '{')		/* Braced name? */
 			{
+				for(vptr = vname, ptr++; *ptr && *ptr != '}' &&
+					(gsize) (vptr - vname) < sizeof vname - 1;)
+				{
+					*vptr++ = *ptr++;
+				}
 				ptr++;
-				if(*ptr == '{')		/* Braced name? */
-				{
-					for(vptr = vname, ptr++; *ptr && *ptr != '}' &&
-						(gsize) (vptr - vname) < sizeof vname - 1;)
-					{
-						*vptr++ = *ptr++;
-					}
-					ptr++;
-				}
-				else
-				{
-					for(vptr = vname; *ptr && (isalnum((guchar) *ptr) || *ptr == '_') &&
-						(gsize) (vptr - vname) < sizeof vname - 1;)
-					{
-						*vptr++ = *ptr++;
-					}
-				}
-				*vptr = '\0';
-				if((vval = g_getenv(vname)) != NULL)
-				{
-					if(*vval == G_DIR_SEPARATOR)
-						g_string_truncate(temp, 0);
-					g_string_append(temp, vval);
-				}
-				ptr--;
 			}
 			else
+			{
+				for(vptr = vname; *ptr && (isalnum((guchar) *ptr) || *ptr == '_') &&
+					(gsize) (vptr - vname) < sizeof vname - 1;)
+				{
+					*vptr++ = *ptr++;
+				}
+			}
+			*vptr = '\0';
+			if((vval = g_getenv(vname)) != NULL)
+			{
+				if(temp == NULL)
+				{
+					temp = g_string_new(component);
+					g_string_truncate(temp, start - component);
+				}
+				if(*vval == G_DIR_SEPARATOR)
+					g_string_truncate(temp, 0);
+				g_string_append(temp, vval);
+			}
+			ptr--;
+		}
+		else
+		{
+			if(temp)
 				g_string_append_c(temp, *ptr);
 		}
+	}
+	if(temp)
+	{
 		stu_strncpy(component, temp->str, PATH_MAX);
 		g_string_free(temp, TRUE);
 	}
@@ -345,65 +661,24 @@ static void eval_home(gchar *component)
 	}
 }
 
-/* 1998-08-23 -	Here's a strtok()-workalike. Almost. The arg <paths> should be either a string
-**		of colon-separated path components (such as "/usr/local:/home/emil:/root"), or
-**		NULL. The function fills in <component> with either the first component, or, when
-**		<paths> is NULL, with the _next_ component (relative to the last one). If this sounds
-**		confusing, read the man page for strtok(3) and associate freely from there. The main
-**		difference between this and strtok(3) is that the latter modifies the string being
-**		tokenized, while this function does not. The <component> buffer is assumed to have
-**		room for PATH_MAX bytes.
-**		Returns 0 if no more components were found, 1 otherwise.
-** 1998-08-24 -	Now supports use of environment variable lookups as a "prefix" in a path component,
-**		thus allowing things like "$HOME/.gentoo/icons". Neat.
-** 1998-08-30 -	Added support for a 'base' path, which is global to this module (set to the current
-**		working dir by a call to fut_initialize()). The base is prepended to any relative
-**		path component, thus making this more predictable and generally nicer.
-** 1998-09-22 -	Rewrote the support for environment variables; now you can embed them and stuff.
-*/
-gint fut_path_component(const gchar *paths, gchar *component)
+/* 2003-11-29 -	Interpolate special things (like $VARIABLES and ~homedirs) into a path. */
+void fut_interpolate(gchar *buffer)
 {
-	static const gchar	*src = NULL;
-	gchar			here, *dst = component;
-	gint			comp_len, use_env = 0;
-
-	if(paths != NULL)
-		src = paths;
-	else if(src == NULL)
-		return 0;
-
-	while(*src == ':')			/* Ignore empty components. */
-		src++;
-	for(; (here = *src) != '\0'; src++)
+	eval_env(buffer);
+	if(buffer[0] == '~')
+		eval_home(buffer);
+	else if(buffer[0] != G_DIR_SEPARATOR && base_path[0] == G_DIR_SEPARATOR)
 	{
-		if(here == ':')
-			break;
-		else
-			*dst++ = here;
-		use_env |= (here == '$');
-	}
-	*dst = '\0';
-	if(here == '\0')			/* End of path list reached? */
-		src = NULL;			/* Makes us stop next time. */
-	comp_len = dst - component;
+		gint	len = strlen(buffer);
 
-	if(use_env)				/* Do we need to look up environment variables? */
-		eval_env(component);
-	if(component[0] == '~')			/* Do we need to look up a home directory? */
-		eval_home(component);
-	else if(component[0] != G_DIR_SEPARATOR)	/* Relative path? */
-	{
-		if(base_path[0] == G_DIR_SEPARATOR)	/* Yes, absolute base available? */
+		/* Make relative path absolute, if buffer space allows it. */
+		if(len + base_len + 2 < PATH_MAX)
 		{
-			if(comp_len + base_len + 2 < PATH_MAX)	/* Room to prepend it? */
-			{
-				memmove(component + base_len + 1, component, comp_len + 1);
-				memcpy(component, base_path, base_len);
-				component[base_len] = G_DIR_SEPARATOR;
-			}
+			memmove(buffer + base_len + 1, buffer, len + 1);
+			memcpy(buffer, base_path, base_len);
+			buffer[base_len] = G_DIR_SEPARATOR;
 		}
 	}
-	return dst > component;
 }
 
 /* ----------------------------------------------------------------------------------------- */
@@ -741,3 +1016,9 @@ void fut_initialize(void)
 	}
 	base_len = strlen(base_path);
 }
+
+/* 2003-11-10 -	Change back to the directory saved in fut_initialize(). Might help profiling. */
+void fut_finalize(void)
+{
+	fut_cd(base_path, NULL, 0U);
+}
