@@ -30,7 +30,6 @@
 #include "cmdseq.h"
 #include "children.h"
 #include "dpformat.h"
-#include "options.h"
 #include "mount.h"
 #include "keyboard.h"
 #include "controls.h"
@@ -86,7 +85,14 @@ static void init_pane(DirPane *dp, gint index)
 	dp->complete.list = NULL;
 	dp->complete.iter = NULL;
 	dp->dir.row = NULL;
-	dp->dir.auxbuf = NULL;
+	dp->dir.row_alloc = 0;
+	dp->dir.num_rows  = 0;
+	dp->dir.name = NULL;
+	dp->dir.name_alloc = 0;
+	dp->dir.name_use   = 0;
+	dp->dir.stat = NULL;
+	dp->dir.stat_alloc = 0;
+	dp->dir.stat_use   = 0;
 	dp->hist = dph_dirhistory_new();
 	dp->dir.path[0] = '\0';
 }
@@ -393,7 +399,6 @@ static void evt_key_press(GtkWidget *wid, GdkEventKey *evt, gpointer user)
 
 		dlg = dlg_dialog_sync_new(NULL, "Hello", "_A|_B");
 		ret = dlg_dialog_sync_wait(dlg);
-		printf("You answered %d\n", ret);
 		dlg_dialog_sync_destroy(dlg);
 	}
 	else if(evt->keyval == GDK_1)
@@ -401,7 +406,6 @@ static void evt_key_press(GtkWidget *wid, GdkEventKey *evt, gpointer user)
 		min->cfg.dp_paning.mode++;
 		if(min->cfg.dp_paning.mode > DPSPLIT_ABS_RIGHT)
 			min->cfg.dp_paning.mode = DPSPLIT_FREE;
-		printf("Pane split mode: %d\n", min->cfg.dp_paning.mode);
 	}
 	else if(evt->keyval == GDK_0)
 		gtk_widget_hide(min->gui->pane[0].scwin);
@@ -435,39 +439,54 @@ static void enter_initial_dirs(MainInfo *min, const gchar *dirleft, const gchar
 		if(dir && *dir)
 		{
 			csq_execute(min, cmd[i]);
-			csq_execute_format(min, "DirEnter 'dir=%s'", dir);
+			csq_execute_format(min, "DirEnter 'dir=%s'", stu_escape(dir));
 		}
 	}
 }
 
+/* 2003-11-25 -	An idle handler that should only trigger once, to run commands from command line. */
+static gint evt_idle_run(gpointer user)
+{
+	MainInfo	*min = user;
+	const GList	*list;
+
+	for(list = opt_arg_get("run", min->opt, min->opt_num); list != NULL; list = g_list_next(list))
+		csq_execute(min, list->data);
+	return FALSE;		/* Causes the handler to be removed, once really is enough. */
+}
+
 int main(int argc, char *argv[])
 {
 	enum { OPT_VERSION = 1 << 0, OPT_ROOTOK = 1 << 1, OPT_NORC = 1 << 2, OPT_NOGTKRC = 1 << 3, OPT_NOFAM = 1 << 4 };
-	OptDesc	od[] = { { '\0', "version",  OPT_ARG_NONE, NULL, OPT_VERSION,	N_("Report the version to standard output, and exit.") },
-			 { '\0', "root-ok",  OPT_ARG_NONE, NULL, OPT_ROOTOK,	N_("Allows gentoo to be run by the root user. Could be dangerous.") },
-			 { '\0', "no-rc",    OPT_ARG_NONE, NULL, OPT_NORC,	N_("Do not load the ~/.gentoorc configuration file; instead, use default values.") },
-			 { '\0', "no-gtkrc", OPT_ARG_NONE, NULL, OPT_NOGTKRC,	N_("Do not load the ~/.gentoogtkrc GTK+ configuration file; instead, use system defaults.") },
+	OptDesc	od[] = { { '\0', "version",  OPT_ARG_NONE, OPT_VERSION,	N_("Report the version to standard output, and exit.") },
+			 { '\0', "root-ok",  OPT_ARG_NONE, OPT_ROOTOK,	N_("Allows gentoo to be run by the root user. Could be dangerous.") },
+			 { '\0', "no-rc",    OPT_ARG_NONE, OPT_NORC,	N_("Do not load the ~/.gentoorc configuration file; instead, use default values.") },
+			 { '\0', "no-gtkrc", OPT_ARG_NONE, OPT_NOGTKRC,	N_("Do not load the ~/.gentoogtkrc GTK+ configuration file; instead, use system defaults.") },
 #if defined HAVE_FAM
-			 { '\0', "no-fam",   OPT_ARG_NONE, NULL, OPT_NOFAM,	N_("Do not use FAM to automatically detect changes in viewed directories.") },
+			 { '\0', "no-fam",   OPT_ARG_NONE, OPT_NOFAM,	N_("Do not use FAM to automatically detect changes in viewed directories.") },
 #endif
-			 { '1',  "left",     OPT_ARG_REQ,  NULL, 0U, N_("Use ARG as path for the left directory pane. Overrides default (and history).") },
-			 { '2',  "right",    OPT_ARG_REQ,  NULL, 0U, N_("Use ARG as path for the right directory pane. Overrides default (and history).") }
+			 { '1',  "left",     OPT_ARG_REQ,      0U, N_("Use ARG as path for the left directory pane. Overrides default (and history).") },
+			 { '2',  "right",    OPT_ARG_REQ,      0U, N_("Use ARG as path for the right directory pane. Overrides default (and history).") },
+			 { 'r',  "run",	     OPT_ARG_REQ_LIST, 0U, N_("Run ARG, a gentoo command. Done before user interaction allowed, but after configuration "
+								      "file has been read in. Can be used many times to run several commands in sequence.") }
 			};
 	guint32		opt_flags, cfg_flags = 0U;
 	GtkWidget	*box;
-	static MainInfo	min;
-
-	memset(&min, 0, sizeof min);
-	min.gui = NULL;
+	static MainInfo	min = { 0 };
 
+	/* Make options available to idle callback that runs commands from invocation. A bit ugly. */
+	min.opt     = od;
+	min.opt_num = sizeof od / sizeof *od;
+	
 #if defined ENABLE_NLS	
 	setlocale(LC_ALL, "");
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
 	for(opt_flags = 0; opt_flags < sizeof od / sizeof *od; opt_flags++)
 		od[opt_flags].help = _(od[opt_flags].help);
+	gtk_set_locale();
 #endif
-	opt_flags = opt_parse(&argc, argv, od, sizeof od / sizeof od[0]);
+	opt_flags = opt_parse(&argc, argv, od, sizeof od / sizeof *od);
 	if(argc < 1)
 		return argc < -1 ? EXIT_FAILURE : EXIT_SUCCESS;
 	if(opt_flags & OPT_VERSION)
@@ -558,12 +577,12 @@ int main(int argc, char *argv[])
 		g_snprintf(welcome, sizeof welcome, _("gentoo v%s by Emil Brink <emil@obsession.se>"), VERSION);
 		gtk_label_set_text(GTK_LABEL(min.gui->top), welcome);
 
-		if(cfg_flags & CLDF_NONE_FOUND)
-			csq_execute(&min, "About");
+		gtk_idle_add(evt_idle_run, &min);
 		gtk_main();
 	}
 	fam_shutdown(&min);
 	mnt_shutdown(&min);
+	fut_finalize();
 
 	return EXIT_SUCCESS;
 }
