@@ -118,6 +118,7 @@ typedef struct CmdSeq		CmdSeq;
 #include "dirhistory.h"
 #include "window.h"
 #include "menus.h"
+#include "options.h"
 
 #define	KEY_NAME_SIZE	(32)
 
@@ -384,6 +385,8 @@ typedef struct {			/* Holds all configuration info. */
 typedef struct GuiInfo	GuiInfo;
 
 struct MainInfo {				/* gentoo's single most central data structure. */
+	OptDesc		*opt;
+	gsize		opt_num;
 	CfgInfo		cfg;
 	GuiInfo		*gui;
 	IconInfo	*ico;
@@ -431,7 +434,9 @@ typedef struct {
 #define	DP_SEL_TYPE(s)		(DP_ROW_TYPE(DP_SEL_ROW(s)))
 #define	DP_SEL_INDEX(p,s)	(DP_ROW_INDEX((p), DP_SEL_ROW(s)))
 
-#define	DPRF_HAS_SIZE		(1<<0)		/* Set when a row has a "real" size (not set for directories unless GetSize:d). */
+enum {	DPRF_HAS_SIZE = 1 << 0,			/* Set when a row has a "real" size (not set for directories unless GetSize:d). */
+	DPRF_LINK_EXISTS = 1 << 1		/* Set when a symlink's target exists. */
+};
 
 typedef struct {			/* Representation of a single line in a directory (a file, typically). */
 	const gchar	*dr_name;		/* Name, points into big block in DirContents. */
@@ -462,7 +467,11 @@ typedef struct {
 	guint		tot_dirs, tot_files;	/* Total number of entries in directory. */
 	guint64		tot_bytes, tot_blocks;	/* Sum of all sizes (in bytes & blocks). */
 	DirRow		*row;			/* A bunch of rows. */
-	gchar		*auxbuf;		/* This holds auxillary info (names, link names, and link stats). */
+	gsize		row_alloc;		/* Number of allocated rows. */
+	gchar		*name;			/* This holds name info (filenames and link names). */
+	gsize		name_alloc, name_use;	/* Allocated and used name bytes. */
+	struct stat	*stat;			/* Info about link targets. */
+	gsize		stat_alloc, stat_use;	/* Allocated and used link stat structs. */
 	SelInfo		sel;
 	FsInfo		fs;
 } DirContents;
@@ -473,6 +482,7 @@ struct DirPane {
 	MainInfo	*main;		/* Handy to have around. */
 	guint		index;		/* Index of *this* pane, in the grand scheme of things. */
 	GtkWidget	*vbox;
+	GtkWidget	*notebook;	/* Notebook holding (normally) path widgetry. */
 	GtkWidget	*path;		/* Combo widget showing path, and path history. */
 	PathComplete	complete;	/* Information for completing the path. */
 	GtkWidget	*hide;		/* A toggle button showing (and controlling) hide status. */
