@@ -61,7 +61,7 @@ GtkWidget * gui_details_button_new(GdkWindow *win)
 /* 1998-07-30 -	Build a group of radio buttons with the given labels.
 ** 1999-05-15 -	Now also sets the GTK+ object user data to the index of each button.
 */
-GSList * gui_radio_group_new(guint num, const gchar *label[], GtkWidget *but[])
+GSList * gui_radio_group_new(guint num, const gchar **label, GtkWidget *but[])
 {
 	GSList	*list = NULL;
 	guint	i;
