@@ -5,7 +5,7 @@
 
 extern GtkWidget *	gui_arrow_button_new(GtkArrowType at);
 extern GtkWidget *	gui_details_button_new(GdkWindow *win);
-extern GSList *		gui_radio_group_new(guint num, const gchar *label[], GtkWidget *but[]);
+extern GSList *		gui_radio_group_new(guint num, const gchar **label, GtkWidget *but[]);
 
 extern gpointer		gui_progress_begin(const gchar *body, const gchar *button);
 extern gboolean		gui_progress_update(gpointer anchor, gfloat value, const gchar *obj);
