@@ -30,25 +30,25 @@ typedef struct {
 /* 1998-07-13 -	This is used to compare hash elements as they are insorted :) into the
 **		singly linked list.
 */
-static gint compare(gpointer a, gpointer b)
+static gint compare(gconstpointer a, gconstpointer b)
 {
-	if((a == NULL) || (b == NULL))
+	if(!a || !b)
 		return 1;
-	return strcmp((char *) a, (char *) b);
+	return strcmp(a, b);
 }
 
 /* 1998-07-13 -	Sort the hash elements into a singly linked list. */
-static void populate_slist(gpointer key, gpointer value, gpointer data)
+static void populate_slist(gpointer key, gpointer value, gpointer user)
 {
-	HDlgInfo	*hdl = (HDlgInfo *) data;
+	HDlgInfo	*hdl = user;
 
-	hdl->slist = g_slist_insert_sorted(hdl->slist, key, (GCompareFunc) compare);
+	hdl->slist = g_slist_insert_sorted(hdl->slist, key, compare);
 }
 
 /* 1998-07-13 -	Populate clist with elements from hash table, via singly linked list. */
 static void populate_clist(gpointer data, gpointer user)
 {
-	HDlgInfo	*hdl = (HDlgInfo *) user;
+	HDlgInfo	*hdl = user;
 	gchar		*line[1];
 	gint		row;
 
@@ -63,11 +63,11 @@ static void populate_clist(gpointer data, gpointer user)
 /* ----------------------------------------------------------------------------------------- */
 
 /* 1998-07-14 -	This gets called when user clicks on rows of the clist. */
-static void evt_clist_select(GtkWidget *wid, gint row, gint col, GdkEventButton *evt)
+static void evt_clist_select(GtkWidget *wid, gint row, gint col, GdkEventButton *evt, gpointer user)
 {
-	HDlgInfo	*hdl = (HDlgInfo *) gtk_object_get_user_data(GTK_OBJECT(wid));
+	HDlgInfo	*hdl = user;
 
-	hdl->row_data = gtk_clist_get_row_data(GTK_CLIST(hdl->clist), row);
+	hdl->row_data = gtk_clist_get_row_data(GTK_CLIST(wid), row);
 
 	if((evt != NULL) && (evt->button == 1) && (evt->type == GDK_2BUTTON_PRESS))
 		dlg_dialog_sync_close(hdl->dlg, 0);
@@ -86,20 +86,18 @@ const gchar * hdl_dialog_sync_new_wait(GHashTable *hash, const gchar *title)
 	scwin = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scwin), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
 	hdl.clist = gtk_clist_new(1);
-	gtk_object_set_user_data(GTK_OBJECT(hdl.clist), (gpointer) &hdl);
-	gtk_signal_connect(GTK_OBJECT(hdl.clist), "select_row", GTK_SIGNAL_FUNC(evt_clist_select), NULL);
+	gtk_signal_connect(GTK_OBJECT(hdl.clist), "select_row", GTK_SIGNAL_FUNC(evt_clist_select), &hdl);
 	gtk_clist_set_selection_mode(GTK_CLIST(hdl.clist), GTK_SELECTION_BROWSE);
 	gtk_widget_set_usize(hdl.clist, 200, 384);
 	gtk_container_add(GTK_CONTAINER(scwin), hdl.clist);
-	gtk_widget_show(hdl.clist);
 	hdl.slist = NULL;
 	g_hash_table_foreach(hash, populate_slist, &hdl);
 	g_slist_foreach(hdl.slist, populate_clist, &hdl);
 	g_slist_free(hdl.slist);
 
-	hdl.dlg  = dlg_dialog_sync_new(scwin, title, _("OK|Cancel"));
+	hdl.dlg  = dlg_dialog_sync_new(scwin, title, NULL);
 	gtk_clist_select_row(GTK_CLIST(hdl.clist), 0, 0);
-	if(dlg_dialog_sync_wait(hdl.dlg) == 0)
+	if(dlg_dialog_sync_wait(hdl.dlg) == DLG_POSITIVE)
 		return hdl.row_data;
 	return NULL;
 }
