@@ -61,7 +61,6 @@ const gchar * idl_dialog_sync_new_wait(MainInfo *min, const gchar *path, const g
 	gtk_clist_set_column_auto_resize(GTK_CLIST(clist), 0, TRUE);
 	gtk_signal_connect(GTK_OBJECT(clist), "select_row", GTK_SIGNAL_FUNC(evt_clist_select_row), selected);
 	gtk_container_add(GTK_CONTAINER(scwin), clist);
-	gtk_widget_show(clist);
 
 	for(here = icons, position = 0U; here != NULL; here = g_list_next(here), position++)
 	{
@@ -89,10 +88,8 @@ const gchar * idl_dialog_sync_new_wait(MainInfo *min, const gchar *path, const g
 		Dialog	*dlg;
 
 		gtk_clist_select_row(GTK_CLIST(clist), selrow, -1);
-/*		gtk_clist_moveto(GTK_CLIST(clist), selrow, -1, 0.5, 0.0);*/
-		gtk_widget_show(scwin);
-		dlg = dlg_dialog_sync_new(scwin, title, _("OK|Cancel"));
-		if(dlg_dialog_sync_wait(dlg) != 0)
+		dlg = dlg_dialog_sync_new(scwin, title, NULL);
+		if(dlg_dialog_sync_wait(dlg) != DLG_POSITIVE)
 			terminate = TRUE;
 		dlg_dialog_sync_destroy(dlg);
 	}
