@@ -192,7 +192,7 @@ static gboolean action_on_list(GSList *actions, const gchar *action)
 */
 static gboolean action_in_intersection(GSList *styles, GSList *current, const gchar *action)
 {
-	GSList	*siter;
+	const GSList	*siter;
 
 	for(siter = styles; siter != NULL; siter = g_slist_next(siter))
 	{
@@ -210,7 +210,7 @@ static void evt_menu_action_activate(GtkWidget *wid, gpointer user)
 	MainInfo	*min;
 
 	min = gtk_object_get_user_data(GTK_OBJECT(wid));
-	csq_execute_format(min, "FileAction action='%s'", (const gchar *) user);
+	csq_execute_format(min, "FileAction action='%s'", user);
 }
 
 /* 2000-12-03 -	Build the Action submenu. It contains names of the actions available in
@@ -218,8 +218,8 @@ static void evt_menu_action_activate(GtkWidget *wid, gpointer user)
 */
 static void menu_build_action(MainInfo *min, GtkWidget *menu)
 {
-	DirPane		*dp = min->gui->cur_pane;
-	GSList		*slist;
+	DirPane	*dp = min->gui->cur_pane;
+	GSList	*slist;
 
 	if((slist = dp_get_selection(dp)) != NULL)
 	{
@@ -270,7 +270,7 @@ static void menu_build_action(MainInfo *min, GtkWidget *menu)
 	{
 		GtkWidget	*item;
 
-		item = gtk_menu_item_new_with_label("(No Selection)");
+		item = gtk_menu_item_new_with_label(_("(No Selection)"));
 		gtk_widget_set_sensitive(item, FALSE);
 		gtk_menu_append(GTK_MENU(menu), item);
 		gtk_widget_show(item);
@@ -305,9 +305,9 @@ static void evt_menu_activate(GtkWidget *wid, gpointer user)
 {
 	MainInfo	*min;
 
-/*	fprintf(stderr, "Now in menu activation code, wid=%p, cmdseq=\"%s\"\n", wid, (const gchar *) user);*/
+/*	fprintf(stderr, "Now in menu activation code, wid=%p, cmdseq=\"%s\"\n", wid, user);*/
 	min = gtk_object_get_user_data(GTK_OBJECT(wid));	
-	csq_execute(min, (const gchar *) user);
+	csq_execute(min, user);
 }
 
 static void evt_menu_show(GtkWidget *wid, gpointer user)
@@ -447,7 +447,7 @@ static void evt_popup_hide(GtkWidget *wid, gpointer user)
 	gtk_signal_emit_by_name(GTK_OBJECT(wid), "destroy_event", NULL, &ret);
 }
 
-gboolean mnu_menu_popup(const MenuInfo *mi, const gchar *name, guint button, guint32 activate_time)
+gboolean mnu_menu_popup(const MenuInfo *mi, const gchar *name, GtkMenuPositionFunc pos, gpointer data, guint button, guint32 activate_time)
 {
 	GtkWidget	*menu;
 
@@ -455,7 +455,7 @@ gboolean mnu_menu_popup(const MenuInfo *mi, const gchar *name, guint button, gui
 	{
 		gtk_signal_connect_after(GTK_OBJECT(menu), "hide", GTK_SIGNAL_FUNC(evt_popup_hide), NULL);
 		gtk_signal_connect(GTK_OBJECT(menu), "destroy_event", GTK_SIGNAL_FUNC(evt_popup_destroy), NULL);
-		gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, button, activate_time);
+		gtk_menu_popup(GTK_MENU(menu), NULL, NULL, pos, data, button, activate_time);
 /*		fprintf(stderr, "Popping up menu at %p\n", menu);*/
 	}
 	return menu != NULL;
@@ -483,10 +483,10 @@ MenuInfo * mnu_menuinfo_new_default(MainInfo *min)
 	mi = mnu_menuinfo_new(min);
 
 	menu = mnu_menu_new(mi, "SelectMenu");
-	mnu_menu_append_simple(menu, "All",       "SelectAll");
-	mnu_menu_append_simple(menu, "None",      "SelectNone");
-	mnu_menu_append_simple(menu, "Toggle",    "SelectToggle");
-	mnu_menu_append_simple(menu, "RegExp...", "SelectRE");
+	mnu_menu_append_simple(menu, _("All"),       "SelectAll");
+	mnu_menu_append_simple(menu, _("None"),      "SelectNone");
+	mnu_menu_append_simple(menu, _("Toggle"),    "SelectToggle");
+	mnu_menu_append_simple(menu, _("RegExp..."), "SelectRE");
 
 /*	menu = mnu_menu_new(mi, "PointlessComplexityMenu");
 	mnu_menu_append_submenu(menu, "Action2", "<ActionMenu>");
@@ -497,16 +497,16 @@ MenuInfo * mnu_menuinfo_new_default(MainInfo *min)
 	mnu_menu_append_submenu(menu, "Action2", "<ActionMenu>");
 */
 	menu = mnu_menu_new(mi, "DirPaneMenu");
-	mnu_menu_append_simple(menu, "Parent", "DirParent");
-	mnu_menu_append_simple(menu, "Other",  "DirFromOther");
-	mnu_menu_append_simple(menu, "Rescan", "DirRescan");
-	mnu_menu_append_submenu(menu, "Select", "SelectMenu");
+	mnu_menu_append_simple(menu, _("Parent"), "DirParent");
+	mnu_menu_append_simple(menu, _("Other"),  "DirFromOther");
+	mnu_menu_append_simple(menu, _("Rescan"), "DirRescan");
+	mnu_menu_append_submenu(menu, _("Select"), "SelectMenu");
 	mnu_menu_append_separator(menu);
-	mnu_menu_append_submenu(menu, "Action", "<ActionMenu>");
+	mnu_menu_append_submenu(menu, _("Action"), "<ActionMenu>");
 /*	mnu_menu_append_submenu(menu, "Time", "<TimeMenu>");*/
-	mnu_menu_append_simple(menu, "Run...", "Run");
+	mnu_menu_append_simple(menu, _("Run..."), "Run");
 	mnu_menu_append_separator(menu);
-	mnu_menu_append_simple(menu, "Configure...", "Configure");
+	mnu_menu_append_simple(menu, _("Configure..."), "Configure");
 /*	mnu_menu_append_submenu(menu, "Pointless", "PointlessComplexityMenu");*/
 
 /*	menu = mnu_menu_new(mi, "ShortcutMenu");
@@ -529,8 +529,6 @@ MenuInfo * mnu_menuinfo_copy(const MenuInfo *mi)
 	nmi = mnu_menuinfo_new(mi->min);
 	g_hash_table_foreach(mi->menus, copy_menu, nmi);
 
-	printf("Uhu, copied %u menus\n", g_hash_table_size(nmi->menus));
-
 	return nmi;
 }
 
@@ -584,7 +582,7 @@ const gchar * mnu_menuinfo_select(const MenuInfo *mi)
 	gtk_container_add(GTK_CONTAINER(scwin), clist);
 	gtk_widget_set_usize(scwin, 200, 384);
 
-	dlg = dlg_dialog_sync_new(scwin, _("Select Menu"), _("OK|Cancel"));
+	dlg = dlg_dialog_sync_new(scwin, _("Select Menu"), NULL);
 	choice = dlg_dialog_sync_wait(dlg);
 	dlg_dialog_sync_destroy(dlg);
 
