@@ -61,7 +61,7 @@ static void show_usage(const OptDesc *desc, gsize num_desc)
 					ns[0] ? "-" : "", ns, ns[0] ? " [ARG]" : "", ns[0] && nl ? ", " : "",
 					nl ? "--" : "", nl ? nl : "", nl ? "[=ARG]" : "");
 			if(desc[i].arg)
-				g_snprintf(opt + len, sizeof opt - len, " (default: %s)", desc[i].arg);
+				g_snprintf(opt + len, sizeof opt - len, " (default: %s)", (const gchar *) desc[i].arg);
 		}
 		else
 			g_snprintf(opt, sizeof opt, "%s%s%s%s%s%s%s",
@@ -119,6 +119,31 @@ static const gchar * short_get_arg(gchar **argv, gint index, const gchar *ptr)
 	return get_next_nonoption_word(argv, index);
 }
 
+/* Store argument, taking care to build list if needed. */
+static gboolean arg_store(const gchar *name, OptDesc *desc, const gchar *arg, gboolean is_short)
+{
+	if(arg)
+	{
+		if(desc->arg_mode == OPT_ARG_REQ_LIST)
+			desc->arg = g_list_append(desc->arg, (gpointer) arg);
+		else
+			desc->arg = (gpointer) arg;
+		return TRUE;
+	}
+	else if(desc->arg_mode == OPT_ARG_REQ || desc->arg_mode == OPT_ARG_REQ_LIST)
+	{
+		gchar	opt[64];
+
+		if(is_short)
+			g_snprintf(opt, sizeof opt, "-%c", desc->name_short);
+		else
+			g_snprintf(opt, sizeof opt, "--%s", desc->name_long);
+		fprintf(stderr, "%s: Missing argument to option %s; try -h or --help for usage\n", name, opt);
+		return FALSE;
+	}
+	return FALSE;
+}
+
 /* Handle a (bunch of) short option(s). */
 static guint32 handle_short_options(gchar **argv, gint index, OptDesc *desc, gsize num_desc, guint32 *flags)
 {
@@ -140,13 +165,8 @@ static guint32 handle_short_options(gchar **argv, gint index, OptDesc *desc, gsi
 				const gchar	*arg;
 
 				arg = short_get_arg(argv, index, ptr);
-				if(arg)
-					desc[i].arg = arg;
-				else if(desc[i].arg_mode == OPT_ARG_REQ)
-				{
-					fprintf(stderr, "%s: Missing argument to option -%c; try -h or --help for usage\n", argv[0], *ptr);
+				if(!arg_store(argv[0], desc + i, arg, TRUE))
 					return 0;
-				}
 				break;
 			}
 		}
@@ -184,13 +204,8 @@ static gint handle_long_option(gchar **argv, gint index, OptDesc *desc, gsize nu
 			if(desc[i].arg_mode)
 			{
 				arg = arg ? arg : get_next_nonoption_word(argv, index);
-				if(arg)
-					desc[i].arg = arg;
-				else if(desc[i].arg_mode == OPT_ARG_REQ)
-				{
-					fprintf(stderr, "%s: Missing argument to option --%s\n", argv[0], opt);
+				if(!arg_store(argv[0], desc + i, arg, FALSE))
 					return 0;
-				}
 			}
 			else if(arg != NULL)
 				fprintf(stderr, "%s: Superfluous argument to option --%s: \"%s\"", argv[0], desc[i].name_long, arg);
@@ -250,7 +265,7 @@ guint32 opt_parse(gint *argc, gchar **argv, OptDesc *desc, gsize num_desc)
 /* Return the argument for option <name>, if found. If <name> is single-character, it's
 ** interpreted as a short name; otherwise it's long. No leading hyphens, please.
 */
-const gchar * opt_arg_get(const gchar *name, const OptDesc *desc, gsize num_desc)
+const gpointer opt_arg_get(const gchar *name, const OptDesc *desc, gsize num_desc)
 {
 	gsize	i;
 
