@@ -5,10 +5,12 @@
 ** 1999-03-13 -	Changes for the new dialog module.
 ** 1999-06-12 -	Added "Skip All" button.
 ** 2001-01-01 -	Added simplistic (and most likely insufficient) recursion-detection.
+** 2003-11-23 -	Now tells caller if collision was with directory or file.
 */
 
 #include "gentoo.h"
 #include "dialog.h"
+#include "errors.h"
 #include "strutil.h"
 #include "fileutil.h"
 #include "overwrite.h"
@@ -52,15 +54,6 @@ void ovw_overwrite_begin(MainInfo *min, const gchar *fmt, guint32 flags)
 
 /* ----------------------------------------------------------------------------------------- */
 
-/* 1998-12-20 -	Get a date for file described by <st>, to be shown in dialog, as a struct tm.
-**		Currently simply returns the "changed"-time, since I feel it's maybe the most
-**		useful.
-*/
-static struct tm * get_date(struct stat *st)
-{
-	return localtime(&st->st_ctime);
-}
-
 static void get_info(MainInfo *min, gchar *buf, gsize buf_size, const gchar *old_file, const gchar *new_file)
 {
 	gchar		o_date[64], n_date[64];
@@ -68,12 +61,12 @@ static void get_info(MainInfo *min, gchar *buf, gsize buf_size, const gchar *old
 
 	if((stat(old_file, &o_st) == 0) && (stat(new_file, &n_st) == 0))
 	{
-		strftime(o_date, sizeof o_date, min->cfg.opt_overwrite.datefmt, get_date(&o_st));
-		strftime(n_date, sizeof n_date, min->cfg.opt_overwrite.datefmt, get_date(&n_st));
-		g_snprintf(buf, (gulong) buf_size, _("\nOld: %u bytes, changed on %s.\n"
-						   "New: %u bytes, changed on %s."),
-						   (guint) o_st.st_size, o_date,
-						   (guint) n_st.st_size, n_date);
+		strftime(o_date, sizeof o_date, min->cfg.opt_overwrite.datefmt, localtime(&o_st.st_ctime));
+		strftime(n_date, sizeof n_date, min->cfg.opt_overwrite.datefmt, localtime(&n_st.st_ctime));
+		g_snprintf(buf, buf_size, _("\nOld: %Lu bytes, changed on %s.\n"
+						   "New: %Lu bytes, changed on %s."),
+						   o_st.st_size, o_date,
+						   n_st.st_size, n_date);
 	}
 }
 
@@ -92,33 +85,38 @@ static gboolean check_recursion(const gchar *f_new, const gchar *f_old)
 	return FALSE;
 }
 
-/* 1998-09-17 -	Inform the overwrite system that the file whose complete (absolute) name is
-**		in <file> is about to be modified in a way considered dangerous by the code
-**		doing it. Return values:
-**		OVW_SKIP	Skip this file, but continue with the overall operation.
-**		OVW_PROCEED	Perform operation on this file, then go on to the next.
-**		OVW_CANCEL	Abort the entire operation.
-** 1998-12-20 -	Added argument <new_file>, which gives the name of the file with which we are
-**		about to overwrite <old_file>, when possible. In other cases, it's NULL.
+static OvwRes proceed(const struct stat *stbuf)
+{
+	return S_ISDIR(stbuf->st_mode) ? OVW_PROCEED_DIR : OVW_PROCEED_FILE;
+}
+
+/* 2003-11-23 -	Inform the overwrite system that the file whose complete (absolute) name is
+**		in <old_file> is about to be overwritten by <new_file> (if given). Returns:
+**		OVW_SKIP	 Skip this file, but continue with the overall operation.
+**		OVW_PROCEED	 No collision, proceed.
+**		OVW_PROCEED_FILE Collision with file, but proceed anyway.
+**		OVW_PROCEED_DIR	 Collision with directory, but proceed anyway.
+**		OVW_CANCEL	 Abort the entire operation.
 */
 OvwRes ovw_overwrite_file(MainInfo *min, const gchar *old_file, const gchar *new_file)
 {
-	gchar	buf[PATH_MAX + 2048];
-	gint	dlg_res, len;
-
 	if(ovw_info.level)
 	{
+		gchar		buf[PATH_MAX + 2048];
+		gint		dlg_res, len;
+		struct stat	stbuf;
+
 		if(!(ovw_info.flags & OVWF_NO_RECURSE_TEST) && check_recursion(new_file, old_file))
 		{
 			errno = EINVAL;
 			return OVW_CANCEL;
 		}
 
-		if(ovw_info.do_all)			/* Already answered "All"? */
-			return OVW_PROCEED;
-
-		if(fut_exists(old_file))		/* Does the file exist? */
+		err_clear(min);
+		if(lstat(old_file, &stbuf) == 0)
 		{
+			if(ovw_info.do_all)		/* Already answered "All"? */
+				return proceed(&stbuf);
 			if(ovw_info.skip_all)		/* Already answered "Skip All"? */
 				return OVW_SKIP;
 
@@ -132,10 +130,10 @@ OvwRes ovw_overwrite_file(MainInfo *min, const gchar *old_file, const gchar *new
 			switch(dlg_res)
 			{
 				case DLG_POSITIVE:	/* OK ? */
-					return OVW_PROCEED;
+					return proceed(&stbuf);
 				case 1:			/* All? */
 					ovw_info.do_all = TRUE;
-					return OVW_PROCEED;
+					return proceed(&stbuf);
 				case 2:			/* Skip? */
 					return OVW_SKIP;
 				case 3:
@@ -145,6 +143,7 @@ OvwRes ovw_overwrite_file(MainInfo *min, const gchar *old_file, const gchar *new
 					return OVW_CANCEL;
 			}
 		}
+		err_clear(min);			/* Don't let EEXIST leak out. */
 		return OVW_PROCEED;
 	}
 	return OVW_CANCEL;
