@@ -32,10 +32,11 @@ static struct {
 	guint		tot_pos;	/* Index of current item. */
 	guint64		byte_tot;	/* Total bytes in operation. */
 	guint64		byte_pos;	/* Position, in bytes. */
-	guint32		item_size;	/* Size of current item being processed. */
-	guint32		item_pos;	/* Position in the item. */
-	struct timeval	time_begin;	/* The time when pgs_progress_begin() was called. */
+	off_t		item_size;	/* Size of current item being processed. */
+	off_t		item_pos;	/* Position in the item. */
+	GTimeVal	time_begin;	/* The time when pgs_progress_begin() was called. */
 	guint		delay_time;	/* Delay until display, in microseconds (!). */
+	guint		last_secs;	/* Elapsed seconds last time we displayed ETA and stuff. */
 
 	MainInfo	*min;
 	GtkWidget	*dlg;
@@ -48,7 +49,9 @@ static struct {
 	GtkWidget	*item;
 	GtkWidget	*item_pgs;
 	GtkWidget	*item_high;
-} pgs_info = { 0 };
+
+	GtkWidget	*eta;		/* Label for elapsed time, speed, and ETA. */
+} pgs_info = { 0U };
 
 /* ----------------------------------------------------------------------------------------- */
 
@@ -62,7 +65,7 @@ static guint count_toplevel_selected(MainInfo *min, guint64 *size)
 	guint	num = 0;
 
 	if(size != NULL)
-		*size = 0;
+		*size = 0U;
 
 	for(iter = slist = dp_get_selection(dp); iter != NULL; iter = g_slist_next(iter), num++)
 	{
@@ -77,14 +80,14 @@ static guint count_toplevel_selected(MainInfo *min, guint64 *size)
 ** 1998-10-01 -	Added size support.
 ** 1998-12-20 -	Now uses the fut_dir_size() function for the recursive scanning.
 */
-static int count_recursive_selected(MainInfo *min, guint64 *size)
+static guint count_recursive_selected(MainInfo *min, guint64 *size)
 {
 	DirPane	*dp = min->gui->cur_pane;
 	GSList	*slist, *iter;
 	guint	num = 0;
 
 	if(size != NULL)
-		*size = 0;
+		*size = 0U;
 	for(iter = slist = dp_get_selection(dp); iter != NULL; iter = g_slist_next(iter), num++)
 	{
 		if(size)
@@ -103,7 +106,7 @@ static int count_recursive_selected(MainInfo *min, guint64 *size)
 
 static gint evt_delete(GtkWidget *wid, GdkEvent *evt, gpointer user)
 {
-	*((gboolean *) user) = TRUE;
+	*(gboolean *) user = TRUE;
 	errno = EINTR;
 	return TRUE;
 }
@@ -111,7 +114,7 @@ static gint evt_delete(GtkWidget *wid, GdkEvent *evt, gpointer user)
 /* 1998-09-30 -	A callback for the big "Cancel" button. */
 static gint evt_cancel_clicked(GtkWidget *wid, gpointer user)
 {
-	*((gboolean *) user) = TRUE;
+	*(gboolean *) user = TRUE;
 	errno = EINTR;
 	return TRUE;
 }
@@ -120,10 +123,9 @@ static gint evt_keypress(GtkWidget *wid, GdkEventKey *evt, gpointer user)
 {
 	if(evt->keyval == GDK_Escape)
 	{
-		*((gboolean *) user) = TRUE;
+		*(gboolean *) user = TRUE;
 		errno = EINTR;
 	}
-
 	return TRUE;
 }
 
@@ -133,6 +135,8 @@ void pgs_progress_begin(MainInfo *min, const gchar *op_name, guint32 flags)
 	gchar		size_buf[32], buf[128];
 	GtkWidget	*cancel, *hbox, *label;
 
+	pgs_info.min = min;
+
 	if(pgs_info.level == 0)
 	{
 		if(flags & PFLG_BUSY_MODE)
@@ -143,8 +147,9 @@ void pgs_progress_begin(MainInfo *min, const gchar *op_name, guint32 flags)
 		pgs_info.show_byte  = (flags & PFLG_BYTE_VISIBLE) ? TRUE : FALSE;
 		pgs_info.show_item  = (flags & PFLG_ITEM_VISIBLE) ? TRUE : FALSE;
 		pgs_info.delayed    = TRUE;
-		pgs_info.delay_time = 250000;
+		pgs_info.delay_time = 250000U;
 		pgs_info.byte_pos   = 0;
+		pgs_info.last_secs  = 0U;
 
 		if(flags & PFLG_COUNT_RECURSIVE)
 			pgs_info.tot_num = count_recursive_selected(min, &pgs_info.byte_tot);
@@ -153,12 +158,12 @@ void pgs_progress_begin(MainInfo *min, const gchar *op_name, guint32 flags)
 
 		pgs_info.tot_pos = 0;
 
-		pgs_info.dlg = gtk_dialog_new();
+		pgs_info.dlg  = gtk_dialog_new();
+		gtk_widget_set_usize(pgs_info.dlg, 384, -1);
 		pgs_info.body = gtk_label_new(op_name);
 		gtk_signal_connect(GTK_OBJECT(pgs_info.dlg), "delete_event", GTK_SIGNAL_FUNC(evt_delete), &pgs_info.cancel);
 		gtk_signal_connect(GTK_OBJECT(pgs_info.dlg), "key_press_event", GTK_SIGNAL_FUNC(evt_keypress), &pgs_info.cancel);
 		gtk_box_pack_start(GTK_BOX(GTK_DIALOG(pgs_info.dlg)->vbox), pgs_info.body, FALSE, FALSE, 0);
-		gtk_widget_show(pgs_info.body);
 
 		hbox = gtk_hbox_new(FALSE, 0);
 		pgs_info.tot_pgs = gtk_progress_bar_new();
@@ -173,54 +178,49 @@ void pgs_progress_begin(MainInfo *min, const gchar *op_name, guint32 flags)
 		{
 			label = gtk_label_new("0");
 			gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);
-			gtk_widget_show(label);
 			gtk_box_pack_start(GTK_BOX(hbox), pgs_info.tot_pgs, TRUE, TRUE, 0);
 			g_snprintf(buf, sizeof buf, "%d", pgs_info.tot_num);
 			label = gtk_label_new(buf);
 			gtk_box_pack_end(GTK_BOX(hbox), label, FALSE, FALSE, 5);
-			gtk_widget_show(label);
 		}
 		gtk_progress_bar_update(GTK_PROGRESS_BAR(pgs_info.tot_pgs), 0.0f);
-		gtk_widget_show(pgs_info.tot_pgs);
 		gtk_box_pack_start(GTK_BOX(GTK_DIALOG(pgs_info.dlg)->vbox), hbox, FALSE, FALSE, 0);
-		gtk_widget_show(hbox);
 
 		if(pgs_info.show_byte)
 		{
-			sze_put_size(pgs_info.byte_tot, size_buf, sizeof size_buf, SZE_AUTO);
+			sze_put_size64(size_buf, sizeof size_buf, pgs_info.byte_tot, SZE_AUTO);
 			g_snprintf(buf, sizeof buf, _("Total (%s)"), size_buf);
 			label = gtk_label_new(buf);
 			gtk_box_pack_start(GTK_BOX(GTK_DIALOG(pgs_info.dlg)->vbox), label, FALSE, FALSE, 0);
-			gtk_widget_show(label);
 			pgs_info.byte_pgs = gtk_progress_bar_new();
 			gtk_box_pack_start(GTK_BOX(GTK_DIALOG(pgs_info.dlg)->vbox), pgs_info.byte_pgs, TRUE, TRUE, 0);
-			gtk_widget_show(pgs_info.byte_pgs);
 		}
-
 		if(pgs_info.show_item)
 		{
 			pgs_info.item = gtk_label_new("");
 			gtk_misc_set_alignment(GTK_MISC(pgs_info.item), 0.0f, 0.5f);
 			gtk_box_pack_start(GTK_BOX(GTK_DIALOG(pgs_info.dlg)->vbox), pgs_info.item, FALSE, FALSE, 5);
-			gtk_widget_show(pgs_info.item);
 			hbox = gtk_hbox_new(FALSE, 0);
 			pgs_info.item_pgs = gtk_progress_bar_new();
 			gtk_box_pack_start(GTK_BOX(hbox), pgs_info.item_pgs, TRUE, TRUE, 0);
 			gtk_progress_bar_update(GTK_PROGRESS_BAR(pgs_info.item_pgs), 0.0f);
-			gtk_widget_show(pgs_info.item_pgs);
 			gtk_box_pack_start(GTK_BOX(GTK_DIALOG(pgs_info.dlg)->vbox), hbox, FALSE, FALSE, 0);
-			gtk_widget_show(hbox);
 		}
-
+		if(!(flags & PFLG_BUSY_MODE))
+		{
+			pgs_info.eta = gtk_label_new("");
+			gtk_box_pack_start(GTK_BOX(GTK_DIALOG(pgs_info.dlg)->vbox), pgs_info.eta, FALSE, FALSE, 0);
+		}
+		else
+			pgs_info.eta = NULL;
 		cancel = gtk_button_new_with_label(_("Cancel"));
 		gtk_signal_connect(GTK_OBJECT(cancel), "clicked", GTK_SIGNAL_FUNC(evt_cancel_clicked), &pgs_info.cancel);
 		gtk_box_pack_start(GTK_BOX(GTK_DIALOG(pgs_info.dlg)->action_area), cancel, TRUE, TRUE, 0);
-		gtk_widget_show(cancel);
 
-		gettimeofday(&pgs_info.time_begin, NULL);
+		g_get_current_time(&pgs_info.time_begin);
 	}
 	else
-		fprintf(stderr, "**PROGRESS: No nesting, please\n");
+		g_warning("pgs_progress_begin() doesn't nest");
 	pgs_info.level++;
 }
 
@@ -238,10 +238,8 @@ void pgs_progress_end(MainInfo *min)
 /* ----------------------------------------------------------------------------------------- */
 
 /* 1998-09-30 -	Begin on a new "item"; typically a file, being <size> bytes. */
-void pgs_progress_item_begin(MainInfo *min, const gchar *name, gsize size)
+void pgs_progress_item_begin(MainInfo *min, const gchar *name, off_t size)
 {
-	gchar	size_buf[32], buf[FILENAME_MAX + 32];
-
 	if(pgs_info.flags & PFLG_BUSY_MODE)
 	{
 		GtkAdjustment	*adj;
@@ -254,22 +252,20 @@ void pgs_progress_item_begin(MainInfo *min, const gchar *name, gsize size)
 			val = adj->value + 1;
 			if(val > adj->upper)
 				val = adj->lower;
-			gtk_progress_set_value(GTK_PROGRESS(pgs_info.tot_pgs), val);
+			gtk_progress_bar_update(GTK_PROGRESS_BAR(pgs_info.tot_pgs), val);
 			scale = 10;
 		}
 	}
 	else
-	{
 		pgs_info.tot_pos++;
-		pgs_info.byte_pos += size;
-		gtk_progress_bar_update(GTK_PROGRESS_BAR(pgs_info.tot_pgs), (float) pgs_info.tot_pos / pgs_info.tot_num);
-	}
 
 	pgs_info.item_size = size;
 
 	if(pgs_info.show_item)
 	{
-		sze_put_size(size, size_buf, sizeof size_buf, SZE_AUTO);
+		gchar	size_buf[32], buf[FILENAME_MAX + 32];
+
+		sze_put_size64(size_buf, sizeof size_buf, size, SZE_AUTO);
 		g_snprintf(buf, sizeof buf, "%s (%s)", name, size_buf);
 		gtk_label_set_text(GTK_LABEL(pgs_info.item), buf);
 	}
@@ -280,29 +276,52 @@ void pgs_progress_item_begin(MainInfo *min, const gchar *name, gsize size)
 /* 1998-09-30 -	Indicate progress operating on the most recently registered item. Our
 **		current position in the item is <pos> bytes.
 */
-PgsRes pgs_progress_item_update(MainInfo *min, guint32 pos)
+PgsRes pgs_progress_item_update(MainInfo *min, off_t pos)
 {
-	gfloat		p = (float) pos / (float) pgs_info.item_size;
-	struct timeval	time_now;
-	gint		micro;
+	GTimeVal	time_now;
 
 	if(pgs_info.dlg != NULL)
 	{
+		g_get_current_time(&time_now);
 		if(pgs_info.delayed)
 		{
-			gettimeofday(&time_now, NULL);
-			micro = 1E6 * (time_now.tv_sec - pgs_info.time_begin.tv_sec) + 
+			gint	micro;
+
+			micro = 1E6 * (time_now.tv_sec  - pgs_info.time_begin.tv_sec) + 
 				      (time_now.tv_usec - pgs_info.time_begin.tv_usec);
 			if(micro >= pgs_info.delay_time)
-				pgs_info.delayed = 0;
+				pgs_info.delayed = FALSE;
 		}
 		if(!pgs_info.delayed)
-			gtk_widget_show(pgs_info.dlg);
-		if(pgs_info.show_item && pgs_info.item_size)
 		{
-			p = (gfloat) pos / (float) pgs_info.item_size;
-			gtk_progress_bar_update(GTK_PROGRESS_BAR(pgs_info.item_pgs), p);
+			if(pgs_info.eta != NULL)
+			{
+				gfloat	secs = (time_now.tv_sec - pgs_info.time_begin.tv_sec) + 
+						1E-6 * (time_now.tv_usec - pgs_info.time_begin.tv_usec);
+				if((guint) secs != pgs_info.last_secs)
+				{
+					gchar	buf[64], spdbuf[16];
+					gfloat	spd;
+					guint	eta;
+
+					pgs_info.last_secs = secs;
+					spd = (pgs_info.byte_pos + pos) / secs;
+					eta = (pgs_info.byte_tot - (pgs_info.byte_pos + pos)) / spd;
+					sze_put_size(spdbuf, sizeof spdbuf, spd, SZE_AUTO);
+					g_snprintf(buf, sizeof buf, _("Elapsed %02d:%02d  Speed %s/s  ETA %02u:%02u"),
+						pgs_info.last_secs / 60, pgs_info.last_secs % 60, spdbuf,
+						eta / 60, eta % 60);
+					gtk_label_set_text(GTK_LABEL(pgs_info.eta), buf);
+				}
+			}
+			gtk_widget_realize(pgs_info.dlg);
+			gdk_window_set_group(pgs_info.dlg->window, pgs_info.min->gui->window->window);
+			gtk_widget_show_all(pgs_info.dlg);
 		}
+		if(pgs_info.show_byte && pgs_info.byte_tot)
+			gtk_progress_bar_update(GTK_PROGRESS_BAR(pgs_info.byte_pgs), (gfloat) (pgs_info.byte_pos + pos) / pgs_info.byte_tot);
+		if(pgs_info.show_item && pgs_info.item_size)
+			gtk_progress_bar_update(GTK_PROGRESS_BAR(pgs_info.item_pgs), (gfloat) pos / pgs_info.item_size);
 		while(gtk_events_pending())
 			gtk_main_iteration();
 		if(pgs_info.cancel)
@@ -316,8 +335,8 @@ void pgs_progress_item_end(MainInfo *min)
 {
 	gint	old_errno = errno;
 
-	if(pgs_info.show_byte && pgs_info.byte_tot)
-		gtk_progress_bar_update(GTK_PROGRESS_BAR(pgs_info.byte_pgs), 1.0);
+	pgs_info.byte_pos += pgs_info.item_size;
+	gtk_progress_bar_update(GTK_PROGRESS_BAR(pgs_info.tot_pgs), (gfloat) pgs_info.tot_pos / pgs_info.tot_num);
 	while(gtk_events_pending())
 		gtk_main_iteration();
 
