@@ -40,10 +40,8 @@ Style * sdl_dialog_sync_new_wait(StyleInfo *si, Style *ignore)
 	tree = GTK_WIDGET(stl_styleinfo_build_partial(si, ignore));
 	gtk_signal_connect(GTK_OBJECT(tree), "selection_changed", GTK_SIGNAL_FUNC(evt_selection_changed), &sel);
 	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scwin), tree);
-	gtk_widget_show(tree);
-	gtk_widget_show(scwin);
 	gtk_widget_set_usize(scwin, 200, 384);
-	dlg = dlg_dialog_sync_new(scwin, _("Select Style"), _("OK|Cancel"));
+	dlg = dlg_dialog_sync_new(scwin, _("Select Style"), NULL);
 	if(dlg_dialog_sync_wait(dlg) != DLG_POSITIVE)
 		sel = NULL;
 	dlg_dialog_sync_destroy(dlg);
