@@ -66,6 +66,12 @@ struct widget_find_data {
 	GtkWidget	*widget;
 };
 
+struct property_rename_data {
+	Style		*style;
+	gchar		name_old[STL_PROPERTY_NAME_SIZE],
+			name_new[STL_PROPERTY_NAME_SIZE];
+};
+
 /* ----------------------------------------------------------------------------------------- */
 
 /* 1999-05-20 -	Create a new, empty, styleinfo. */
@@ -987,7 +993,7 @@ static GSList * style_get_branch(Style *stl)
 **		to sort on styles. Comparison is done so that styles with common parents
 **		are grouped.
 */
-gint stl_style_compare_hierarchy(Style *stla, Style *stlb)
+gint stl_style_compare_hierarchy(const Style *stla, const Style *stlb)
 {
 	GSList	*la, *lb;
 
@@ -1292,23 +1298,45 @@ gboolean stl_style_property_is_unique(Style *stl, const gchar *property)
 	return FALSE;
 }
 
+/* 2004-02-24 -	Traverse and propagate property renaming downwards in tree. Recursive. */
+static void traverse_property_rename(GNode *node, gpointer user)
+{
+	struct property_rename_data	*data = user;
+
+	stl_style_property_rename(node->data, data->name_old, data->name_new);
+}
+
 /* 1999-05-25 -	Rename <property> to <new_name>. If there is already a property with that name
 **		in the given <stl>, rename will fail and return FALSE. Else, TRUE is returned.
+** 2004-02-24 -	This was broken. It must recurse all child Styles, and let the renaming take
+**		effect in (all) child styles, or there will be "orphan" properties left about.
 */
 gboolean stl_style_property_rename(Style *stl, const gchar *property, const gchar *new_name)
 {
 	if((stl != NULL) && (property != NULL) && (new_name != NULL))
 	{
-		StlProp	*pr;
-
 		if(g_hash_table_lookup(stl->properties, new_name) == NULL)
 		{
+			StlProp	*pr;
+
 			if((pr = g_hash_table_lookup(stl->properties, property)) != NULL)
 			{
+				GNode	*me;
+
+				/* Remove from hash table, rename, and re-insert. */
 				g_hash_table_remove(stl->properties, pr->name);
 				stu_strncpy(pr->name, new_name, sizeof pr->name);
 				g_hash_table_insert(stl->properties, pr->name, pr);
-				styleinfo_update(stl->styleinfo);
+				/* Name changed. Now traverse children, and do the same there. Yucky. */
+				if((me = g_node_find(stl->styleinfo->styles, G_PRE_ORDER, G_TRAVERSE_ALL, stl)) != NULL)
+				{
+					struct property_rename_data	data;
+
+					data.style = stl;
+					g_snprintf(data.name_old, sizeof data.name_old, "%s", property);
+					g_snprintf(data.name_new, sizeof data.name_new, "%s", new_name);
+					g_node_children_foreach(me, G_TRAVERSE_ALL, traverse_property_rename, &data);
+				}
 				return TRUE;
 			}
 		}
