@@ -76,7 +76,7 @@ void		stl_style_set_name_widget(GtkWidget *wid, const gchar *name);
 const gchar *	stl_style_get_name(Style *stl);
 GString *	stl_style_get_name_full(Style *stl);
 
-gint		stl_style_compare_hierarchy(Style *stla, Style *stlb);
+gint		stl_style_compare_hierarchy(const Style *stla, const Style *stlb);
 
 /* Setting a non-existant property adds it to the style. Setting a property with
 ** the wrong kind of value will not do anything, but is stupid, so don't do that.
