@@ -116,9 +116,20 @@ static gint evt_delete(GtkWidget *wid, GdkEvent *evt, gpointer user)
 */
 static gint evt_keypress(GtkWidget *wid, GdkEventKey *event, gpointer user)
 {
+	TxvInfo	*txi = gtk_object_get_user_data(GTK_OBJECT(wid));
+
 	if(event->keyval == GDK_Escape)
 		really_destroy(wid);
-
+	else if(event->keyval == GDK_Home)
+	{
+		GtkAdjustment	*adj = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(txi->scwin));
+		gtk_adjustment_set_value(adj, 0.0f);
+	}
+	else if(event->keyval == GDK_End)
+	{
+		GtkAdjustment	*adj = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(txi->scwin));
+		gtk_adjustment_set_value(adj, adj->upper);
+	}
 	return FALSE;
 }
 
@@ -634,6 +645,7 @@ GtkWidget * txv_open(MainInfo *min, const gchar *label)
 	gtk_widget_show_all(vbox);
 
 	gtk_widget_grab_focus(txi->text);
+
 	return txi->win;
 }
 
