@@ -142,7 +142,7 @@ void typ_type_destroy(FType *type)
 **		in <type>. Returns 1 if so, otherwise 0.
 ** 1999-03-14 -	Made the access to the DirRow (previously DirLine) a lot more abstract.
 */
-static gint check_perm(FType *type, DirRow *row)
+static gint check_perm(const FType *type, DirRow *row)
 {
 	gint	pr = 1, pw = 1, px = 1;
 	mode_t	mode = 0;
