@@ -7,6 +7,7 @@
 #include "gentoo.h"
 
 #include <ctype.h>
+#include <stdlib.h>
 
 #include "window.h"
 
@@ -58,7 +59,7 @@ WinInfo * win_wininfo_new_default(MainInfo *min)
 
 	wi = win_wininfo_new(min);
 
-	win_window_new(wi, WIN_MAIN,		WIN_TYPE_SIMPLE_TOPLEVEL, FALSE, "gentoo v" VERSION, "main");
+	win_window_new(wi, WIN_MAIN,		WIN_TYPE_SIMPLE_TOPLEVEL, FALSE, geteuid() == 0 ? "gentoo [root]" : "gentoo", "main");
 	win_window_pos_grab_set(wi, WIN_MAIN, TRUE);
 	win_window_size_set(wi, WIN_MAIN, 764, 932);
 	win_window_size_use_set(wi, WIN_MAIN, TRUE);
@@ -155,7 +156,7 @@ static void evt_pos_grab_clicked(GtkWidget *wid, gpointer user)
 	gdk_window_get_position(min->gui->window->window, &wd->x, &wd->y);
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(gtk_object_get_data(GTK_OBJECT(wid), "spin1")), wd->x);
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(gtk_object_get_data(GTK_OBJECT(wid), "spin2")), wd->y);
-	*((gboolean *) gtk_object_get_data(GTK_OBJECT(wid), "modified")) = TRUE;
+	*(gboolean *) gtk_object_get_data(GTK_OBJECT(wid), "modified") = TRUE;
 }
 
 /* Grab size. Works only for main gentoo window. See comment for pos_grab above. */
@@ -168,7 +169,7 @@ static void evt_size_grab_clicked(GtkWidget *wid, gpointer user)
 	gdk_window_get_size(min->gui->window->window, &wd->width, &wd->height);
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(gtk_object_get_data(GTK_OBJECT(wid), "spin1")), wd->width);
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(gtk_object_get_data(GTK_OBJECT(wid), "spin2")), wd->height);
-	*((gboolean *) gtk_object_get_data(GTK_OBJECT(wid), "modified")) = TRUE;
+	*(gboolean *) gtk_object_get_data(GTK_OBJECT(wid), "modified") = TRUE;
 }
 
 static GtkWidget * subframe_build(const WinInfo *wi, WinDef *win, gint type, gboolean *modified)
@@ -549,6 +550,8 @@ GtkWidget * win_window_open(const WinInfo *wi, guint32 id)
 		gtk_widget_realize(wid);
 		if(wi->min->gui != NULL && wi->min->gui->window != NULL)
 			gdk_window_set_group(wid->window, wi->min->gui->window->window);
+		if(wd->title != NULL)
+			win_window_set_title(wid, wd->title);
 		return wid;
 	}
 	return NULL;
@@ -565,6 +568,37 @@ void win_window_relink(const WinInfo *wi, guint32 id, GtkWidget *window)
 		gtk_object_set_data(GTK_OBJECT(window), "windef", wd);
 }
 
+/* 2002-08-10 -	Set title of a window. Since it seems to break if locale is set, we simply
+**		un-set the locale temporarily. Hopefully, noone sets window titles in inner
+**		loops around here.
+** NOTE NOTE	This works for *all* windows, not just those created through this module.
+**		That might surprise, but it felt more sensical to stick it in here anyway.
+*/
+void win_window_set_title(GtkWidget *win, const gchar *title)
+{
+#if defined ENABLE_NLS
+	gchar		obuf[256];	/* Buffer, subsequent setlocale() call kills it otherwise. */
+	const gchar	*op;
+
+	/* Get current locale, before slamming "C" in there. */
+	if((op = setlocale(LC_ALL, NULL)) != NULL)
+	{
+		const gchar	*sl;
+
+		g_snprintf(obuf, sizeof obuf, "%s", op);
+		sl = setlocale(LC_ALL, "C");
+	}
+#endif
+	/* If ENABLE_NLS is undefined, this is all we do. */
+	gtk_widget_realize(win);
+	gtk_window_set_title(GTK_WINDOW(win), title);
+
+#if defined ENABLE_NLS
+	if(op != NULL)
+		setlocale(LC_ALL, obuf);
+#endif
+}
+
 /* 1999-12-23 -	Use this rather than gtk_widget_show() when you want to the window to appear, since
 **		this gives us a chance to set the window's size and position according to preference.
 */
