@@ -152,8 +152,7 @@ static void od_scrolled_box_init (ODScrolledBox *box) {
 
 /* new and destroy */
 GtkWidget * od_scrolled_box_new (void) {
-	GtkWidget *widget, *viewport, *scrollbar, *frame;
-	GtkScrolledWindow *scwin;
+	GtkWidget *widget, *viewport, *scrollbar;
   #if BORDERLESS
 	static GtkStyle *style = NULL;
   #endif
